

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//*********************************************************************
// args.h
//*********************************************************************

#ifndef ARGS_H
#define ARGS_H

#ifndef FARBOOK_H
#include "atom.h"
#include "common.h"
#include "strlist.h"
#endif

const short maxNrParms = 16;

_FBDEF(CArgs)

class FBC_USE CArgs: public CAtom {
   public:
      static FB_USE_(PCArgs) TheArgs;

#ifndef _SLANGER
      FB_USE CArgs(int _argc, Pchar _argv[]);
#ifndef FV_UNIX
      FB_USE CArgs(Pchar _lpCmdLine, Pchar _names = 0);
#endif
      FB_USE ~CArgs();
#endif // _SLANGER

      virtual FB_USE_(short) Ima(void);
      virtual FB_USE_(Pchar) IAm(void);
      virtual FB_USE_(bool)  Isa(const short _type);

#ifndef _SLANGER
      FB_USE_(bool) NewArgs(int _argc, Pchar _argv[]);
      FB_USE_(bool) NewArgs(Pchar _lpCmdLine, Pchar _names = 0);
#endif // _SLANGER

      FB_USE_(short) Number (void);

#ifndef _SLANGER
      FB_USE_(bool)  VArg   (Pchar _name, Pchar _value, ushort _size, Pchar _dflt = 0);
      FB_USE_(Pchar) Arg    (short _iArg, bool _alc);
#endif // _SLANGER
      // ***Warning: Reserved for farSlang use only***
      FB_USE_(Pchar) VArg   (Pchar _name, Pchar _dflt = 0);
      FB_USE_(Pchar) Arg    (short _iArg);          // farSlang version

      FB_USE_(bool)  FArg   (char  _flag);
      FB_USE_(long)  IArg   (void);

      FB_USE_(void) SetArg  (short _iArg, Pchar _val);
      FB_USE_(void) SetFArg (char  _flag);
      FB_USE_(void) ClrFArg (char  _flag);
      FB_USE_(void) SetIArg (long  _val);

      FB_USE_(long) GetParentSignature(void) {
         return parentSignature;
         }
#ifdef FV_UNIX
      bool GetExecPath(Pchar _execName, Pchar _fullPath, ushort _size);
#endif

protected:
#if defined(FB_MACINTOSH)
/*?
      void                receiveInfo (void);
      static pascal
      OSErr receiveDocs (const AppleEvent* _ae,
                               AppleEvent* _reply,
                               long        _refcon);
?*/
/*?
      static pascal
      OSErr receiveParms(const AppleEvent* _ae,
                               AppleEvent*       _reply,
                               long              _refcon);
      static
      bool  getParmsName(Pchar _name, short _size);

      static
      OSErr FSSpecToFullPath(FSSpec _fss, Pchar _path, short _size);
      static
      OSErr FSSpecToDirectoryPath(FSSpec _fss,
                                  Pchar  _path,
                                  short _size);
      static
      void  PStrCat              (StringPtr _dst, StringPtr _src);
      static
      void  PStrInsert           (StringPtr _dst, StringPtr _src);
?*/
#endif  // FB_MACINTOSH

      //---------------------------------------------------------------
      // Definition for class to store the values of named parameters
      //
      class aNamedParm {
         public:
            aNamedParm(void);
            virtual ~aNamedParm();

            void  SetName(Pchar _name);
            Pchar GetName(void);

            void  SetValue(Pchar _value);
            Pchar GetValue(void);

            bool Match(Pchar _name);

            bool AtEnd(void);

         protected:
            Pchar name;
            Pchar value;
         }; // aNamedParm

      static long        parentSignature;
      static long        myFlags;
      static long        myValue;
      static short       myArgC;
      static Pchar       myArgV[maxNrParms];
      static aNamedParm  names[maxNrParms];
      static Pchar       parms;
      static PCStrList   docList;

   private:
      CArgs(void) {};
   }; // CArgs

#endif // ARGS_H

