
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//*********************************************************************
// atom.cpp
//*********************************************************************
//
// Note: 99/12/31 pjm) had added a receiver member to CAtom so that any
// atom could receive a message. I might want to add this back in, but
// it was causing lots of problems compiling farWin so I am pulling it
// back out. The added code is commented out for now
//
// TBD: (00/8/8 pjm) whenever a CItem destructs, it needs to know if it
// is in a container, and it must be able to extricate itself. This is
// necessary to properly implement the AddRef() and Release() methods.
//
//*********************************************************************

#include "prefixb.h"
#include "atom.h"
#include "common.h"
//#include "heapwrap.h"
//?#include "syslog.h"

char typeNames[][iVars+1] = {
   {""},
   {"CArgs"},
   {"CAtom"},
   {"CManifest"}, // out of order but will deprecate farBooks if changed
   {"CBook"},
   {"CBookInfo"},
   {"CBlock"},
   {"CBlowfish"},
   {"boolean"},
   {"CSession"},
   {"CSessionDict"},
   {"chars"},
   {"CChunk"},
   {"CCloseSession"},
   {"CComm"},
   {"CCommCmmd"},
   {"CCommResult"},
   {"CConnection"},
   {"CConnectionDict"},
   {"CConsole"},
   {"CCriticalSection"},
   {"CListen"},
   {"CFarSlang"},
   {"CFile"},
   {"CFileList"},
   {"CFindItem"},
   {"CFindList"},
   {"CHash"},
   {"CHshList"},
   {"CIniList"},
   {"CItem"},
   {"CProcess"},
   {"CList"},
   {"long"},
   {"CMap"},
   {"COpenSession"},
   {"CPort"},
   {"CPreferences"},
   {"CPtrItem"},
   {"CRecord"},
   {"CReceiveMessage"},
   {"CRef"},
   {"CRefs"},
   {"CRifList"},
   {"CSearchFile"},
   {"CSemaphore"},
   {"CSendMessage"},
   {"CSocket"},
   {"CStrItem"},
   {"CStrList"},
   {"CString"},
   {"CThread"},
   {"CTree"},
   {"CObject"},
   {"ListBlock"},
   {"CDOMException"},
   {"CDOMImplementation"},
   {"CdomRelations"},
   {"CdomMethods"},
   {"CNode"},
   {"CNodeList"},
   {"CNamedNodeMap"},
   {"CElement"},
   {"CAttr"},
   {"CCharacterData"},
   {"CText"},
   {"CCDATASection"},
   {"CEntityReference"},
   {"CEntity"},
   {"CProcessingInstruction"},
   {"CComment"},
   {"CDocument"},
   {"CDocumentType"},
   {"CDocumentFragment"},
   {"CNotation"},
   {"COpenConnect"},
   {"CRegX"},
   {"CFSSAX"},
   {"CLocalBook"},
   {"CRemoteBook"},
   {"CFileItem"},
   {"CFileOps"},
   {"CSearch"},
   {"CRemoteFile"},
   {"CPoint"},
   {"CUrl"},
   {"CXPath"},
   {"CXPathItem"},
   {"CDate"},
   {"CTime"},
   {"CBugger"},
   {"CVar"},
   {"CVars"}
   };

long idSource = 0;
bool CAtom::debugLeaks = false;

//---------------------------------------------------------------------
//
Pchar CAtom::GetTypeName(short _iType) {
   if (_iType <= 0 || _iType > iTree)
      return typeNames[0];
   else
      return typeNames[_iType];
   } // GetTypeName

//---------------------------------------------------------------------
//
CAtom::CAtom(void):
   id    (0),
   err   (0),
   id2   (0),
   dirty (false),
   nrRefs(0) {

   cls = iAtom;            // add this to every CAtom-based constructor

   id  = idSource++;
   id2 = id;

   if (debugLeaks) {
      char msg[32];
      sprintf(msg, "%08ld new dflt", id);
      ::OutputDebugString(msg);
      }

#if defined(FV_THREADS)
#if defined(FV_MSWIN32)
   ::InitializeCriticalSection(&crtsec);
#endif
#endif
   } // CAtom

//---------------------------------------------------------------------
//
CAtom::CAtom(PCAtom _atom):
   id    (0),
   err   (FVE_OK),
   id2   (0),
   dirty (false),
   nrRefs(0) {

   cls = iAtom;

   id  = idSource++;
   id2 = id;

/*?
if (debugLeaks) {
      char msg[32];
      sprintf(msg, "%08d new copy", id);
      ::OutputDebugString(msg);
      }
?*/

#if defined(FV_THREADS)
#if defined(FV_MSWIN32)
   ::InitializeCriticalSection(&crtsec);
#endif
#endif
   } // CAtom

//---------------------------------------------------------------------
//
CAtom::~CAtom() {

/*?
   if (debugLeaks) {
      char msg[32];
      sprintf(msg, "%08d old %08d", id, id2);
      ::OutputDebugString(msg);
      }
?*/

#if defined(FV_THREADS)
#if defined(FV_MSWIN32)
   ::DeleteCriticalSection(&crtsec);
#endif
#endif

   // Check for double delete
   if (id2 != id)
      Assert(id2 == id);
   id2 = 0;
   } // ~CAtom

//---------------------------------------------------------------------
//
short CAtom::Ima(void) {
   return iAtom;
   } // Ima

//---------------------------------------------------------------------
//
Pchar CAtom::IAm(void) {
   return Pchar("Atom");
   } // IAm

//---------------------------------------------------------------------
//
bool CAtom::Isa(const short _type) {
   return (_type == iAtom);
   } // Isa

//---------------------------------------------------------------------
//
void CAtom::SetError(ulong _err) {
   err = _err;
   } // SetError

//---------------------------------------------------------------------
//
ulong CAtom::Error(void) {
   return err;
   } // Error

//---------------------------------------------------------------------
//
void CAtom::Enter(void) {
#if defined(FV_THREADS)
#if  defined(FV_MSWIN32)
   ::EnterCriticalSection(&crtsec);
#endif
#endif
   } // Enter

//---------------------------------------------------------------------
//
void CAtom::Leave(void) {
#if defined(FV_THREADS)
#if  defined(FV_MSWIN32)
   ::LeaveCriticalSection(&crtsec);
#endif
#endif
   } // Leave

//---------------------------------------------------------------------
//
bool CAtom::CallBack(ulong _value) {
   return false;
   } // CallBack

//---------------------------------------------------------------------
//??DMY
//CAtom CAtom::operator =(PCAtom _atom) {
//   dirty  = false;
//   nrRefs = 0;
//   return *this;
//   } // operator =
//
//---------------------------------------------------------------------
//
bool CAtom::BlockTo(PCAtom _block) {
   return false;
   } // BlockTo

//---------------------------------------------------------------------
//
bool CAtom::ToBlock(PCAtom _block) {
   return false;
   } // ToBlock

//---------------------------------------------------------------------
//
ulong CAtom::BookSize(void) {
   return 0;
   } // BookSize

//---------------------------------------------------------------------
//
bool CAtom::ReadFile(PCAtom _input, bool _isBook) {
//??DMY   _input;
   return false;
   } // ReadFile

//---------------------------------------------------------------------
//
bool CAtom::WriteFile(PCAtom _output, bool _isBook) {
//??DMY   _output;
   return false;
   } // WriteFile

//---------------------------------------------------------------------
//
bool CAtom::ReadRemote(PCAtom _input) {
   _input;

   err = FVE_FARERROR;

   return false;
   } // ReadRemote

//---------------------------------------------------------------------
//
bool CAtom::WriteRemote(PCAtom _output) {
   _output;

   err = FVE_FARERROR;

   return false;
   } // WriteRemote

//---------------------------------------------------------------------
//
bool CAtom::Load(Pchar _fileName) {
   _fileName;

   err = FVE_FARERROR;

   return false;
   } // Load

//---------------------------------------------------------------------

//
bool CAtom::Save(Pchar _fileName) {
   _fileName;

   err = FVE_FARERROR;

   return false;
   } // Save

