
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


//*********************************************************************
// block.h
//*********************************************************************

#ifndef BLOCK_H
#define BLOCK_H

#ifndef FARBOOK_H
#include "list.h"
#ifndef FV_NOPOINTS
#include "point.h"
#endif // FV_NOPOINTS
#endif

_FBDEF(CBlock)
class FBC_USE CBlock: public CItem {
   public:
      FB_USE CBlock         (void);              // default constructor
      FB_USE CBlock         (PCBlock _block); //copy constructor
      FB_USE CBlock         (const Pchar _buffer, ulong _size = 0);
#ifndef _SLANGER
#ifndef FV_UNIX	
      FB_USE CBlock         (const HGLOBAL _handle);
#endif // FV_UNIX
#endif // _SLANGER
      virtual FB_USE ~CBlock();

      virtual FB_USE_(short) Ima(void);
      virtual FB_USE_(Pchar) IAm(void);
      virtual FB_USE_(bool)  Isa(const short _type);

      FB_USE_(PCBlock) SubBlock(long _begin = 0, long _end = 0);

#ifndef _SLANGER
      FB_USE_(PCBlock) operator()(long _begin = 0, long _end = 0);
                                                     // return subblock
#endif // _SLANGER
      FB_USE_(bool)    Equals (PCBlock _block);

      FB_USE_(bool)    Alloc  (ulong _sz, ulong _mod = 0);
      FB_USE_(bool)    ReAlloc(ulong _sz, ulong _mod = 0);

#ifndef _SLANGER
      FB_USE_(HGLOBAL) Handle (void);
#endif // _SLANGER

      FB_USE_(bool)    Locked (void);

#ifndef _SLANGER
      FB_USE_(Pchar)   Lock   (void);
      FB_USE_(Pchar)   Pointer(void);
#endif // _SLANGER

      FB_USE_(bool)    Empty  (void);
      FB_USE_(ulong)   GetSize(void);
      virtual
      FB_USE_(ulong)   GetContentSize(void);

      FB_USE_(void)    Clear    (char _ch);

      FB_USE_(void)    Copy     (Pchar    _buffer, ulong _size);
      FB_USE_(void)    Copy     (Pchar    _string);
      FB_USE_(void)    Copy     (PCBlock _blk);

      // Use these three commands together
      FB_USE_(void)    Reset    (ulong _iBuff = 0);
      FB_USE_(char)    GetChar  (void);
      FB_USE_(void)    BackUp   (ulong i = 1);

      FB_USE_(bool)    NewBlock (PCBlock _blk);
      FB_USE_(bool)    NewBlock (const Pchar _buffer, ulong _size = 0);

      FB_USE_(void)    Create   (ulong _size = 0);
      FB_USE_(void)    Grow     (ulong _size = 0);
      FB_USE_(void)    Trim     (void);

      FB_USE_(void)    AddByte  (uchar    _byte);
      FB_USE_(void)    AddBuffer(Pchar    _buffer, ulong _size);
      FB_USE_(void)    AddBuffer(PCBlock _blk,    ulong _size);

      FB_USE_(void)    AddChar  (char     _ch);
      FB_USE_(void)    AddString(Pchar    _string);
      FB_USE_(void)    AddString(PCBlock _blk);

      FB_USE_(ulong)   NrChars  (void);
      FB_USE_(char)    GetChar  (ulong _index);
      FB_USE_(void)    PutChar  (ulong _index, char _value);

      // The following two methods assume that lines are separated by
      // line breaks, not zeros.
      FB_USE_(bool)    GetFirstLine(Pchar _line, ushort _size);
      FB_USE_(bool)    GetNextLine (Pchar _line, ushort _size);

      FB_USE_(ulong)   NrShorts (void);
      FB_USE_(short)   GetShort (ulong _index);
      FB_USE_(void)    PutShort (ulong _index, short _value);

      FB_USE_(ulong)   NrLongs  (void);
      FB_USE_(long)    GetLong  (ulong _index);
      FB_USE_(void)    PutLong  (ulong _index, long _value);

#ifndef FV_NOPOINTS
      FB_USE_(ulong)   NrPoints(void);
      FB_USE_(PCPoint) GetPoint(ulong _index);
      FB_USE_(long)    GetX    (ulong _index);
      FB_USE_(long)    GetY    (ulong _index);

      FB_USE_(void)    PutPoint(ulong _index, PCPoint _point);
      FB_USE_(void)    PutPoint(ulong _index, long _x, long _y);
      FB_USE_(void)    PutX    (ulong _index, long _x);
      FB_USE_(void)    PutY    (ulong _index, long _y);
#endif // FV_NOPOINTS

      FB_USE_(void)    PutBlock (Pvoid _block, ulong _size);

      FB_USE_(void)    Unlock   (void);
      FB_USE_(void)    Free     (void);

#ifndef _SLANGER
      FB_USE_(void)    Lose     (void);             // Use with caution
#endif // _SLANGER

      enum saveMode { saveAsBinary, saveAsText };

      FB_USE_(void) SetMode(saveMode _saveAs) { saveAs = _saveAs; }

      virtual FB_USE_(bool) BlockTo(PCAtom _block);
      virtual FB_USE_(bool) ToBlock(PCAtom _block);

      // io methods
      virtual FB_USE_(ulong) BookSize (void);

      virtual FB_USE_(bool)  ReadFile (PCAtom _input,  bool _isBook);
      virtual FB_USE_(bool)  WriteFile(PCAtom _output, bool _isBook);

      virtual FB_USE_(bool)  Load     (Pchar _fileName);
      virtual FB_USE_(bool)  Save     (Pchar _fileName);

   protected:
      HGLOBAL  h;                         // OS handle to memory buffer
      Pchar    p;                                // Pointer when locked
      Pchar    tp;                              // Pointer for GetLines
      short    ct;
      ulong    sz;                          // Size of allocated buffer
      long     pBuff;                   // Index of next available byte
      long     iBuff;                                     // Work index
      saveMode saveAs;                       // Save mode (binary/text)
      };  // CBlock

inline ulong CBlock::NrChars(void) {
   if (p)
      return pBuff;
   else
      return 0;
   } // NrChars

inline ulong CBlock::NrShorts(void) {
   if (p)
      return pBuff / 2;
   else
      return 0;
   } // NrShorts

inline ulong CBlock::NrLongs(void) {
   if (p)
      return pBuff / 4;
   else
      return 0;
   } // NrLongs

#ifndef FV_NOPOINTS
inline ulong CBlock::NrPoints(void) {
   if (p)
      return pBuff / 8;
   else
      return 0;
   } // NrPoints
#endif // FV_NOPOINTS

#endif // BLOCK_H

