
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


//*********************************************************************
// book.h
//*********************************************************************
//
// Base class for the farBook access system
//
//*********************************************************************

#ifndef BOOK_H
#define BOOK_H

#ifndef FARBOOK_H
#include "bookinfo.h"
#include "list.h"
#include "refs.h"
#endif

const short ContentPart  = 0;

// Mark/link types - see AMark below
const char anAlias     = 'A';               // an object alias mark/key
const char aDuplicate  = 'D';            // a de-dupped frame alias key
const char aHilite     = 'H';            // a highlighted text mark/key
const char aName       = 'N';                     // an object name key
const char aTarget     = 'A';                      // a target mark/key
const char aSearch     = 'S';                // stemmed word search key
const char isPrepped   = '$';                 // key is already prepped

// Alias types for nodes
const Pchar UniqueType  = Pchar("U");
const Pchar AliasType   = Pchar("A");
const Pchar ChildType   = Pchar("C");
const Pchar ParentType  = Pchar("P");
const Pchar LinkType    = Pchar("L");
const Pchar ILinkType   = Pchar("I");
const Pchar KeywordType = Pchar("K");
const Pchar SearchType  = Pchar("S");

const ulong rootDID    = 0x00000100;      // farBook root descriptor id

const long codeMark    = -1;

typedef ulong ADescNdx;                   // descriptor file index type
typedef ulong AViewNdx;                         // view file index type

enum ABookAction {noAction, readAction, writeAction, createAction};
enum AWaitKind   {dontWaitForLock, waitForLock};
enum APrepKind   {openInputFile, openOutputFile, closeFile};

typedef char AMark;

#ifndef _SLANGER

// Node descriptor format --------------------------------------------

__FBDEF(ALNdx)
struct ALNdx {
   ALNdx(void):
      where(0),
      size (0) {
      }
   ulong where;
   ulong size;
   }; // aLNdx

__FBDEF(aDesc)
class aDesc {
   public:
      static void  NrParts(short _nrParts);
      static short NrParts(void);

      aDesc(void);
      aDesc(const RaDesc _desc);
      virtual ~aDesc();

      void  Clear    (void);

      short Type     (short _iPart);
      void  Type     (short _iPart, short  _type);

      ulong Where    (short _iPart);
      void  Where    (short _iPart, ulong _where);

      ulong Size     (short _iPart);
      void  Size     (short _iPart, ulong _size);

      bool  ReadDesc (RCFile _file);
      bool  WriteDesc(RCFile _file);

   protected:
      static
      short  nrParts;
      pALNdx parts;
   }; // aDesc

#endif // _SLANGER

// farBook object -----------------------------------------------------

enum AnAction {toTerminate, toContinue};

_FBDEF(CBook)
class FBC_USE CBook: public CAtom {
   public:

      FB_USE CBook(Pchar _bookName);
      virtual FB_USE ~CBook();

      virtual FB_USE_(short) Ima(void);
      virtual FB_USE_(Pchar) IAm(void);
      virtual FB_USE_(bool)  Isa(const short _type);

      FB_USE_(Pchar) Name   (bool _alc);

      virtual FB_USE_(Pchar) GetFVL (void);

      virtual FB_USE_(bool)  Slow   (void);

      // Book parts methods
      FB_USE_(void)  NrParts(short _nrParts);
      FB_USE_(short) NrParts(void);

      // Book information methods
      FB_USE_(bool)  PutInfoField(Pchar _name, Pchar _value);

      // ***Warning: Reserved for farSlang use only***
      FB_USE_(Pchar) GetInfoField(Pchar _name);

      FB_USE_(bool)  GetInfoField(Pchar _name,
                                  Pchar _value,
                                  short _size);

      virtual FB_USE_(bool) UpdateInfo(void);

      virtual FB_USE_(bool) Create(long _nrParts,
                                   long _nrRefSlots,
                                   long _nrHshSlots,
                                   long _sizeKey,
                                   long _nrKeys);   // Create a farBook
      virtual FB_USE_(bool) Open  (void);
      virtual FB_USE_(bool) Close (void);

      // Lock and unlock an farBook region
      virtual FB_USE_(bool) Lock    (PCRef _ref, AWaitKind _wait);
      virtual FB_USE_(bool) Unlock  (PCRef _ref);

      // Methods to support serialization
      virtual FB_USE_(PCBookInfo) GetBookInfo(void);
      virtual FB_USE_(PCRefs) GetFile (void);
      virtual FB_USE_(bool)   GetDesc (ulong _loc, PCRecord _desc);

   // link i/o --------------------------------------------------------

      virtual FB_USE_(bool) MakeBookKey   (AMark   _mark,
                                           Pchar   _key,
                                           PCRef   _ref,
                                           bool    _trunc = false);
      virtual FB_USE_(bool) IfBookKey     (AMark   _mark,
                                           Pchar   _key,
                                           bool    _trunc = false);
      virtual FB_USE_(bool) FindBookKey   (AMark   _mark,
                                           Pchar   _key,
                                           PCList  _pFind,
                                           bool    _trunc = false);
      virtual FB_USE_(bool) DeleteBookKey (AMark   _mark,
                                           Pchar   _key,
                                           PCRef   _ref,
                                           bool    _trunc = false);
      virtual FB_USE_(bool) RenameNode    (Pchar   _oldName,
                                           Pchar   _newName,
                                           bool    _trunc = false);

      // Search engine interface
      virtual FB_USE_(bool) Search        (Pchar    _domain,
                                           Pchar    _expression,
                                           bool     _fulText,
                                           PCList   _pFind);

   // node i/o -------------------------------------------------------

      virtual FB_USE_(PCAtom) ReadPart    (PCRef _ref);
      virtual FB_USE_(bool)   WritePart   (PCRef _ref, PCAtom _part);
      virtual FB_USE_(bool)   CreateNode  (PCRef _ref, PCAtom _body);

      virtual FB_USE_(PCAtom) GetContent  (PCRef      _ref);

      virtual FB_USE_(bool)   HasTargets  (AMark       _mark,
                                           Pchar       _key,
                                           bool        _trunc = false);

   protected:
      void prepKey  (AMark    _mark,
                     Pchar    _key,
                     short    _size,
                     short    _szKey);
      ulong calcSize(ulong    _size);

      Pchar       bookName;                        // farBook file name
      PCBookInfo  bookInfo;               // farBook information record
      bool        rnly;                          // True when read-only
      bool        shared;                           // True when shared
      ABookAction action;
      aRef        iDesc;                   // current frame file offset
      aDesc       lDesc;                    // current frame descriptor
      PCRecord    sizes;                // list of excesses to allocate
      ulong       error;
   }; // CBook

#endif // BOOK_H

