
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//*********************************************************************
// bookinfo.cpp
//*********************************************************************

#include "prefixb.h"
#include "bookinfo.h"
#include "file.h"

//---------------------------------------------------------------------
//
CBookInfo::CBookInfo(ushort _hdrSize):
   CRecord(),
   hdrSize(_hdrSize) {
   } // CBookInfo

//---------------------------------------------------------------------
//
CBookInfo::~CBookInfo() {
   } // ~CBookInfo

//---------------------------------------------------------------------
//
RTTI_CPP(BookInfo, Record)

//---------------------------------------------------------------------
// Read the record from the beginning of the file
//
bool CBookInfo::ReadFile(PCAtom _input, bool _isBook) {
   TYPECAST(_input, File)->Seek(0, fromHead);
   return CRecord::ReadFile(_input, _isBook);
   } // ReadFile

//---------------------------------------------------------------------
// Write the record to the beginning of the file
//
bool CBookInfo::WriteFile(PCAtom _output, bool _isBook) {
   Assert(hdrSize);

   // Compute and check the size of the record
   ushort size = ushort(strlen(GetTemplate(false)) + strlen(GetDelimStr(false)) + 2);
   Assert(size <= hdrSize);

   // Write the record to the farBook
   TYPECAST(_output, File)->Seek(0, fromHead);
   if (!(CRecord::WriteFile(_output, _isBook)))
      return false;

   // Pad the header with spaces
   if (hdrSize > size) {
      char pad[sizeString];
      memset(pad, 0, sizeString);
      if (!(TYPECAST(_output, File)->WriteBlock(&pad, hdrSize - size)))
         return false;
      }

   return true;
   } // WriteFile

