
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//*********************************************************************
//
// Common.cpp
//
//*********************************************************************

//---------------------------------------------------------------------
//
//                      Miscellaneous routines
//
//---------------------------------------------------------------------

#include "prefixb.h"
#include "common.h"
#ifndef FV_NOLOG
#include "syslog.h"
#endif

#if defined(FV_UNIX)
void OutputDebugString(Pchar msg) { msg; };
#endif

//---------------------------------------------------------------------
// Assert receiver routine
//
Pvoid fvAssert(Pchar _exp, Pchar _file, int _line) {

#if defined(_DEBUG)

#if defined(_CATCH)
   char mssg[sizeString];
   sprintf(mssg, "Assert: %s %s at line %d", _exp, _file, _line);
#ifndef FV_NOLOG
   SysLog::Log('X', mssg);
#endif
   ::MessageBox(0, mssg, "farBook.dll", MB_OK);
   throw (mssg);

#else

#if   defined(FV_MSWINDOWS)
   char mssg[sizeString];
   sprintf(mssg, "Assert:\n%s\n%s at line %d", _exp, _file, _line);
   assert(strlen(mssg) < sizeString);
#ifndef FV_NOLOG
   SysLog::Log('X', mssg);
#endif
   ::MessageBox(0, mssg, "farBook.dll", MB_OK);
   assert(0);

#elif defined(FV_MACINTOSH)
   fflush (stdout);
   fprintf(stderr, "Assert:\n%s\n%s at line %d", _exp, _file, _line);
   fflush (stderr);

#elif defined(FV_UNIX)
   #warning FV_UNIX TBD (fvAssert)
#endif // FV_UNIX
   abort();

#endif // _CATCH   

#endif // _DEBUG
   return Pvoid(0);
   } // fvAssert

//---------------------------------------------------------------------
// Return -1 if _s not in _t, offset otherwise.
//
short StrPos(Pchar _s, Pchar _t, bool _ignCase) {
   if (!_t)
      return -1;
   short i = -1;

   // Case insensitive case
   if (_ignCase) {
      Pchar s = 0; STRINIT(s, _s);
      CapStr(s);
      Pchar t = 0; STRINIT(t, _t);
      CapStr(t);
      Pchar p = strstr(t, s);
      if (p)
         i = short(p - t);
      delete[] s;
      delete[] t;
      }

   // Case sensitive case
   else {
   Pchar p = strstr(_t, _s);
      if (p)
         i = short(p - _t);
      }
   return i;
   } // StrPos

//---------------------------------------------------------------------
// Return -1 if the specified word is not present in the specified
// string. Otherwise return the index of the match. The match is made
// only when the word is delimited. 
//
short WrdPos(Pchar _word, Pchar _t, bool _ignCase) {

   int len = strlen(_word);

   short n = StrPos(_word, _t, _ignCase);
   short i = n;
   while (n >= 0) {
      if ((i == 0         || IsDelim(_t[i-1])) &&
          (_t[i+len] == 0 || IsDelim(_t[i+len])))
         return i;
         
      n = StrPos(_word, _t+i+len, _ignCase);
      i = i + len + n;
      }

   return -1;
   } // WrdPos

//---------------------------------------------------------------------
// Return -1 if _ch not in _s, offset otherwise.
//
short ChrPos(char _ch, Pchar _s) {
   if (!_s)
      return -1;
   Pchar p = strchr(_s, _ch);
   if (!p)
      return -1;
   short i = short(p - _s);
   return i;
   } // ChrPos

//---------------------------------------------------------------------
// assign _ch to _s
// Note: assumes that _s is at least two bytes long
//
Pchar ChrCpy(Pchar _s, char _ch) {
   _s[0] = _ch;
   _s[1] = '\0';

   return _s;
   } // ChrCpy

//---------------------------------------------------------------------
// append _ch to _s
//
Pchar ChrCat(Pchar _s, char _ch, ushort _size) {
   char work[sizeName];
   work[0] = _ch;
   work[1] = 0;

if (strlen(_s) >= _size-1)
   Assert(strlen(_s) < _size-1);

   Strcat(_s, work, _size);

   return _s;
   } // ChrCat

//---------------------------------------------------------------------
// Insert _s into _t at _where. Note that this code will cause serious
// problems if _t is not big enough to insert _s.
//
Pchar StrNsrt(Pchar _s, Pchar _t, ushort _where, ushort _size) {
   int lenT = strlen(_t);
   int lenS = strlen(_s);

   Assert(lenS + lenT < _size);
   Assert(_where <= lenT);

   for (int i = lenT; i >= _where; i--)
      _t[i+lenS] = _t[i];
   strncpy(_t+_where, _s, lenS);

   return _t;
   } // StrNsrt

//---------------------------------------------------------------------
// Insert _ch into _t at _where. Note that this code will cause serious
// problems if _t is not big enough to insert _ch.
//
Pchar ChrNsrt(char _ch, Pchar _t, ushort _where, ushort _size) {
   short len = strlen(_t);

   Assert(len < _size-1);
   Assert(_where <= len);

   for (short i = len; i >= _where; i--)
      _t[i+1] = _t[i];
   _t[_where] = _ch;

   return _t;
   } // ChrNsrt

//---------------------------------------------------------------------
// Replace the old character with the new in the string. It is
// permissible to replace a character with the string terminator char,
// but that does stop the scan immediately.
//
Pchar ChrRep(char _old, char _new, Pchar _s) {
   Assert(_old);
   for (Pchar p = _s; *p; p++) {
      if (*p == _old)
         *p = _new;
      if (!*p)
         break;
      }

   return _s;
   } // ChrRep

//---------------------------------------------------------------------
// Capitalize the characters in _s
//
Pchar CapStr(Pchar _s) {
   Pchar p = _s;
   if (_s) while (*p) {
      *p = char(toupper(*p));
      p++;
      }

   return _s;
   } // CapStr

//---------------------------------------------------------------------
// Capitalize the character in _ch
//
char CapChr(char & _ch) {
   _ch = char(toupper(_ch));
   return _ch;
   } // CapChr

//---------------------------------------------------------------------
// Make lowercase the string pointed to by _s
//
Pchar LwrStr(Pchar _s) {
   Pchar p = _s;
   if (_s) while (*p) {
      *p = char(tolower(*p));
      p++;
      }

   return _s;
   } // LwrStr

//---------------------------------------------------------------------
// Capitalize the character in _ch
//
char LwrChr(char & _ch) {
   _ch = char(tolower(_ch));
   return _ch;
   } // LwrChr

//---------------------------------------------------------------------
// Return part of _s before _pat in _r. If _pat not in _s, return empty
// string. N.B., _r must be at least as large as _s.
//
Pchar Before(Pchar _pat, Pchar _s, Pchar _r, ushort _sz) {
   memset(_r, 0, _sz);
   short i = StrPos(_pat, _s);
   if (i >= 0)
      strncpy(_r, _s, Min(i, _sz-1));

   return _r;
   } // Before

//---------------------------------------------------------------------
// Return part of _s after _pat in _r. If _pat not in _s, return empty
// string. N.B., _r must be at least as large as _s.
Pchar After(Pchar _pat, Pchar _s, Pchar _r, ushort _sz) {
   memset(_r, 0, _sz);
   int i = StrPos(_pat, _s);
   if (i >= 0) {
      Pchar p = _s + i + strlen(_pat);
      int   s = strlen(_s) - i - strlen(_pat);
      if (s > 0)
         Strcpy(_r, p, _sz);
      }

   return _r;
   } // After

//---------------------------------------------------------------------
// Remove duplicate sequences of _ch from _s. Returns
// the resulting length of the string.
//
short Squeeze(Pchar _s, char _ch) {
   Pchar p = _s;
   while (*p)
      if (*p == _ch) {
         Pchar q = p;
         while (*p && *p == _ch)
            p++;
         strcpy(q+1, p);
         }
      else
         p++;
   return short(strlen(_s));
   } // Squeeze

//---------------------------------------------------------------------
// Remove duplicate white space from the specified string
//
short SqueezeWS(Pchar _s) {
   Pchar p = _s;
   while (*p)
      if (*p <= ' ') {
         *p = ' ';
         Pchar q = p+1;
         while (*p && *p <= ' ')
            p++;
         strcpy(q, p);
         p = q;
         }
      else
         p++;
   return short(strlen(_s));
   } // SqueezeWS

//---------------------------------------------------------------------
// Remove all spaces from _s.
//
short Pack(Pchar _s) {
   Pchar t;
   ushort sz = ushort(strlen(_s));
   if (sz == 0)
      return 0;
   for (t = _s+sz-1; t != _s ; t--)
      if (*t == ' ')
         strcpy(t, t+1);
   return short(strlen(_s));
   } // Pack

//---------------------------------------------------------------------
// Remove spaces (and other characters in [0x00..0x1f] from both ends of
// _s. Returns the resulting length of the string.
//
short Trim(Pchar _s, whichEnd _end) {
   if (_s == 0)
      return 0;
      
   Pchar t;
   ushort sz = ushort(strlen(_s));
   if (!sz)
      return 0;

   // trim front end
   if (_end == frontEnd || _end == bothEnds) {
      for (t = _s; *t && *t <= ' '; t++);
      sz  = ushort(strlen(t));
      if (t != _s)
         memmove(_s, t, sz+1);
      }

   if (!sz)
      return 0;

   // trim back end
   if (_end == bothEnds || _end == backEnd)
      for (t = _s+sz-1; t >= _s && *t && *t <= ' '; t--)
         *t = 0;

   return short(strlen(_s));
   } // Trim

//---------------------------------------------------------------------
// remove spaces and delimiters from ends of _s
//
short TrimDlm(Pchar _s, whichEnd _end) {
   Pchar t;
   if (strlen(_s) == 0)
      return 0;

   // trim front end
   if (_end == frontEnd || _end == bothEnds) {
      for (t = _s; *t && IsDelim(*t); t++);
      if (t != _s)
         strcpy(_s, t);
      }

   // trim back end
   if (_end == bothEnds || _end == backEnd)
      for (t = _s+strlen(_s)-1; (t != _s) && IsDelim(*t); t--)
         *t = '\0';

   return short(strlen(_s));
   } // TrimDlm

//---------------------------------------------------------------------
// Pad _s to the specified size
//
Pchar PadStr(Pchar _s, ushort _sz, ushort _size) {
   Pchar p = _s+strlen(_s);
   for (unsigned int i = strlen(_s); i < _sz && i < _size; i++, p++) {
      *p     = ' ';
      *(p+1) = 0;
      }

   return _s;
   } // PadStr

//---------------------------------------------------------------------
// Use to construct keys for a farBook keyword activity
//
void MakeKey(Pchar _outKey,
             Pchar _domain,
             Pchar _targetType,
             Pchar _inKey,
             ushort _size) {

   Strcpy(_outKey, _domain,     _size);
   ChrCat(_outKey, ':',         _size);
   Strcat(_outKey, _targetType, _size);
   ChrCat(_outKey, '.',         _size);
   Strcat(_outKey, _inKey,      _size);
   Pack  (_outKey);
   CapStr(_outKey);
   } // MakeKey

//---------------------------------------------------------------------
// Character map table for dense hash
//
bool initMap = true;
char mapTable[256];                                    // map key chars

//---------------------------------------------------------------------
// Initialize table to map key characters into dense number set
// Lifted from Microsoft's WinHelp hashing algorithm (Extracted
// from the code by Ron Burke)
//
void initMapTable(void) {
   char i;
   memset(mapTable, 0, 256);
   for (i = 'A'; i <= 'Z'; ++i)
      mapTable[i] = (i - 'A') + 0x11;
   for (i = 'a'; i <= 'z'; ++i)
      mapTable[i] = (i - 'a') + 0x11;
   for (i = '1'; i <= '9'; ++i)
      mapTable[i] = (i - '1') + 0x01;
   mapTable['0'] = 0x0A;
   mapTable['.'] = 0x0C;
   mapTable['_'] = 0x0D;
   } // initMapTable

//---------------------------------------------------------------------
//
char MapChar(char _ch) {
   if (initMap)
      initMapTable();
   initMap = false;

   return mapTable[_ch];
   } // MapChar

//---------------------------------------------------------------------
// Encode from ASCII characters to entity string. Return true if any
// encodes took place.
//
//    &lt;    less-than
//    &gt;    greater-than
//    &amp;   ampersand
//    &apos;  apostrophe
//    &quot;  quote
//
// Also handles characters that are in the range &#x00; - &#x01f; and
// &#x80; - &#xff;
//
bool EncodeXML(Pchar _s, ushort _sz) {
   Pchar work = new char[_sz];
   work[0]  = 0;
   Pchar w  = work;
   long sz  = _sz;
   bool enc = false;
   for (Pchar p = _s; *p; p++)
      if      (--sz == 0)
         Assert(0);
      else if (*p == '\r' ||
               *p == '\n' ||
               *p == '\t') {
         // Possible buffer overflow problem here
         *w = *p;
         w++;
         if (w-work >= _sz-1)
            Assert(0);
         *w = 0;
         }
      else if (*(unsigned char*)p > 0x080 ||
          *(unsigned char*)p < 0x020) {
         char temp[sizeName];
         unsigned int i = *(unsigned char*)p;
         sprintf(temp, "&#%d;", i);
         Strcat(work, temp, sizeString);
         w  += strlen(temp);
         enc = true;
         }
      else
         switch (*p) {
            case '<':
               Strcat(work, "&lt;",   sizeString);
               w  += 4;
               enc = true;
               break;
            case '>':
               Strcat(work, "&gt;",   sizeString);
               w  += 4;
               enc = true;
               break;
            case '&':
               Strcat(work, "&amp;",  sizeString);
               w  += 5;
               enc = true;
               break;
            case '\'':
               Strcat(work, "&apos;", sizeString);
               w  += 6;
               enc = true;
               break;
            case '"':
               Strcat(work, "&quot;", sizeString);
               w  += 6;
               enc = true;
               break;
            default:
               // Possible buffer overflow problem here
               *w = *p;
               w++;
               if (w-work >= _sz-1)
                  Assert(0);
               *w = 0;
            }

   Strcpy(_s, work, _sz);
   delete[] work;

   return enc;
   } // EncodeXML

//---------------------------------------------------------------------
// Decode from entity string to ASCII characters. Return true if any
// decodes took place. (what about an error?
//
//    &lt;    less-than
//    &gt;    greater-than
//    &amp;   ampersand
//    &apos;  apostrophe
//    &quot;  quote
//
// Also handles characters that are in the range &#0x00; - &#0x01f; and
// &#0x80; - &#0xff;
//
bool DecodeXML(Pchar _s, ushort _sz) {
   char work[sizeString];
   work[0]  = 0;
   ushort i = 0;
   bool dec = false;
   Pchar p  = _s;
   while (*p) {
      if (*p == '&')
         // Check for standard named entity
         if      (strncmp(p, "&lt;",   4) == 0) {
            work[i] = '<';
            p      += 4;
            dec     = true;
            }
         else if (strncmp(p, "&gt;",   4) == 0) {
            work[i] = '>';
            p      += 4;
            dec     = true;
            }
         else if (strncmp(p, "&amp;",  5) == 0) {
            work[i] = '&';
            p      += 5;
            dec     = true;
            }
         else if (strncmp(p, "&apos;", 6) == 0) {
            work[i] = '\'';
            p      += 6;
            dec     = true;
            }
         else if (strncmp(p, "&quot;", 6) == 0) {
            work[i] = '"';
            p      += 6;
            dec     = true;
            }
         else if (*(p+1) == '#') {
            char temp[sizeName];
            temp[0] = 0; assert(false);
            p += 2;
            while (*p && *p != ';') {
               ChrCat(temp, *p, sizeName);
               p++;
               }
            unsigned char ch = atoi(temp);
            work[i] = ch;
            dec     = true;
            }
         else {
            work[i] = *p;
            p++;
            }
      else {
         work[i] = *p;
         p++;
         }
      i++;
      Assert(i < sizeString);
      }

   Strcpy(_s, work, _sz)

   return dec;
   } // DecodeXML

//---------------------------------------------------------------------
// Determine if _s is a number. 0xn, +n, -n, as well as nL are accepted
//
#undef isdigit

bool IsNumber(Pchar _s) {
   char work[sizeString];
   Strcpy(work, _s, sizeString);
   Trim(work);
   CapStr(work);
   short len = (signed)strlen(work);
   if (work[0] == '0' && work[1] == 'X') {
      if (work[strlen(work)-1] == 'L')    // Drop optional trailing "L"
         work[strlen(work)-1] = 0;
      for (short i = 2; i < len; i++)
         if (isdigit(work[i]) == 0  &&
             work[i]          < 'A' &&
             work[i]          > 'F')
            return false;
      }

   else {
      short i = 0;
      if      (work[0] == '+')
         i++;
      else if (work[0] == '-')
         i++;
      else if (!isdigit(work[i]))
         return false;
      if (work[strlen(work)-1] == 'L')    // Drop optional trailing "L"
         work[strlen(work)-1] = 0;
      for (; i < len; i++)
         if (isdigit(work[i]) == 0)
            return false;
      }

   return true;
   } // IsNumber

//---------------------------------------------------------------------
// Return double value of numeric string
// (supports decimal only.)
//
double StrToDouble(Pchar _s) {
   return atof(_s);
   } // StrToDouble

//---------------------------------------------------------------------
// Return long value of numeric string. Supports decimal and
// hexadecimal only. Form is n[L].
//
long StrToLong(Pchar _s) {
   Pchar p;
   char work[sizeString];
   Strcpy(work, _s, sizeString);
   Trim(work);
   CapStr(work);
//?while(work && work[0] == '0')
//?   strcpy(work, work+1);
   if (work[strlen(work)-1] == 'L')       // Drop optional trailing "L"
      work[strlen(work)-1] = 0;
   if (work[0] == '-')
      return strtol (work, &p, 0);
//?else
//?   return strtoul(work, &p, 0);
   else if (work[0] == '0' && work[1] == 'X')
      return strtoul(work, &p, 16);
//?else if (work[0] == '0')
//?   return strtoul(work, &p,  8);
   else
      return strtoul(work, &p, 10);
   } // StrToLong

//---------------------------------------------------------------------
// Return integer value of numeric string.
// (supports decimal and hexadecimal only.)
//
short StrToShort(Pchar _s) {
   Pchar p;
   char work[sizeString];
   Strcpy(work, _s, sizeString);
   Trim  (work);
   CapStr(work);
   if (work[0] == '-')
      return short(strtol (work, &p, 0));
//?else
//?   return short(strtoul(work, &p, 0));
   else if (work[0] == '0' && work[1] == 'X')
      return short(strtoul(work, &p, 16));
//?else if (work[0] == '0')

//?   return short(strtoul(work, &p,  8));
   else
      return short(strtoul(work, &p, 10));
   } // StrToShort

//---------------------------------------------------------------------
//
#if   defined(FV_MSWINDOWS)
bool toSwap = false;

#elif defined(FV_MACINTOSH)
bool toSwap = true;

#elif defined(FV_UNIX)
bool toSwap = false;

#endif // FV_UNIX

//---------------------------------------------------------------------
//
void SetSwap(bool _toSwap) {
   toSwap = _toSwap;
   } // SetSwap

//---------------------------------------------------------------------
//  Swap bytes in specified short
//
short SwapShort(short _n) {
   if (toSwap)
      return (ushort(_n) >> 8) + (ushort(_n&0xff) << 8);
   else
      return _n;
   } // SwapShort

//---------------------------------------------------------------------
//  Swap bytes in specified long
//
long SwapLong(long _n) {
   if (toSwap)
      return ((ulong(_n) & ulong(0xff000000L)) >> 24) |
             ((ulong(_n) & ulong(0x00ff0000L)) >>  8) |
             ((ulong(_n) & ulong(0x0000ff00L)) <<  8) |
             ((ulong(_n) & ulong(0x000000ffL)) << 24);
   else
      return _n;
   } // SwapLong

//---------------------------------------------------------------------
// Shuffle a byte/character array
//
// Hint: use a nine digit number as the seed. Even 111111111 is good.
// If you don't need reproducibility, then set the seed parameter to
// zero, for which the routine will use random numbers as seed values.
//
// the following algorithm is from
//    http://en.wikipedia.org/wiki/Fisher-Yates_shuffle
//
// I'm not using it here, but maybe I should be. The bias issues are
// subtle .
//
// void shuffle(int cards[], int n)
//{
//    while (n > 1)
//    {
//        int k, tmp;
//        k = rand()%(n--);  // introduces modulo bias (see below)
//        tmp = cards[n];
//        cards[n] = cards[k];
//        cards[k] = tmp;
//    }
//}
//
//---------------------------------------------------------------------
void Shuffle (Pchar _w, int _size, long _seed) {
   long seed = 0;

   // Shuffle the string
   for (int i = 0; i < _size; ++i) {
      // Feed the seed
      if (seed == 0)
         seed = _seed != 0 ? _seed : rand();

      // Extract a table offset
      int j  = seed % _size;
      seed   = seed >> 1;

      // Swap the ith entry with the jth entry
      char w = _w[i];
      _w[i]  = _w[j];
      _w[j]  = w;
      }
   } // Shuffle

//---------------------------------------------------------------------
// Kill time by giving it to the system. Specify time in milliseconds.
//
void Pause(ulong _msecs) {
#if   defined(FV_MSWINDOWS)
   ::Sleep(_msecs);

#elif defined(FV_MACINTOSH)

#elif defined(FV_UNIX)
   if (_msecs > 1000)
      ::sleep(_msecs / 1000);                 // granularity is seconds
   else
      ::usleep(_msecs);                  // granularity is milliseconds
#endif
   } // Pause

//---------------------------------------------------------------------
// Return number of milliseconds since start
// (Mac resolution only 1/60th of a second)
//
ulong GetTicks(void) {
#if   defined(FV_MSWINDOWS)
   return ::GetTickCount();

#elif defined(FV_MACINTOSH)
   return (::TickCount() * 100L) / 6L;

#elif defined(FV_UNIX)
   #warning FV_UNIX TBD (GetTicks)
#endif // FV_UNIX
   } // GetTicks

//---------------------------------------------------------------------
// Return time in hh:mm:ss format
//
void GetTime(Pchar _time, short _size) {
   time_t t;
   time(&t);
   char time[sizeWord];
   Strcpy(time, ctime(&t), sizeWord);
   // _time = Mon Nov 21 11:31:54 1983\n\0
   Pchar p;
   strtok(time, " ");                                            // Mon
   strtok(NULL, " ");                                            // Nov
   strtok(NULL, " ");                                             // 21
   p = strtok(NULL, " ");                                   // 11:31:54
   if (strlen(p) < 8) {
      ChrCpy(_time, '0');
      Strcat(_time, p, _size);
      }
   else
      Strcpy(_time, p, _size);
   } // GetTime

//---------------------------------------------------------------------
// LongTime = 10000*HH + 100*MM + SS
//
long LongTime(Pchar _time) {
   char t[sizeLine];
   Strcpy(t, _time, sizeLine);

   long time;

   Pchar p;

   // Process hours
   p = strtok(t, ":");
   if (!p)
      return 0;
   if (*p == '0')
      p++;
   time = StrToLong(p);

   // Process minutes
   p = strtok(0, ":");
   if (!p)
      return 0;
   if (*p == '0')
      p++;
   time = 100 * time + StrToLong(p);

   // Process seconds
   p = strtok(0, ":");
   if (!p)
      return 0;
   time = 100 * time + StrToLong(p);

   return time;
   } // LongTime

enum DATEFORM {DF_MDY, DF_MD20Y, DF_YMD, DF_20YMD};

//---------------------------------------------------------------------
// Return date in Mon Nov 21 1983 format
//
void GetDate(Pchar _date, short _size) {
   time_t t;
   time(&t);
   char date[sizeWord];
   Strcpy(date, ctime(&t), sizeWord);

   // _time = Mon Nov 21 11:31:54 1983\n\0
   Pchar p;

   // get the month
   char month[sizeWord];
   strtok(date,  " ");                                           // Mon
   p = strtok(NULL,  " ");                                       // Nov
   Strcpy(month, p, sizeWord);

   // get the day
   char day[sizeWord];
   p = strtok(NULL,  " ");                                        // 21
   Strcpy(day, p, sizeWord);

   // get the year
   strtok(NULL,  " ");                                      // 11:31:54
   p = strtok(NULL,  " ");                                    // 1983\n

   // put the date together
   Strcpy(_date, month, _size);
   Strcat(_date, " ",   _size);
   Strcat(_date, day,   _size);
   Strcat(_date, ", ",  _size);
   Strcat(_date, p,     _size);
   _date[strlen(_date)-1] = '\0';                        // clip the \n
   } // GetDate

//---------------------------------------------------------------------
// LongDate = 10000*YYYY + 100*MM + DD
//
long LongDate(Pchar _date) {
   char d[sizeLine];
   Strcpy(d, _date, sizeLine);

   long date;

   Pchar p;

   // Process months
   p = strtok(d, "/");
   if (!p)
      return 0;
   if (*p == '0')
      p++;
   date = StrToLong(p);

   // Process days
   p = strtok(0, "/");
   if (!p)
      return 0;
   if (*p == '0')
      p++;
   date = 100 * date + StrToLong(p);

   // Process years
   p = strtok(0, "/");
   if (!p)
      return 0;
   date = date + 10000 * StrToLong(p);

   return date;
   } // LongDate

//---------------------------------------------------------------------
// Return HTTP-date according to the following specification:
// (Uses the frfc1123-date form, as in
//
//    Thu, 10 Feb 2000 18:32:12 GMT)
//
// HTTP-date    = rfc1123-date | rfc850-date | asctime-date
//    rfc1123-date = wkday "," SP date1 SP time SP "GMT"
//    rfc850-date  = weekday "," SP date2 SP time SP "GMT"
//    asctime-date = wkday SP date3 SP time SP 4DIGIT
//    date1        = 2DIGIT SP month SP 4DIGIT
//                      ; day month year (e.g., 02 Jun 1982)
//    date2        = 2DIGIT "-" month "-" 2DIGIT
//                      ; day-month-year (e.g., 02-Jun-82)
//    date3        = month SP ( 2DIGIT | ( SP 1DIGIT ))
//                      ; month day (e.g., Jun  2)
//    time         = 2DIGIT ":" 2DIGIT ":" 2DIGIT
//                      ; 00:00:00 - 23:59:59
//    wkday        = "Mon"       | "Tue"      |
//                   "Wed"       | "Thu"      |
//                   "Fri"       | "Sat"      | "Sun"
//    weekday      = "Monday"    | "Tuesday"  |
//                   "Wednesday" | "Thursday" |
//                   "Friday"    | "Saturday" | "Sunday"
//    month        = "Jan"       | "Feb"      |
//                   "Mar"       | "Apr"      |
//                   "May"       | "Jun"      |
//                   "Jul"       | "Aug"      |
//                   "Sep"       | "Oct"      |
//                   "Nov"       | "Dec"
//
// GetSystemTime returns date and time in the SYSTEMTIME struct,
// which has the following fields:
//
//    time.wYear 1995
//    time.wMonth 1-12
//    time.wDayOfWeek 0-6
//    time.wDay 1-31
//    time.wHour 1-12 or 0-23 depends on locale
//    time.wMinute 0-59
//    time.wSecond 0-59
//
const char dayOfWeek[] = "SunMonTueWedThuFriSat";
const char month[]     = "JanFebMarAprMayJunJulAugSepOctNovDec";

void GetGMT(Pchar _date, short _size) {
   Assert(_size > 29);

#if   defined(FV_MSWINDOWS)
   short i;
   char work[sizeName];

   // Get date and time in Greenwich Mean Time
   SYSTEMTIME time;
   ::GetSystemTime(&time);

   memset(_date, 0, _size);

   // Get the day of the week
   i = time.wDayOfWeek * 3;
   strncpy(_date, dayOfWeek+i, 3);

   // Get the day of the month
   sprintf(work, ", %02d ", time.wDay);
   Assert(strlen(work) < sizeName);
   Strcat(_date, work, _size);

   // Get the month
   i = (time.wMonth-1) * 3;
   strncat(_date, month+i, 3);

   // Get the year
   sprintf(work, " %d ", time.wYear);
   Assert(strlen(work) < sizeName);
   Strcat(_date, work, _size);

/* if it is a twelve-hour clock, how do we know whether it is AM or PM?
   // Get hour mode ("0" = 12hr, "1" = 24hr)
   char itime[sizeName];
   ::GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_ITIME, itime, sizeName);
   if (itime[0] == '0')
      time.wHour -= 1;
   if (pm???)
      time.wHour += 12;
*/
   // Get the hour:minute:second
   sprintf(work, "%02d:%02d:%02d GMT", time.wHour,
                                       time.wMinute,
                                       time.wSecond);
   Assert(strlen(work) < sizeName);
   Strcat(_date, work, _size);

#elif defined(FV_MACINTOSH)

#elif defined(FV_UNIX)
   #warning FV_UNIX TBD (GetGMT)
#endif // FV_UNIX
   } // GetGMT

//---------------------------------------------------------------------
// Create a UUID as a char-array - dropping extra delimiters
// (Borland requires #include <ole2.h> and \bc5\lib\ole2w32.lib)
// (MSVC++ requires ???)
//
Pchar GetID(Pchar _id, short _size) {
#if defined(FV_MSWINDOWS)
   Assert(_size >= 33);
   GUID guid;
   ::CoCreateGuid(&guid);
   wchar_t sguid[sizeLine];
   StringFromGUID2(guid, sguid, sizeLine);
   memset(_id, 0, _size);
   short j = 0;
   for (short i = 0; sguid[i]; i++)
      if (!IsDelim(sguid[i]))
         _id[j++] = sguid[i];
#elif defined(FV_MACINTOSH)

#elif defined(FV_UNIX)
   #warning FV_UNIX TBD (GetID)
#endif         

   return _id;
   } // GetID


