
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


//*********************************************************************
// common.h
//*********************************************************************

//---------------------------------------------------------------------
//
//            Miscellaneous constants, types, and routines
//
// Some stuff provided to aid portability
//
//---------------------------------------------------------------------

#ifndef COMMON_H
#define COMMON_H

#ifndef FARBOOK_H
#include "errs.h"
#endif

//---------------------------------------------------------------------
//
enum whichEnd  {frontEnd, bothEnds, backEnd};
enum SEARCH    {looking, missing, found};
enum direction {noDir, up, down,
                       top, bottom,
                       left, right,
                       center};

//---------------------------------------------------------------------
// Macro to return a bit inserted into a word.
//
#define bitSet(byte, bit, val) \
   (((byte) & ~(1 << (bit))) | \
    ((((val) & 1) << (bit))))

//---------------------------------------------------------------------
// Macro to return a right justified bit from a word.
//
#define bitGet(byte, bit) \
   (((byte) & (1 << (bit))) >> (bit))

//=====================================================================
// Some useful templates

//---------------------------------------------------------------------
// Return the smaller of _a and _b
//
#define Min(a, b) ((a <= b) ? a : b)

//---------------------------------------------------------------------
// Return the larger of _a and _b
//

#define Max(a, b) ((a >= b) ? a : b)

//---------------------------------------------------------------------
// Return true if _i between _lo and _hi
//
template <class t>
   inline t InBnd(const t _i, const t _lo, const t _hi) {
      return ((_i >= _lo && _i <= _hi) ? true : false);
      }

//---------------------------------------------------------------------
// Bound _i between _lo and _hi
//
template <class t>
   inline t Bound(const t _i, const t _lo, const t _hi) {
       return ((_i < _lo) ? _lo : ((_i > _hi) ? _hi : _i));
       }

//---------------------------------------------------------------------
// Wrap _i between _lo and _hi
//
template <class t>
   inline t Wrap(const t _i, const t _lo, const t _hi) {
      return ((_i < _lo) ? _hi : ((_i > _hi) ? _lo : _i));
      }

//---------------------------------------------------------------------
// Return true if _i is odd
//
template <class t>
   inline t Odd(const t _i) {
      return (_i & 1);
      }

//---------------------------------------------------------------------
// Return absolute value of _i
//
template <class t>
   inline t Abs(const t _i) {
      return (_i < 0 ? -_i : _i);
      }

//---------------------------------------------------------------------
// Return true if _s empty
//
inline bool StrEmpty(Pchar _s) {
   return (_s == 0 || _s[0] == 0);
   };

extern "C" {

#if defined(FV_UNIX)
void OutputDebugString(Pchar msg);
#endif

//=====================================================================
// Handy string functions
//
// with character arguments
//
FBF_USE_(short) ChrPos (char  _ch, Pchar _s);         // find _ch in _s
FBF_USE_(Pchar) ChrCpy (Pchar _s, char _ch);        // assign _ch to _s
FBF_USE_(Pchar) ChrCat (Pchar _s, char _ch, ushort _size);  // _s + _ch
FBF_USE_(Pchar) ChrNsrt(char _ch, Pchar _t, ushort _where, ushort _size);
                                        // insert _ch into _t at _where
FBF_USE_(char)  CapChr (char & _ch);    // capitalize the letter in _ch
FBF_USE_(char)  LwrChr (char & _ch);// make the letter in _ch lower case
FBF_USE_(Pchar) ChrRep (char _old, char _new, Pchar _s);
                // replace the old character with the new in the string

// with string arguments
//
FBF_USE_(short) StrPos (Pchar _s, Pchar _t, bool _ignCase = false);
                                                       // find _s in _t
FBF_USE_(short) WrdPos (Pchar _s, Pchar _t, bool _ignCase = false);
                                                       // find _s in _t
FBF_USE_(Pchar) StrNsrt(Pchar _s, Pchar _t, ushort _where, ushort _size);
                                         // insert _s into _t at _where
FBF_USE_(Pchar) CapStr (Pchar _s);      // capitalize the letters in _s
FBF_USE_(Pchar) LwrStr (Pchar _s);     // Make letters in _s lower case
FBF_USE_(Pchar) Before (Pchar _pat,
                        Pchar _s, Pchar _r, ushort _sz);
                                       // return part of _s before _pat
FBF_USE_(Pchar) After  (Pchar _pat,
                        Pchar _s, Pchar _r, ushort _sz);
                                        // return part of _s after _pat
FBF_USE_(short) Squeeze(Pchar _s, char _ch);
                                      // remove dup seqs of _ch from _s
FBF_USE_(short) SqueezeWS(Pchar _s);
                                        // Condense white space from _s
FBF_USE_(short) Pack   (Pchar _s);         // remove all spaces from _s
FBF_USE_(short) Trim   (Pchar _s, whichEnd end = bothEnds);
                                       // remove spaces from ends of _s
FBF_USE_(short) TrimDlm(Pchar _s, whichEnd end = bothEnds);
                            // remove spaces and delims from ends of _s
FBF_USE_(Pchar) PadStr (Pchar _s, ushort _sz, ushort _size);
                                                 // pad _s to speced sz

FBF_USE_(void)  MakeKey(Pchar _outKey,
                        Pchar _domain,
                        Pchar _targetType,
                        Pchar _inKey,
                        ushort _size);

//---------------------------------------------------------------------
// Character map table for dense hashing
//
FBF_USE_(char)  MapChar(char _ch);

//---------------------------------------------------------------------
// XML support routines
//
FBF_USE_(bool)  EncodeXML(Pchar _s, ushort _sz);
FBF_USE_(bool)  DecodeXML(Pchar _s, ushort _sz);

//---------------------------------------------------------------------
// Sundry property test services

//---------------------------------------------------------------------
// Return true if _ch is a delimiter
//
inline bool IsDelim(char _ch) {
   return (isalnum(_ch) ? false : true);
   } // IsDelim

//---------------------------------------------------------------------
//
FBF_USE_(bool)IsNumber(Pchar _s);                  // is _s is a number

//---------------------------------------------------------------------
//
FBF_USE_(double) StrToDouble(Pchar _s);
FBF_USE_(long)   StrToLong  (Pchar _s);
FBF_USE_(short)  StrToShort (Pchar _s);

//---------------------------------------------------------------------
// Swap services
//
FBF_USE_(void)  SetSwap  (bool  _toSwap);
FBF_USE_(short) SwapShort(short _n);
FBF_USE_(long)  SwapLong (long  _n);

//---------------------------------------------------------------------
// Random number stuff
//
FBF_USE_(int)  Random  (int _max);
FBF_USE_(void) Shuffle (Pchar _w, int _size, long _seed = 0);
                                                   // shuffle the array

//---------------------------------------------------------------------
// Date and time services
//
FBF_USE_(ulong) GetTicks(void);        // return time since app started

FBF_USE_(void)  GetTime (Pchar _time, short _size); // form is hh:mm:ss
FBF_USE_(long)  LongTime(Pchar _time);

FBF_USE_(void)  GetDate (Pchar _date, short _size); // form is mm/dd/yy
FBF_USE_(long)  LongDate(Pchar _date);

FBF_USE_(void)  GetGMT  (Pchar _date, short _size);      // HTTP format

FBF_USE_(void)  Pause   (ulong _msecs);

//---------------------------------------------------------------------
// Obtain a unique identifier from the system
//
FBF_USE_(Pchar) GetID(Pchar _id, short _size);

} // extern "C"

#endif // COMMON_H

