
//********************************************************************
// Errs.h
//*********************************************************************

#ifndef ERRS_H
#define ERRS_H

#include "prefixb.h"

//---------------------------------------------------------------------
// Error constants
//
// Note that you can determine within which error group a particular
// error is by first dividing by 100, then multiplying by 100.
//
typedef ulong ErrorMsg;

const ErrorMsg FVE_BOOKRR            =  2000;     // farBook error base
const ErrorMsg FVE_WINERR            =  4000;      // farWin error base
const ErrorMsg FVE_SLNERR            =  6000;    // farSlang error base

const ErrorMsg FVE_APPERROR          = 10000;         // App error base

const ErrorMsg FVE_OK                =     0;   // everything worked ok

// File errors
const ErrorMsg FVE_ENDOFFILE         = FVE_BOOKRR-   1;
const ErrorMsg FVE_FILEERROR         = FVE_BOOKRR;
const ErrorMsg FVE_UNEXPECTEDEND     = FVE_BOOKRR+   2;
const ErrorMsg FVE_FARERROR          = FVE_BOOKRR+   3;
const ErrorMsg FVE_NOFILENAME        = FVE_BOOKRR+   4;
const ErrorMsg FVE_NOFILE            = FVE_BOOKRR+   5;
const ErrorMsg FVE_NOPATH            = FVE_BOOKRR+   6;
const ErrorMsg FVE_NOHANDLE          = FVE_BOOKRR+   7;
const ErrorMsg FVE_NOACCESS          = FVE_BOOKRR+   8;
const ErrorMsg FVE_BADACCESS         = FVE_BOOKRR+   9;
const ErrorMsg FVE_BADHANDLE         = FVE_BOOKRR+  10;
const ErrorMsg FVE_NOMEMORY          = FVE_BOOKRR+  11;
const ErrorMsg FVE_BADENVIRONMENT    = FVE_BOOKRR+  12;
const ErrorMsg FVE_BADFORMAT         = FVE_BOOKRR+  13;
const ErrorMsg FVE_NOFILES           = FVE_BOOKRR+  14;
const ErrorMsg FVE_NOSPACE           = FVE_BOOKRR+  15;
const ErrorMsg FVE_RDONLY            = FVE_BOOKRR+  16;
const ErrorMsg FVE_BADBLOCK          = FVE_BOOKRR+  17;
const ErrorMsg FVE_UNKNOWNFILE       = FVE_BOOKRR+  18;
const ErrorMsg FVE_FILEISOPEN        = FVE_BOOKRR+  19;
const ErrorMsg FVE_BADOPEN           = FVE_BOOKRR+  20;
const ErrorMsg FVE_FILENOTOPEN       = FVE_BOOKRR+  21;
const ErrorMsg FVE_FILENOTINPUT      = FVE_BOOKRR+  22;
const ErrorMsg FVE_FILENOTOUTPUT     = FVE_BOOKRR+  23;
const ErrorMsg FVE_BADCLOSE          = FVE_BOOKRR+  24;
const ErrorMsg FVE_FLUSHFAILED       = FVE_BOOKRR+  25;
const ErrorMsg FVE_LOCKFAILED        = FVE_BOOKRR+  26;
const ErrorMsg FVE_UNLOCKFAILED      = FVE_BOOKRR+  27;
const ErrorMsg FVE_BADSEEK           = FVE_BOOKRR+  28;
const ErrorMsg FVE_BADWRITE          = FVE_BOOKRR+  29;
const ErrorMsg FVE_BADREAD           = FVE_BOOKRR+  30;
const ErrorMsg FVE_SHORTREAD         = FVE_BOOKRR+  31;
const ErrorMsg FVE_CANTREMOVEFILE    = FVE_BOOKRR+  32;
const ErrorMsg FVE_CANTRENAMEFILE    = FVE_BOOKRR+  33;

const ErrorMsg FVE_NOITEMTOWRITE     = FVE_BOOKRR+  34;
const ErrorMsg FVE_BADRIFFFORMAT     = FVE_BOOKRR+  35;

// Record errors
const ErrorMsg FVE_NODATA            = FVE_BOOKRR+ 100;

// Comm errors
const ErrorMsg FVE_INPROGRESS        = FVE_BOOKRR+ 200;
const ErrorMsg FVE_CMMDINVALID       = FVE_BOOKRR+ 201;
const ErrorMsg FVE_CMMDDATAINVALID   = FVE_BOOKRR+ 202;
const ErrorMsg FVE_CONNECTIONNOTFOUND= FVE_BOOKRR+ 203;
const ErrorMsg FVE_CONNECTIONNOTOPEN = FVE_BOOKRR+ 204;
const ErrorMsg FVE_SESSIONNOTFOUND   = FVE_BOOKRR+ 205;
const ErrorMsg FVE_SESSIONNOTOPEN    = FVE_BOOKRR+ 206;
const ErrorMsg FVE_SENDFAILED        = FVE_BOOKRR+ 207;
const ErrorMsg FVE_RECEIVEFAILED     = FVE_BOOKRR+ 208;
const ErrorMsg FVE_RECEIVEABORTED    = FVE_BOOKRR+ 209;
const ErrorMsg FVE_HEADERSINVALID    = FVE_BOOKRR+ 210;
const ErrorMsg FVE_SOCKETNOTOPEN     = FVE_BOOKRR+ 211;

// Local Book errors
const ErrorMsg FVE_BOOKERROR         = FVE_BOOKRR+ 300;
const ErrorMsg FVE_BADBOOK           = FVE_BOOKRR+ 301;
const ErrorMsg FVE_OLDBOOK           = FVE_BOOKRR+ 302;
const ErrorMsg FVE_NOBOOK            = FVE_BOOKRR+ 303;
const ErrorMsg FVE_NOFIND            = FVE_BOOKRR+ 304;
const ErrorMsg FVE_NOINSERT          = FVE_BOOKRR+ 305;
const ErrorMsg FVE_EMPTYKEY          = FVE_BOOKRR+ 306;
const ErrorMsg FVE_LONGKEY           = FVE_BOOKRR+ 307;
const ErrorMsg FVE_DUPKEY            = FVE_BOOKRR+ 308;
const ErrorMsg FVE_NODELETE          = FVE_BOOKRR+ 309;
const ErrorMsg FVE_BADREQUEST        = FVE_BOOKRR+ 310;
const ErrorMsg FVE_LOCKRANGEERR      = FVE_BOOKRR+ 311;
const ErrorMsg FVE_ENDFILE           = FVE_BOOKRR+ 312;

// Remote File/Book errors
const ErrorMsg FVE_REMOTEBOOKERROR   = FVE_BOOKRR+ 400;
const ErrorMsg FVE_LOGCONNECTFAILED  = FVE_BOOKRR+ 401;
const ErrorMsg FVE_OPENSESSIONFAILED = FVE_BOOKRR+ 402;
const ErrorMsg FVE_OPENCONNECTFAILED = FVE_BOOKRR+ 403;
const ErrorMsg FVE_RESPONSEFAILED    = FVE_BOOKRR+ 404;
const ErrorMsg FVE_SENDMSSGERR       = FVE_BOOKRR+ 405;
const ErrorMsg FVE_RESPONSEMSSGERR   = FVE_BOOKRR+ 406;
const ErrorMsg FVE_SEARCHNOTALLOWED  = FVE_BOOKRR+ 407;

// Regex errors
const ErrorMsg FVE_REGEXERROR        = FVE_BOOKRR+ 500;
const ErrorMsg FVE_NOMATCH           = FVE_BOOKRR+ 501;
const ErrorMsg FVE_NULL              = FVE_BOOKRR+ 502;
const ErrorMsg FVE_BADOPTION         = FVE_BOOKRR+ 503;
const ErrorMsg FVE_BADMAGIC          = FVE_BOOKRR+ 504;
const ErrorMsg FVE_UNKNOWNNODE       = FVE_BOOKRR+ 505;
const ErrorMsg FVE_NOSUBSTRING       = FVE_BOOKRR+ 506;

// Topic errors
const ErrorMsg FVE_NODEERROR         = FVE_BOOKRR+ 600;
const ErrorMsg FVE_BOOKOK            = FVE_BOOKRR+ 601;
const ErrorMsg FVE_NOLIST            = FVE_BOOKRR+ 602;
const ErrorMsg FVE_NOMANIFEST        = FVE_BOOKRR+ 603;
const ErrorMsg FVE_NOFRAME           = FVE_BOOKRR+ 604;
const ErrorMsg FVE_BADFRAME          = FVE_BOOKRR+ 605;
const ErrorMsg FVE_BADNAME           = FVE_BOOKRR+ 606;
const ErrorMsg FVE_DUPNAME           = FVE_BOOKRR+ 607;
const ErrorMsg FVE_CANTMAKENAME      = FVE_BOOKRR+ 608;
const ErrorMsg FVE_BADLISTNDX        = FVE_BOOKRR+ 609;
const ErrorMsg FVE_OUTOFRANGE        = FVE_BOOKRR+ 610;
const ErrorMsg FVE_DUPLOCK           = FVE_BOOKRR+ 611;
const ErrorMsg FVE_BADBODY           = FVE_BOOKRR+ 612;

// XML Document errors
const ErrorMsg FVE_NOTHINGTOPARSE    = FVE_BOOKRR+ 700;

// XPath evaluator errors
const ulong XPE_UNDERRUN             = FVE_BOOKRR+ 800;
const ulong XPE_NODEEXPECTED         = FVE_BOOKRR+ 801;
const ulong XPE_NODESETEXPECTED      = FVE_BOOKRR+ 802;
const ulong XPE_STRINGEXPECTED       = FVE_BOOKRR+ 803;
const ulong XPE_NUMBEREXPECTED       = FVE_BOOKRR+ 804;
const ulong XPE_BOOLEANEXPECTED      = FVE_BOOKRR+ 805;
const ulong XPE_OPNOTEXPECTED        = FVE_BOOKRR+ 806;
const ulong XPE_AXISNAMEUNKNOWN      = FVE_BOOKRR+ 807;
const ulong XPE_WRONGNRARGUMENTS     = FVE_BOOKRR+ 808;
const ulong XPE_PROCINSTEXPECTED     = FVE_BOOKRR+ 809;
const ulong XPE_STACKEMPTY           = FVE_BOOKRR+ 810;
const ulong XPE_STACKNOTEMPTY        = FVE_BOOKRR+ 811;
const ulong XPE_FUNCTIONUNKNOWN      = FVE_BOOKRR+ 812;
const ulong XPE_BADOPERANDTYPE       = FVE_BOOKRR+ 813;
const ulong XPE_EMPTYRESULT          = FVE_BOOKRR+ 814;
const ulong XPE_CONTEXTEXPECTED      = FVE_BOOKRR+ 815;
const ulong XPE_PATHEXPECTED         = FVE_BOOKRR+ 816;
const ulong XPE_DIVIDEBYZERO         = FVE_BOOKRR+ 817;

// XPath parser errors
const ulong XPE_UNKNOWNENTITY        = FVE_BOOKRR+ 850;
const ulong XPE_BADENTITY            = FVE_BOOKRR+ 851;
const ulong XPE_DOUBLECOLONEXPECTED  = FVE_BOOKRR+ 852;
const ulong XPE_QNAMEEXPECTED        = FVE_BOOKRR+ 853;
const ulong XPE_LPARENEXPECTED       = FVE_BOOKRR+ 854;
const ulong XPE_RPARENEXPECTED       = FVE_BOOKRR+ 855;
const ulong XPE_RPARENNOTEXPECTED    = FVE_BOOKRR+ 856;
const ulong XPE_RBRACKETEXPECTED     = FVE_BOOKRR+ 857;
const ulong XPE_VARNAMEEXPECTED      = FVE_BOOKRR+ 858;
const ulong XPE_LITERALEXPECTED      = FVE_BOOKRR+ 859;
const ulong XPE_UNEXPECTEDEND        = FVE_BOOKRR+ 860;
const ulong XPE_EQUALSIGNEXPECTED    = FVE_BOOKRR+ 861;
const ulong XPE_UNKNOWNOPERATOR      = FVE_BOOKRR+ 862;
const ulong XPE_TOOMANYCOLONS        = FVE_BOOKRR+ 863;
const ulong XPE_PARSEERROR           = FVE_BOOKRR+ 864;
const ulong XPE_NOVARS               = FVE_BOOKRR+ 865;
const ulong XPE_NOTIMPLEMENTED       = FVE_BOOKRR+ 866;

// XPath evaluator errors
const ulong FPE_UNDERRUN             = FVE_BOOKRR+ 900;
const ulong FPE_TOPICEXPECTED        = FVE_BOOKRR+ 901;
const ulong FPE_FINDLISTEXPECTED     = FVE_BOOKRR+ 902;
const ulong FPE_STRINGEXPECTED       = FVE_BOOKRR+ 903;
const ulong FPE_NUMBEREXPECTED       = FVE_BOOKRR+ 904;
const ulong FPE_BOOLEANEXPECTED      = FVE_BOOKRR+ 905;
const ulong FPE_OPNOTEXPECTED        = FVE_BOOKRR+ 906;
const ulong FPE_AXISNAMEUNKNOWN      = FVE_BOOKRR+ 907;
const ulong FPE_WRONGNRARGUMENTS     = FVE_BOOKRR+ 908;
const ulong FPE_PROCINSTEXPECTED     = FVE_BOOKRR+ 909;
const ulong FPE_STACKEMPTY           = FVE_BOOKRR+ 910;
const ulong FPE_STACKNOTEMPTY        = FVE_BOOKRR+ 911;
const ulong FPE_FUNCTIONUNKNOWN      = FVE_BOOKRR+ 912;
const ulong FPE_BADOPERANDTYPE       = FVE_BOOKRR+ 913;
const ulong FPE_EMPTYRESULT          = FVE_BOOKRR+ 914;
const ulong FPE_CONTEXTEXPECTED      = FVE_BOOKRR+ 915;
const ulong FPE_PATHEXPECTED         = FVE_BOOKRR+ 916;
const ulong FPE_DIVIDEBYZERO         = FVE_BOOKRR+ 917;

// XPath parser errors
const ulong FPE_UNKNOWNENTITY        = FVE_BOOKRR+ 950;
const ulong FPE_BADENTITY            = FVE_BOOKRR+ 951;
const ulong FPE_DOUBLECOLONEXPECTED  = FVE_BOOKRR+ 952;
const ulong FPE_QNAMEEXPECTED        = FVE_BOOKRR+ 953;
const ulong FPE_LPARENEXPECTED       = FVE_BOOKRR+ 954;
const ulong FPE_RPARENEXPECTED       = FVE_BOOKRR+ 955;
const ulong FPE_RPARENNOTEXPECTED    = FVE_BOOKRR+ 956;
const ulong FPE_RBRACKETEXPECTED     = FVE_BOOKRR+ 957;
const ulong FPE_VARNAMEEXPECTED      = FVE_BOOKRR+ 958;
const ulong FPE_LITERALEXPECTED      = FVE_BOOKRR+ 959;
const ulong FPE_UNEXPECTEDEND        = FVE_BOOKRR+ 960;
const ulong FPE_EQUALSIGNEXPECTED    = FVE_BOOKRR+ 961;
const ulong FPE_UNKNOWNOPERATOR      = FVE_BOOKRR+ 962;
const ulong FPE_TOOMANYCOLONS        = FVE_BOOKRR+ 963;
const ulong FPE_PARSEERROR           = FVE_BOOKRR+ 964;

// Object errors
const ulong FOB_OBJECTNAMEMISSING    = FVE_BOOKRR+ 980;
const ulong FOB_CREATEINTERFACEFAILED= FVE_BOOKRR+ 981;
const ulong FOB_CREATEOBJECTFAILED   = FVE_BOOKRR+ 982;
const ulong FOB_FINDMETHODFAILED     = FVE_BOOKRR+ 983;
const ulong FOB_WRONGPARAMETERCOUNT  = FVE_BOOKRR+ 984;
const ulong FOB_INCOMPATPARAMETERTYPE= FVE_BOOKRR+ 985;
const ulong FOB_INVALIDPARAMETERTYPE = FVE_BOOKRR+ 986;
const ulong FOB_EXECMETHODFAILED     = FVE_BOOKRR+ 987;
const ulong FOB_INVALIDRETURNTYPE    = FVE_BOOKRR+ 988;

#endif // ERRS_H

