
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


//*********************************************************************
// file.h
//*********************************************************************

#ifndef FILE_H
#define FILE_H

#ifndef FARBOOK_H
#include "atom.h"
#ifndef FV_NOREMOTE
#include "comm.h"
#endif // FV_NOREMOTE
#include "common.h"
#include "record.h"
#ifndef FV_NOTHREAD
#include "thread.h"
#endif // FV_NOTHREAD
#endif // FARBOOK_H

const char atEOL = 0xff;                 // Returned by ReadChar at EOL
                                  // WriteChar writes EOL when received

enum StdInOut {notStd, stdIn, stdOut};

enum DateStamp {createDate,
                lastAccess,
                lastWrite};

enum EBase  {fromHead,                                // seek from head
             fromCurr,                             // seek from current
             fromTail};                               // seek from tail

// _data format: |fileName
//               |crdate|crtime|crsecs
//               |moddate|modtime|modsecs
//               |size  + |dir/data

_FBDEF(CFile)
class FBC_USE CFile: public CAtom {
   public:
#ifndef FV_NOREMOTE
      static FB_USE_(PCFile) NewFile(PCComm _comm,
                                     Pchar  _userName,
                                     Pchar  _URI);
#endif

      FB_USE CFile           (void);
      FB_USE CFile           (const Pchar _name);
      FB_USE CFile           (const Pchar _name,
                              const Pchar _path,
                                    bool  _share = false);
      FB_USE CFile           (PCFile _file);
      virtual FB_USE ~CFile  ();

      virtual FB_USE_(short) Ima(void);
      virtual FB_USE_(Pchar) IAm(void);
      virtual FB_USE_(bool)  Isa(const short _type);

#ifndef _SLANGER
      FB_USE_(Pchar) Name(bool _alc);      // return the full file name
#endif // _SLANGER
      FB_USE_(Pchar) Path(void);
      FB_USE_(Pchar) Name(void);                    // farSlang version

      static FB_USE_(Pchar) GetDefaultDelim(void);

      FB_USE_(void) SetEOLDelim(const Pchar _delim);
                                        // Set end-of-line delimiter(s)
                           // note: restrict size to less than 16 bytes

      virtual FB_USE_(bool) Exists     (void);  // check if file exists
      virtual FB_USE_(bool) Rename     (Pchar _name);// rename the file
      virtual FB_USE_(bool) Backup     (void);       // backup the file
      virtual FB_USE_(bool) Delete     (void);       // delete the file

      virtual FB_USE_(bool) Create     (void);  // create a closed file

      FB_USE_(bool) FileIsOpen (void) { return fileIsOpen; }

      virtual FB_USE_(bool) OpenInput  (void);   // open file for input
      virtual FB_USE_(bool) OpenOutput (void);// create file for output
      virtual FB_USE_(bool) OpenAppend (void);        // append to file
      virtual FB_USE_(bool) OpenUpdate (void); // open for input/output

      virtual FB_USE_(bool) Close      (void);            // close file

      // (local only)
      // _data format: |fileName
      //               |crdate|crtime|crsecs
      //               |moddate|modtime|modsecs
      //               |size  + |dir/data
      FB_USE_(void) GetFileData(PCRecord _data);

      FB_USE_(bool)  atEOF     (void);            // Return true at EOF
      FB_USE_(void)  SetError  (ulong _err);
      FB_USE_(void)          Error   (ulong _err);
      virtual FB_USE_(ulong) Error   (void);

//?   FB_USE_(ulong) GetSysErr (void);      // Return last system error

      virtual FB_USE_(bool) Lock       (ulong _offset, ulong _size);
      virtual FB_USE_(bool) Unlock     (ulong _offset, ulong _size);

      FB_USE_(ulong)Size       (void);           // Return size of file
      FB_USE_(bool) Seek       (ulong _offset, EBase _from = fromHead);
                                                          // Set offset
      FB_USE_(ulong)Where      (void);     // Return offset within file

      FB_USE_(bool) Flush      (void);         // Flush buffers to file

      FB_USE_(char) ReadChar   (bool _upCase = false);// Read character

#ifndef _SLANGER
      FB_USE_(bool) ReadString (Pchar _p, ulong _size);// Read a string
//?   FB_USE_(bool) ReadString (PCString _s);          // Read a string
      FB_USE_(bool) ReadBlock  (Pvoid _p, ulong _size,  // Read a block
                                          Pulong _asize = 0);
#endif // _SLANGER
      // farSlang version
      FB_USE_(Pchar) ReadString(void);     // nothing for ReadBlock :-(

      FB_USE_(bool) WriteChar  (char  _p);         // Write a character
      FB_USE_(bool) WriteString(Pchar _p);    // Write a string and eol
//?   FB_USE_(bool) WriteString(PCString _s);// Write a string and eol
      FB_USE_(bool) WriteSubStr(Pchar _p);  // Write a string w/out eol
//?   FB_USE_(bool) WriteSubStr(PCString _s);  // Write a str w/out eol
      FB_USE_(bool) WriteBlock (Pvoid _p, ulong _size);// Write a block

#ifndef _SLANGER
      FB_USE_(bool) Printf     (Pchar _format, ...);// format and write
#endif // _SLANGER

   protected:
      static char eolDelim[];          // default end-of-line delimiter

      void  construct (Pchar _name, Pchar _path);
      void  setupFile (Pchar _name, Pchar _path);
      ulong logSysErr (void);
      bool  read      (Pvoid  _buffer,
                       ulong  _size,
                       Pulong _asize   = 0,
                       bool   _shortOk = false);
      bool  write     (Pvoid _buffer, ulong _size);

      Pchar             name;                      // full name of file
      StdInOut          stdInOut;               // standard in/out flag
      bool              share;
#ifndef FV_NOTHREAD
      PCCriticalSection fileCS;
#endif
      bool              fileExists;
      bool              fileIsOpen;
      bool              fileCanWrite;
      FB_HANDLE         file;
      char              delim[sizeName];   // current end-of-line delim
      long              whereAt;            // position within the file
      ulong             sysErr;               // operating system error
      char              ch;
   }; // CFile

#endif // FILE_H

