
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


//*********************************************************************
// findlist.h
//*********************************************************************

#ifndef FINDLIST_H
#define FINDLIST_H

#ifndef FARBOOK_H
#include "book.h"
#include "hash.h"
#include "list.h"
#endif

//---------------------------------------------------------------------
// Find item class
//
_FBDEF(CFindItem)
class FBC_USE CFindItem: public CItem {
   public:
      FB_USE CFindItem          (PCBook _book,
                                 PCRef  _ref,
                                 Pchar   _name,
                                 ulong   _position);
      FB_USE CFindItem          (PCFindItem _item);

      virtual FB_USE ~CFindItem ();

      virtual FB_USE_(short) Ima(void);
      virtual FB_USE_(Pchar) IAm(void);
      virtual FB_USE_(bool)  Isa(const short _type);

      FB_USE_(void)   SetName    (Pchar _name);
#ifndef _SLANGER
      FB_USE_(Pchar)  GetName    (bool  _alc);
#endif // _SLANGER
      FB_USE_(Pchar)  GetName    (void);            // farSlang version

      FB_USE_(PCBook) GetBook    (void);
      FB_USE_(void)   SetBook    (PCBook _book);
      FB_USE_(PCRef)  GetRef     (void);
      FB_USE_(void)   SetRef     (PCRef _ref) { ref = new CRef(_ref); }
      FB_USE_(bool)   RefEquals  (PCRef _ref);
      FB_USE_(void)   SetPosition(ulong  _position);
      FB_USE_(ulong)  GetPosition(void);

   protected:
      PCBook book;                   // farBook in which node was found
      PCRef  ref;          // reference describing where node was found
      Pchar  name;                              // node name (if known)
      ulong  position;                     // active character position
   }; // CFindItem

//---------------------------------------------------------------------
// Find list class
//
_FBDEF(CFindList)
class FBC_USE CFindList: public CList {
   public:
      FB_USE CFindList(void);
      FB_USE CFindList(PCFindList _list);
      virtual FB_USE ~CFindList();

      virtual FB_USE_(short) Ima(void);
      virtual FB_USE_(Pchar) IAm(void);
      virtual FB_USE_(bool)  Isa(const short _type);

      FB_USE_(ulong) Error    (void);          // Get last access error
      FB_USE_(void)  Error    (ulong _error);

      FB_USE_(PCFindItem)
                    FindItem  (void);
      FB_USE_(PCFindItem)
                    FindItem  (long   _where);
      FB_USE_(bool) FindKey   (PCBook _book,
                               char    _mark,           // char = aMark
                               Pchar   _name,
                               bool    _trunc = false);
      FB_USE_(bool) FindRef   (PCRef  _ref);
      FB_USE_(bool) FindInsert(PCFindItem _item);
      FB_USE_(bool) FindInsert(PCBook _book,
                               PCRef  _ref,
                               Pchar   _name,
                               ulong   _position);

      FB_USE_(bool) GetNames  (PCBook _book);

   protected:
      ulong error;
   }; // CFindList

#endif // FINDLIST_H

