
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//*********************************************************************
// hshlist.cpp
//*********************************************************************

#include "prefixb.h"
#include "hshlist.h"

const ushort hashWide = 32;

//---------------------------------------------------------------------
// Constructor
//
CHshList::CHshList(short _wide):
   CItem(),
   wide (_wide) {
   if (wide == 0)
      wide = hashWide;
   table = new PCList[wide];
   for (short i = 0; i < wide; ++i)
      table[i] = 0;
   } // CHshList

//---------------------------------------------------------------------
// Destructor
//
CHshList::~CHshList() {
   list = 0; 
   Flush();
   delete[] table;
   table = 0;
   } // ~CHshList

//---------------------------------------------------------------------
//
RTTI_CPP(HshList, Item)

//---------------------------------------------------------------------
// Return the hash value of the specified event
//
ushort CHshList::hash(Pvoid _key) {
   return (ushort(labs(ulong(_key))) % wide);
   } // hash

//---------------------------------------------------------------------
// Add a list to the table. If already present, return false.
//
bool CHshList::addTable(ushort _i) {
   if (table[_i])
      return false;

   table[_i] = new CList();

   return true;   
   } // addTable

//---------------------------------------------------------------------
// Hash the key parameter, then add the item to the specified list.
//
void CHshList::Insert(Pvoid _key, PCItem _pItem, AtWhere _where) {
   Assert(_pItem);
   PCList lst = List(_key);
   lst->Insert(_pItem, _where);
   } // Insert

//---------------------------------------------------------------------
// Hash the specified key parameter, then return the pointer to the
// list in the corresponding table entry.
//
PCList CHshList::List(Pvoid _key) {
   ushort i = hash(_key);
   addTable(i);
   list     = table[i];
   return list;
   } // List

//---------------------------------------------------------------------
//
PCList CHshList::GetList(short _list) {
   if (_list >= wide)
      return 0;
   else
      return table[_list];
      } // GetList

//---------------------------------------------------------------------
//
PCItem CHshList::GetFirstItem(Pvoid _key) {
   key = _key;
   List(key);
   if (list && list->FindFirst(key))
      return list->Item();
   else
      return 0;
   } // GetFirstItem

//---------------------------------------------------------------------
//
PCItem CHshList::GetNextItem(void) {
   if (list && list->FindNext(key))
      return list->Item();
   else
      return 0;
   } // GetNextItem

//---------------------------------------------------------------------
//
PCItem CHshList::Remove(AtWhere _where, bool _deleteIt) {
   Assert(list);

   return list->Remove(_where, _deleteIt);
   } // Remove

//---------------------------------------------------------------------
// Flush and destroy the lists in the table.
//
void CHshList::Flush(void) {
   for (short i = 0; i < wide; ++i) {
//?   delete table[i];
      table[i] = 0;
      }
   } // Flush

