
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


//********************************************************************
// lclbook.h
//*********************************************************************

#ifndef LCLBOOK_H
#define LCLBOOK_H

#ifndef FARBOOK_H
#include "book.h"
#endif

// farBook object -------------------------------------------------------

_FBDEF(CLocalBook)
class FBC_USE CLocalBook: public CBook {
   public:
      FB_USE CLocalBook(Pchar _name);
      virtual FB_USE ~CLocalBook();

      virtual FB_USE_(short) Ima(void);
      virtual FB_USE_(Pchar) IAm(void);
      virtual FB_USE_(bool)  Isa(const short _type);

      virtual FB_USE_(ulong) Error   (void);
      virtual FB_USE_(void)  SetError(ulong _error) {
         err = _error;
         }
      FB_USE_(void)          Error   (ulong _error) {
         err = _error;
         }

      // Book properties method
      virtual FB_USE_(bool) UpdateInfo(void);

      virtual FB_USE_(bool) Create(long _nrParts,
                                   long _nrRefSlots,
                                   long _nrHshSlots,
                                   long _sizeKey,
                                   long _nrKeys);   // Create a farBook
      virtual FB_USE_(bool) Open  (void);               // Open a farBook
      virtual FB_USE_(bool) Close (void);

      // Lock and unlock a farBook region
      virtual FB_USE_(bool) Lock    (PCRef _ref, AWaitKind _wait);
      virtual FB_USE_(bool) Unlock  (PCRef _ref);

      virtual FB_USE_(PCRefs) GetFile (void);

      virtual FB_USE_(bool)   GetDesc (ulong _loc, PCRecord _desc);

   // link i/o --------------------------------------------------------

      virtual FB_USE_(bool) MakeBookKey   (AMark   _mark,
                                           Pchar   _key,
                                           PCRef  _ref,
                                           bool    _trunc = false);
      virtual FB_USE_(bool) IfBookKey     (AMark   _mark,
                                           Pchar   _key,
                                           bool    _trunc = false);
      virtual FB_USE_(bool) FindBookKey   (AMark   _mark,
                                           Pchar   _key,
                                           PCList _pFind,
                                           bool    _trunc = false);
      virtual FB_USE_(bool) DeleteBookKey (AMark   _mark,
                                           Pchar   _key,
                                           PCRef  _ref,
                                           bool    _trunc = false);
      virtual FB_USE_(bool) RenameNode    (Pchar   _oldName,
                                           Pchar   _newName,
                                           bool    _trunc = false);

   // node i/o -------------------------------------------------------


   public:
      virtual FB_USE_(PCAtom) ReadPart    (PCRef      _ref);
      virtual FB_USE_(bool)   WritePart   (PCRef      _ref,
                                           PCAtom     _part);
      virtual FB_USE_(bool)   CreateNode  (PCRef      _ref,
                                           PCAtom     _body);

      virtual FB_USE_(PCAtom) GetContent  (PCRef      _ref);

   protected:
      bool   openNode  (PCRef _ref, ABookAction _action);
      bool   closeNode (void);

      ulong calcSize   (ulong _size);
      PCAtom readPart  (void);
      bool   writePart (PCAtom _part);

      void error       (AnAction _action, Pchar _mssg, ushort _errCd);
      ulong checkErrors(PCFile _f);

      PCRefs theFile;
      PCList locks;
   }; // CLocalBook

#endif // LCLBOOK_H

