
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//*********************************************************************
// list.cpp
//*********************************************************************
//
// Indexing into the list is one-based.
//
// there seems to be a bug in Replace. I had a short list of PtrItems
// each pointing to an editWindow. I needed to change the item to point
// to a char array as in the following code:
//
//   if (edits.GotoHead()) do {
//      PCEditWindow edit = PCEditWindow(PCPtrItem(edits.Item())->GetPtr());
//      char line[sizeString];
//      edit->GetText(line, sizeString);
//      Pchar text         = new char[strlen(line)+1];
//      strcpy(text, line);
//      PCPtrItem p = new CPtrItem(text);
//      edits.Replace(p);
//      } while (edits.GotoNext());
//
// this code didn't bomb but it apparently screwed up the heap because
// subsequent news failed. It appeared that the problem was somewhere
// in Replace. I never found out what was wrong though.
//
//*********************************************************************
//
// Lifted from Java documentation. Contains some good observations and
// caveats that I should include in my own documentation, whenever that
// happens. Of course, the list class is the only linear collection
// mechanism that farBook offers. "When all you have is a hammer, every-
// thing looks like a nail."
//
// begin quote:
//
// An ordered collection (also known as a sequence). The user of this
// interface has precise control over where in the list each element is
// inserted. The user can access elements by their integer index
// (position in the list), and search for elements in the list.
//
// Unlike sets, lists typically allow duplicate elements. More formally,
// lists typically allow pairs of elements e1 and e2 such that
// e1.equals(e2), and they typically allow multiple null elements if
// they allow null elements at all. It is not inconceivable that someone
// might wish to implement a list that prohibits duplicates, by throwing
// runtime exceptions when the user attempts to insert them, but we
// expect this usage to be rare.
//
// The List interface places additional stipulations, beyond those
// specified in the Collection interface, on the contracts of the
// iterator, add, remove, equals, and hashCode methods. Declarations for
// other inherited methods are also included here for convenience.
//
// The List interface provides four methods for positional (indexed)
// access to list elements. Lists (like Java arrays) are zero based.
// Note that these operations may execute in time proportional to the
// index value for some implementations (the LinkedList class, for
// example). Thus, iterating over the elements in a list is typically
// preferable to indexing through it if the caller does not know the
// implementation.
//
// The List interface provides a special iterator, called a ListIterator,
// that allows element insertion and replacement, and bidirectional
// access in addition to the normal operations that the Iterator
// interface provides. A method is provided to obtain a list iterator
// that starts at a specified position in the list.
//
// The List interface provides two methods to search for a specified
// object. From a performance standpoint, these methods should be used
// with caution. In many implementations they will perform costly linear
// searches.
//
// The List interface provides two methods to efficiently insert and
// remove multiple elements at an arbitrary point in the list.
//
// Note: While it is permissible for lists to contain themselves as
// elements, extreme caution is advised: the equals and hashCode methods
// are no longer well defined on a such a list.
//
// Some list implementations have restrictions on the elements that they
// may contain. For example, some implementations prohibit null
// elements, and some have restrictions on the types of their elements.
// Attempting to add an ineligible element throws an unchecked
// exception, typically NullPointerException or ClassCastException.
// Attempting to query the presence of an ineligible element may throw
// an exception, or it may simply return false; some implementations
// will exhibit the former behavior and some will exhibit the latter.
// More generally, attempting an operation on an ineligible element
// whose completion would not result in the insertion of an ineligible
// element into the list may throw an exception or it may succeed, at
// the option of the implementation. Such exceptions are marked as
// "optional" in the specification for this interface.
//
// end quote
//
//*********************************************************************

#include "prefixb.h"
#ifndef FV_NOFACTORY
#include "factory.h"
#endif
#include "file.h"
//#include "heapwrap.h"
#include "list.h"

//=====================================================================
//
CItem::CItem(void):
   CAtom(),
   next (0),
   last (0) {
   } // CItem

//---------------------------------------------------------------------
//
CItem::~CItem() {
   } // ~CItem

//---------------------------------------------------------------------
//
RTTI_CPP(Item, Atom)

//---------------------------------------------------------------------
// Don't trust this one to tell the truth - override it!
//
bool CItem::match (Pvoid p) {
   p;
   return true;
   } // match
/*? any use in this? don't think so
//---------------------------------------------------------------------
//
PCItem CItem::Copy(long _begin, long _end) {
   _begin; _end;
   return new CItem;
   } // Copy
?*/

//---------------------------------------------------------------------
//
PCItem CItem::PrevSibItem(void) {
   return last;
   } // PrevSibItem

//---------------------------------------------------------------------
//
PCItem CItem::NextSibItem(void) {
   return next;
   } // NextSibItem

//---------------------------------------------------------------------
//
bool CItem::operator ==(PCItem _item) {
   _item;
   return true;
   } // operator ==

//---------------------------------------------------------------------
//
bool CItem::operator !=(PCItem _item) {
   _item;
   return false;
   } // operator !=

//=====================================================================
// CPtrItem

//---------------------------------------------------------------------
//
CPtrItem::CPtrItem(Pvoid _p):
   CItem(),
   p    (_p) {
   } // CPtrItem

//---------------------------------------------------------------------
//
CPtrItem::~CPtrItem() {
   } // ~CPtrItem

//---------------------------------------------------------------------
//
RTTI_CPP(PtrItem, Item)

//---------------------------------------------------------------------
//
Pvoid CPtrItem::GetPtr(void) {
   return p;
   } // GetPtr

//---------------------------------------------------------------------
//
void CPtrItem::PutPtr(Pvoid _p) {
   p = _p;
   } // // PutPtr
   
//=====================================================================
// CList

//---------------------------------------------------------------------
// Construct an empty list
//
CList::CList(void):
   CItem  (),
   info   (0),
   top    (0),
   bot    (0),
   cur    (0),
   itemNr (0),
   nrItems(0),
   charNr (0),
   going  (0) {
   } // CList

//---------------------------------------------------------------------
// Delete the list
//
CList::~CList() {
if (going)
   Assert(!going);

   while (top)
      Remove();
   } // ~CList

//---------------------------------------------------------------------
//
RTTI_CPP(List, Item)
/*?

//---------------------------------------------------------------------
// Copy this.
// if _begin > 0, then offset beginning as specified.
// if _end   > 0, then offset ending as specified.
// if _begin < 0, then add nrItems to set beginning.
// if _end   < 0, then add nrItems to set ending.
// if _begin = 0 and _end = 0, then copy entire list.
//
//??DMY PCList CList::Copy(long _begin, long _end) {
PCItem CList::Copy(long _begin, long _end) {
//?PCList pList = new CList();
   PCItem pList = new CList();
   if (top) {
      // Save state of this list
      long iNr  = itemNr;
      pCItem cr = cur;

      // Bound the sublist to copy
      long begin = 1;
      long end   = nrItems;
      if (_begin == 0 && _end == 0);
      else {
         if      (_begin < 0)
            begin = _begin + nrItems;
         else if (_begin > 0)
            begin = _begin;
         if      (_end   < 0)
            end   = _end   + nrItems;
         else if (_end   > 0)
            end   = _end;
         }
      if (begin <= nrItems && end >= 1) {
         begin = Bound(begin, 1L, nrItems);
         end   = Bound(end,   1L, nrItems);

         // Copy each item into the new list
         for (int i = begin; i <= end; i++) {
            GotoItem(i);
            TYPECAST(pList, List)->Insert(Item()->Copy());
            }
         }

      // Restore state of this list
      itemNr = iNr;
      cur    = cr;
      }
   return pList;
   } // Copy
?*/

//---------------------------------------------------------------------
// Return true if list is equal.
// (must be same length and every item must be equal)
//
bool CList::operator ==(PCItem _item) {
   pCList pList = pCList(_item.getPtr());
   if (!pList)
      return false;
   else if (nrItems != pList->nrItems)
      return false;
   else {
      // Save state of lists
      long iNr1  = itemNr;
      long iNr2  = pList->itemNr;
      pCItem cr1 = cur;
      pCItem cr2 = pList->cur;

      bool compare = true;
      for (int i = 1; i < nrItems; i++) {
         GotoItem(i);
         pList->GotoItem(i);
         // Check that the two items have the same type
         compare = Item()->Isa(pList->Item()->Ima());
         if (!compare)
            break;
         // Check that the two items have the same value
         compare = (Item() == pList->Item());
         if (!compare)
            break;
         }

      // Restore state of lists
      itemNr        = iNr1;
      pList->itemNr = iNr2;
      cur           = cr1;
      pList->cur    = cr2;
      return compare;
      }
   } // operator ==

//---------------------------------------------------------------------
// Return true if list is not equal.
//
bool CList::operator !=(PCItem _item) {
   pCList pList = pCList(_item.getPtr());
   if (!pList)
      return false;
   else if (nrItems != pList->nrItems)
      return true;
   else {
      // Save state of lists
      long iNr1  = itemNr;
      long iNr2  = pList->itemNr;
      pCItem cr1 = cur;
      pCItem cr2 = pList->cur;

      bool compare = true;
      for (int i = 1; i < nrItems; i++) {
         GotoItem(i);
         pList->GotoItem(i);
         // Check that the two items have the same type
         compare = Item()->Isa(pList->Item()->Ima());
         if (!compare)
            break;
         // Check that the two items have the same value
         compare = (Item() == pList->Item());
         if (!compare)
            break;
         }

      // Restore state of lists
      itemNr        = iNr1;
      pList->itemNr = iNr2;
      cur           = cr1;
      pList->cur    = cr2;
      return !compare;
      }
   } // operator !=

//---------------------------------------------------------------------
// Insert an item into the list (private)
//
// Inserts the specified item into the sublist between
// p1 and p2. Assumes that p1 and p2 are adjacent. Won't work if they
// aren't. Note that this should be a critical section when
// multithreading.
//
void CList::insItem(pCItem _p1, pCItem _p2, pCItem _item) {
   _item->next = _p2;
   if (_p1)
      _p1->next = _item;

   _item->last = _p1;
   if (_p2)
      _p2->last = _item;

   SetDirty(true, shallow);
   } // insItem

//---------------------------------------------------------------------
// Delete an item from the list (private)
//
// Note that this should be a critical section when multithreading.
//
void CList::rmvItem(pCItem _item) {
   // Keep the item until we are willing to let it go
   PCItem p = _item;

   if (_item->last)
      _item->last->next = _item->next;
   if (_item->next)
      _item->next->last = _item->last;

   _item->next = 0;
   _item->last = 0;

   SetDirty(true, shallow);
   } // rmvItem

//---------------------------------------------------------------------
// Report the number of items in the list
//
long CList::Number(void) {
   return nrItems;
   } // Number

//---------------------------------------------------------------------
// Report the number of items in the list
//
long CList::Length(void) {
   return nrItems;
   } // Length

//---------------------------------------------------------------------
// Report the position in the list
//
long CList::Where(void) {
   return itemNr;
   } // Where

//---------------------------------------------------------------------
// Report true if at the top of the list
//
bool CList::AtHead(void) {
   return (itemNr <= 1);
   } // AtHead

//---------------------------------------------------------------------
// Report true if at the end of the list
//
bool CList::AtTail(void) {
   return (itemNr >= nrItems);
   } // AtTail

//---------------------------------------------------------------------
// Set the position in the list
//
bool CList::GotoItem(long _itemNr) {
   Assert(!going);

   if (!top)
      return false;
   if      (_itemNr < 0)
      _itemNr += nrItems + 1;

   if      (_itemNr <= 1) {
      itemNr = 1;
      cur    = top;
      }
   else if (_itemNr >= nrItems) {
      itemNr = nrItems;
      cur    = bot;
      }
   else if (_itemNr < itemNr) while (_itemNr < itemNr) {
      itemNr--;
      cur    = cur->last;
      }
   else while (_itemNr > itemNr) {
      itemNr++;
      cur    = cur->next;
      }
   charNr = 0;
   return (_itemNr == itemNr);
   } // GotoItem

//---------------------------------------------------------------------
// Go to the first item in the list
//
bool CList::GotoHead(void) {
   Assert(!going);

   if (!top)
      return false;
   itemNr = 1;
   cur    = top;
   charNr = 0;
   return true;
   } // GotoHead

//---------------------------------------------------------------------
// Go to the last item in the list
//
bool CList::GotoTail(void) {
   Assert(!going);

   if (!top)
      return false;
   itemNr = nrItems;
   cur    = bot;
   charNr = 0;
   return true;
   } // GotoTail

//---------------------------------------------------------------------
// Go to the next item in the list
//
bool CList::GotoNext(void) {
   Assert(!going);

   if (!top)
      return false;
   if (itemNr == nrItems)
      return false;
   itemNr++;
   cur    = cur->next;
   charNr = 0;
   return true;
   } // GotoNext

//---------------------------------------------------------------------
// Go to the previous item in the list
//
bool CList::GotoPrev(void) {
   Assert(!going);

   if (!top)
      return false;
   if (itemNr <= 1)
      return false;
   itemNr--;
   cur    = cur->last;
   charNr = 0;
   return true;
   } // GotoPrev

//---------------------------------------------------------------------
//
PCItem CList::GetHeadItem(void) {
   return top;
   } // GetHeadItem

//---------------------------------------------------------------------
PCItem CList::GetTailItem(void) {
   return bot;
   } // GetTailItem

//---------------------------------------------------------------------
// Insert an item at the active position in the list
//
// Thread p into the list. If the list is empty, initialize all
// components. Set the current pointer to the new item. Increment the
// number of items to reflect the insertion. The item now belongs to
// the list and can be deleted by the CList.Remove method.

//
bool CList::Insert(PCItem _item, AtWhere _where) {
   Assert(!going);
   Assert(_item);
   Assert(_item->Isa(iItem));

   // insert the item into the list
   if      (!top) {
      top     = _item;
      bot     = _item;
      itemNr  = 1;
      nrItems = 0;
      }
   else if (_where == before || _where == atTop)
      if      (top.getPtr() == cur) {
         insItem(0, top, _item);
         top = _item;
         }
      else if (_where == atTop) {
         GotoHead();
         insItem(0, top, _item);
         top = _item;
         }
      else // _where == before
         insItem(cur->last, cur, _item);
   else {
      // _where == after or atEnd
      if (_where == atEnd) {
         GotoTail();
         bot = _item;
         }
      insItem(cur, cur->next, _item);
      if (cur == bot)
         bot = _item;
      itemNr++;
      }

   cur = _item;
   nrItems++;
   return true;
   } // Insert

//---------------------------------------------------------------------
// Insert an item at the active position in the list
//
bool CList::PtrInsert(PCAtom _p, AtWhere _where) {
   PCItem p = new CPtrItem(_p);
   return Insert(p, _where);
   } // PtrInsert

//---------------------------------------------------------------------
// Insert an item at the active position in the list
//
bool CList::PtrInsert(Pvoid _p, AtWhere _where) {
   PCItem p = new CPtrItem(_p);
   return Insert(p, _where);
   } // PtrInsert

//---------------------------------------------------------------------
// Remove the active item from the list. Default result is the next
// item becomes active. note: Going logic added to avoid double
// jeopardy when an item removes itself from the list as part of its
// destruction.
//
PCItem CList::Remove(AtWhere _where, bool _deleteIt) {
   if (!nrItems)
      return 0;

   pCItem nxt;
   pCItem lst;
   PCItem it;

   if (going)                          // Check for removal in progress
      if (cur == going)
         return 0;               // Trying to remove item being removed
      else
         Assert(false);               // Can't remove two items at once
   else
      going = cur;               // Remember this item is being removed

   if      (cur == top) {                         // Removing head item
      nxt = cur->next;
      lst = cur->next;
      top = nxt;
      }
   else if (cur == bot) {                         // Removing tail item
      nxt = cur->last;
      lst = cur->last;
      bot = lst;
      itemNr--;
      }
   else {                                     // Removing interior item
      nxt = cur->next;
      lst = cur->last;
      }

   if (!_deleteIt)
      it  = cur;

   rmvItem(cur);               // Remove the current item from the list

   charNr = 0;
   nrItems--;

   if      (nrItems == 0) {                  // Clean up for empty list
      top    = 0;
      bot    = 0;
      itemNr = 0;
      cur    = 0;
      }
   else if (_where == atTop) {            // Make the head item current
      cur    = top;
      if (cur)
         itemNr = 1;
      }
   else if (_where == before) {       // Make the previous item current
      cur    = lst;
      if (itemNr > 1)
         itemNr--;
      }
   else if (_where == after)              // Make the next item current
      cur    = nxt;
   else if (_where == atEnd) {            // Make the tail item current
      itemNr = nrItems;
      cur    = bot;
      }
   else
      Assert(false);             // Don't recognize the AtWhere command

   going = 0;                              // Not removing anything now
   return it;
   } // Remove

//---------------------------------------------------------------------
// Return the item at the active position in the list
//
PCItem CList::Item(void) {
if (!cur)
   Assert(cur);
   return cur;
   } // Item

//---------------------------------------------------------------------
// Return the item at the specified position in the list
//
PCItem CList::Item(long _where) {
   if (GotoItem(_where))
      return cur;
   else
      return 0;
   } // Item

//---------------------------------------------------------------------
// Return the item at the active position in the list
//
Pvoid CList::PtrItem(void) {
   if (cur)
      return pCPtrItem(cur)->p;
   else
      return 0;
   } // PtrItem

//---------------------------------------------------------------------
// Return the item at the specified position in the list
//
Pvoid CList::PtrItem(long _where) {
   if (GotoItem(_where)) {
      Assert(cur->Isa(iPtrItem));
      return pCPtrItem(cur)->p;
      }
   else
      return 0;
   } // Item

//---------------------------------------------------------------------
// Replace the item at the active position in the list
//
bool CList::Replace(PCItem _item) {
   Assert(!going);
   Assert(_item);
   Assert(_item->Isa(iItem));

   if (!cur)
      return false;

   if (itemNr == nrItems) {
      // tail is special case
      Remove();
      return Insert(_item, after);
      }
   else {
      Remove();
      return Insert(_item, before);
      }
   } // Replace

//---------------------------------------------------------------------
// Replace the item at the active position in the list
// Note: _p is not reference counted!!!
//
bool CList::PtrReplace(Pvoid _p) {
   Assert(!going);
   Assert(cur);
   Assert(cur->Isa(iPtrItem));

   pCPtrItem(cur)->p = _p;
   return true;
   } // PtrReplace

//---------------------------------------------------------------------
// Find an item in the list. Starts at the current item.
//
bool CList::FindItem(Pvoid _p) {
   Assert(!going);

   if (!top)
      return false;

   pCItem savCur = cur;
   long savItm   = itemNr;
   for (itemNr, cur; itemNr <= nrItems; itemNr++, cur = cur->next)
      if (cur->match(_p))
         // found a match
         return true;

   itemNr = savItm;
   cur    = savCur;
   return false;
   } // FindItem

//---------------------------------------------------------------------
// Find an item in the list - first time. Starts at the top of the
// list.
//
bool CList::FindFirst(Pvoid _p) {
   Assert(!going);

   if (!top)
      return false;

   pCItem savCur = cur;
   long   savItm = itemNr;

   for (itemNr = 1, cur = top; itemNr <= nrItems; itemNr++, cur = cur->next)
      if (cur->match(_p))
         // found a match
         return true;

   itemNr = savItm;
   cur    = savCur;
   return false;
   } // FindFirst

//---------------------------------------------------------------------
// Find an item in the list. Starts at the next item in the list.
//
bool CList::FindNext(Pvoid _p) {
   Assert(!going);

   if (!top)
      return false;

   pCItem savCur = cur;
   long   savItm = itemNr;

   for (itemNr++, cur = cur->next; itemNr <= nrItems;  itemNr++, cur = cur->next)
      if (cur->match(_p))
         // found a match
         return true;

   cur    = savCur;
   itemNr = savItm;
   return false;
   } // FindNext

//---------------------------------------------------------------------
// Find an item in the list using its address.
//
long CList::FindWhere(PCItem _item) {
   Assert(!going);

   if (!top)
      return 0;

   pCItem savCur = cur;
   long   savItm = itemNr;

   for (itemNr = 1, cur = top; itemNr <= nrItems; itemNr++, cur = cur->next)
      if (cur == _item)
         // found the item
         return itemNr;

   itemNr = savItm;
   cur    = savCur;
   return 0;
   } // FindWhere

//---------------------------------------------------------------------
// Remove all the items from the list
//

void CList::Flush(void) {
if (going)
   Assert(!going);

   while (top)
      Remove();
   } // Flush

//---------------------------------------------------------------------
// Return the size of space the list occupies in the farBook
//
ulong CList::BookSize(void) {
   ulong size = sizeof(nrItems);

   if (!top)
      return size;

   pCItem savCur = cur;
   long   savItm = itemNr;

   for (itemNr = 1, cur = top;
        itemNr <= nrItems;
        itemNr++, cur = cur->next)
      size += cur->BookSize() + sizeof(short);

   itemNr = savItm;
   cur    = savCur;
   return size;
   } // BookSize

//---------------------------------------------------------------------
// Read the list from the farBook. Assumes that the file pointer points
// to the number of items to be read if the input is a farBook,
// otherwise, read items until eof.
//
bool CList::ReadFile(PCAtom _input, bool _isBook) {
   PCFile input = TYPECAST(_input, File);
   Assert(input);

   // Read the number of items from the farBook
   ulong nr    = 32767;
   if (_isBook)
      if (!(input->ReadBlock(&nr, sizeof(nr))))
         return false;

   // Read the items from the file
   for (ulong i = 0; i < nr; i++)
      if (!readItem(_input, _isBook))
         if (_isBook) {
            if (input->Error() == FVE_ENDOFFILE)
               input->SetError(FVE_UNEXPECTEDEND);
            return false;
            }
         else {
            if (input->Error() == FVE_ENDOFFILE)
               return true;
            return false;
            }

   return true;
   } // ReadFile

//---------------------------------------------------------------------
// Write the list to the farBook. If the output is a farBook, write the
// number of items first, then write the items. Saves the list position.
//
bool CList::WriteFile(PCAtom _output, bool _isBook) {
   PCFile output = TYPECAST(_output, File);
   Assert(output);

   // Write the number of items to the farBook
   if (_isBook)
      if (!output->WriteBlock(&nrItems, sizeof(nrItems)))
          return false;

   long where = itemNr;

   // Write the items to the file
   if (GotoHead()) do
      if (!writeItem(_output, _isBook)) {
         GotoItem(where);
         return false;
         }
      while (GotoNext());

   GotoItem(where);
   return true;
   } // WriteFile

//---------------------------------------------------------------------
// Read a list from the specified (remote/local) file
//
bool CList::ReadRemote(PCAtom _input) {
   PCFile input = TYPECAST(_input, File);

   err = FVE_OK;

   if (!(input->OpenInput())) {
      err = input->Error();
      return false;
      }

   // Read the items from the file
   do {
      if (!(readItem(_input, false))) {
//?      err = input->Error();
         if (err == FVE_ENDOFFILE)
            err = FVE_OK;
         input->Close();
         return (err == FVE_OK);
         }
      } while (true);
   } // ReadRemote

//---------------------------------------------------------------------
// Write the contents of the list to the specified (remote/local) file.
//
bool CList::WriteRemote(PCAtom _output) {
   PCFile output = TYPECAST(_output, File);

   err = FVE_OK;

   output->SetEOLDelim(CFile::GetDefaultDelim());
   if (!(output->OpenOutput())) {
      err = output->Error();
      return false;
      }

   long where = itemNr;

   // Write the items to the file
   PCAtom it = TYPECAST(output, Atom);
   if (GotoHead()) do {
      if (!writeItem(it, false)) {
         err = output->Error();
         output->Close();
         return false;
         }
      } while (GotoNext());

   GotoItem(where);

   output->Close();

   return true;
   } // WriteRemote

//---------------------------------------------------------------------
// Read a list from the specified file
//
bool CList::Load(Pchar _fileName) {
   PCAtom  input = new CFile(_fileName);
   return ReadRemote(input);
   } // Load

//---------------------------------------------------------------------
// Write the contents of the list to the specified file.
//
bool CList::Save(Pchar _fileName) {
   PCAtom output = new CFile(_fileName);
   return WriteRemote(output);
   } // Save

//---------------------------------------------------------------------
// Read an item from the specified file. (protected)
//
bool CList::readItem(PCAtom _input, bool _isBook) {
   PCFile input = TYPECAST(_input, File);
   Assert(input);

   // Read the item type from the file
   short itIsa;
   if (!(input->ReadBlock(&itIsa, sizeof(short))))
      return false;

#ifndef FV_NOFACTORY
   // Read the item from the file
   Assert(CFactory::TheFactory);
   PCAtom atom = CFactory::TheFactory->MakeAtom(itIsa);
   PCItem item = TYPECAST(atom, Item);
   if (!(item->ReadFile(_input, _isBook)))
     return false;

   Insert(item, after);
#else
   Assert(false);
#endif // FV_NOFACTORY

   return true;
   } // readItem

//---------------------------------------------------------------------
// Write an item to the specified file. (protected)
//
bool CList::writeItem(PCAtom _output, bool _isBook) {
   PCFile output = TYPECAST(_output, File);
   Assert(output);

   if (Empty())
      return false;                               // FVE_NOITEMTOWRITE;

   // Write the item type to the file
   short itIsa = Item()->Ima();
   if (!(output->WriteBlock(&itIsa, sizeof(short))))
      return false;

   // Write the item to the file
   return Item()->WriteFile(_output, _isBook);
   } // writeItem


//---------------------------------------------------------------------
//
void CList::SetDirty(bool _dirty, depth _deep) {
   CAtom::SetDirty(_dirty, shallow);
   if (_deep == deep) {
      long where = itemNr;

      if (GotoHead()) do
         Item()->SetDirty(_dirty, _deep);
         while (GotoNext());

      GotoItem(where);
      }
   } // SetDirty

//---------------------------------------------------------------------
//
bool CList::IsDirty(depth _deep) {
   bool drty = CAtom::IsDirty(shallow);
   if (_deep == deep) {
      long where = itemNr;

      if (GotoHead()) do
         drty |= Item()->IsDirty(_deep);
         while (GotoNext());
      GotoItem(where);
      }

   return drty;
   } // IsDirty


