
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


//*********************************************************************
// list.h
//*********************************************************************

#ifndef LIST_H
#define LIST_H

#ifndef FARBOOK_H
#include "atom.h"
#include "common.h"
#endif

//=====================================================================
// Item class declaration
//=====================================================================

_FBDEF(CItem)

class FBC_USE CItem: public CAtom {
   friend class CList;

   public:
      FB_USE CItem(void);
      virtual FB_USE ~CItem();

      virtual FB_USE_(short) Ima(void);
      virtual FB_USE_(Pchar) IAm(void);
      virtual FB_USE_(bool)  Isa(const short _type);

//?   virtual FB_USE_(PCItem) Copy(long _begin = 0, long _end = 0);

      virtual FB_USE_(PCItem) PrevSibItem(void);
      virtual FB_USE_(PCItem) NextSibItem(void);

      // Conditional operators
      virtual FB_USE_(bool) operator ==(PCItem _item);
      virtual FB_USE_(bool) operator !=(PCItem _item);

   protected:
      virtual FB_USE_(bool) match(Pvoid p);

      PCItem  next, last;
   }; // CItem

#ifndef _SLANGER
//=====================================================================
// Item class for CItem pointer items
//=====================================================================

//---------------------------------------------------------------------
// Class to allow a pointer of an object to be placed on a list instead
// of the object itself. This allows an object to be on any number of
// lists safely.
//
_FBDEF(CPtrItem)

class FBC_USE CPtrItem: public CItem {
   friend class CList;

   public:
      FB_USE CPtrItem(Pvoid _p);
      virtual FB_USE ~CPtrItem();

      virtual FB_USE_(short) Ima(void);
      virtual FB_USE_(Pchar) IAm(void);
      virtual FB_USE_(bool)  Isa(const short _type);

      virtual FB_USE_(Pvoid) GetPtr(void);
      virtual FB_USE_(void)  PutPtr(Pvoid _p);

   protected:
      Pvoid p;

   private:
      CPtrItem(void) {};
   }; // CPtrItem

#endif // _SLANGER

//=====================================================================
// List class declaration
//=====================================================================

enum AtWhere {atTop, before, after, atEnd, sorted, unique};

_FBDEF(CList)

class FBC_USE CList: public CItem {
   public:
      FB_USE CList(void);
      virtual FB_USE ~CList();

      virtual FB_USE_(short) Ima(void);
      virtual FB_USE_(Pchar) IAm(void);
      virtual FB_USE_(bool)  Isa(const short _type);

      // Copy this
//?   virtual FB_USE_(PCItem) Copy(long _begin = 0, long _end = 0);

      // Conditional operators
      virtual FB_USE_(bool) operator ==(PCItem _item);
      virtual FB_USE_(bool) operator !=(PCItem _item);

      // State report methods
      FB_USE_(bool)   Empty     (void);    // true if the list is empty
      FB_USE_(long)   Number    (void);      // number of items in list
      FB_USE_(long)   Length    (void);      // number of items in list
      FB_USE_(long)   Where     (void);         // index of active item
      FB_USE_(bool)   AtHead    (void);        // true if at first item
      FB_USE_(bool)   AtTail    (void);         // true if at last item

      // Position control methods
      FB_USE_(bool)   GotoItem  (long _item); // true if specified item
      FB_USE_(bool)   GotoHead  (void);             // true if any item
      FB_USE_(bool)   GotoTail  (void);             // true if any item
      FB_USE_(bool)   GotoNext  (void);            // true if next item
      FB_USE_(bool)   GotoPrev  (void);            // true if prev item

      FB_USE_(PCItem) GetHeadItem(void);
      FB_USE_(PCItem) GetTailItem(void);

#ifndef _SLANGER
      // Search methods
      FB_USE_(bool)   FindItem  (Pvoid _p);       // true if item found
      FB_USE_(bool)   FindFirst (Pvoid _p); // true if first item found
      FB_USE_(bool)   FindNext  (Pvoid _p);  // true if next item found
#endif // _SLANGER


      FB_USE_(long)   FindWhere (PCItem _item);    // position if found

      // Single item manipulator methods
      FB_USE_(bool)   Insert    (PCItem  _item,
                                 AtWhere _where    = atEnd);
      FB_USE_(PCItem) Remove    (AtWhere _where    = after,
                                 bool    _deleteIt = true);
                                                  // remove active item
      FB_USE_(PCItem) Item      (void);       // return pointer to item
      FB_USE_(PCItem) Item      (long _where);// return ptr to ith item
      FB_USE_(bool)   Replace   (PCItem  _item); // replace active item

      virtual FB_USE_(void) SetDirty(bool _dirty, depth _deep);
      virtual FB_USE_(bool) IsDirty (depth _deep);

      FB_USE_(bool)   PtrInsert (PCAtom _p, AtWhere _where = atEnd);

#ifndef _SLANGER
      FB_USE_(bool)   PtrInsert (Pvoid _p, AtWhere _where = atEnd);
      FB_USE_(Pvoid)  PtrItem   (void);
      FB_USE_(Pvoid)  PtrItem   (long _where);
      FB_USE_(bool)   PtrReplace(Pvoid _p);

      FB_USE_(void)   SetInfo   (Pvoid _info) { info = _info; }
      FB_USE_(Pvoid)  GetInfo   (void)        { return info; }

#endif // _SLANGER

      // Manipulator method
      virtual FB_USE_(void)   Flush     (void);    // discard all items in list

      FB_USE_(PCItem) GetGoing  (void) { return going; }

      // io methods
      virtual FB_USE_(ulong) BookSize (void);

      virtual FB_USE_(bool)  ReadFile   (PCAtom _input,  bool _isBook);
      virtual FB_USE_(bool)  WriteFile  (PCAtom _output, bool _isBook);

      virtual FB_USE_(bool)  ReadRemote (PCAtom _input);
      virtual FB_USE_(bool)  WriteRemote(PCAtom _output);

      virtual FB_USE_(bool)  Load       (Pchar _fileName);
      virtual FB_USE_(bool)  Save       (Pchar _fileName);

   protected:
      inline void exchange(PCItem _p, PCItem _q) {
         PCItem r = _p; _p = _q; _q = r;
         }
      void  insItem          (pCItem _p1, pCItem _p2, pCItem _item);
      void  rmvItem          (pCItem _item);

      // Per item io methods
      virtual bool  readItem (PCAtom _input,  bool _isBook);
      virtual bool  writeItem(PCAtom _output, bool _isBook);

      Pvoid  info;
      PCItem top,
             bot;
      pCItem cur;
      long   itemNr,
             nrItems;
      short  charNr;
      pCItem going;
   }; // CList

//---------------------------------------------------------------------
// true if the list is empty
//
inline bool CList::Empty(void) {
   return (nrItems == 0);
   } // Empty

#endif // LIST_H

