
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


//*********************************************************************
// misc.h
//*********************************************************************

#ifndef MISC_H
#define MISC_H

#ifndef FARBOOK_H
#include "atom.h"
#include "record.h"
#endif
enum FileType {anyFile, datFile, dirFile};

FBF_USE_(void) DateStr     (long  _year, long _month,  long _day,
                            Pchar _date,
                            short _size);
FBF_USE_(void) TimeStr     (long  _hour, long _minute, long _second,
                            Pchar _time,
                            short _size);

FBF_USE_(void) Encrypt     (Pchar _key, Pchar _text);
FBF_USE_(void) Decrypt     (Pchar _key, Pchar _text);

FBF_USE_(void) CutDrive    (Pchar _name);
FBF_USE_(void) GetDrive    (Pchar _drive, Pchar _name, short _size);
FBF_USE_(void) CutFilePath (Pchar _name);
FBF_USE_(void) GetFilePath (Pchar _path,  Pchar _name, short _size);
FBF_USE_(void) CutExtension(Pchar _name);
FBF_USE_(void) GetExtension(Pchar _name,  Pchar _ext, short _size);
FBF_USE_(void) CatPath     (Pchar _path,  Pchar _name, short _size);
FBF_USE_(void) CatName     (Pchar _path,  Pchar _name, Pchar _ext, short _size);
FBF_USE_(void) ParseName   (Pchar _path,  Pchar _name, Pchar _ext,
                            short _szPath, short _szExt);
FBF_USE_(void) MakeFileName(Pchar _name, short _size);

FBF_USE_(bool) MatchPattern(const Pchar _pat, const Pchar _data);

#if   defined(FV_MACINTOSH)
FBF_USE_(bool) InitSpec(   Pchar _path, FSSpec &_spec);
#endif // FV_MACINTOSH

FBF_USE_(void) GetCurrentDir(Pchar _path, int _size);

FBF_USE_(bool) ChangeDisk  (Pchar _name);
FBF_USE_(bool) MakeDir     (Pchar _name, Rulong _err);
FBF_USE_(bool) RemoveDir   (Pchar _name, Rulong _err);
FBF_USE_(bool) ChangeDir   (Pchar _name, Rulong _err);

FBF_USE_(bool) FindFileInPath(Pchar _name,
                              Pchar _path,
                              Pchar _fname,
                              short _size);
FBF_USE_(bool) FileExists  (Pchar _name, Rulong _err);

FBF_USE_(FileType)
               GetFileType (Pchar _fileName);

FBF_USE_(bool) GetFileInfo (Pchar _name,    PCRecord _data);
FBF_USE_(bool) RenameFile  (Pchar _oldName, Pchar _newName, Rulong _err);
FBF_USE_(bool) BackupFile  (Pchar _name, Pchar _backupExt, Rulong _err);
FBF_USE_(bool) RemoveFile  (Pchar _name, Rulong err);


const short SRF_NRFIELDS = 9;
const short SRF_NAME     = 1;
const short SRF_CRDATE   = 2;
const short SRF_CRTIME   = 3;
const short SRF_CRSECS   = 4;
const short SRF_MODDATE  = 5;
const short SRF_MODTIME  = 6;
const short SRF_MODSECS  = 7;
const short SRF_SIZE     = 8;
const short SRF_TYPE     = 9;

// _data format: |fileName
//               |crdate|crtime|crsecs
//               |moddate|modtime|modsecs
//               |size|dir/data
// (2003/11/24 pjm crtime seems to be unreliable?!)
//
// An object of this class is reusable since FindFirstName resets all
// members. 

_FBDEF(CSearchFile)
class FBC_USE CSearchFile: public CAtom {
   public:
      FB_USE CSearchFile(void);
      virtual FB_USE ~CSearchFile() {}

      virtual FB_USE_(short) Ima(void);
      virtual FB_USE_(Pchar) IAm(void);
      virtual FB_USE_(bool)  Isa(const short _type);

      FB_USE_(bool) FindFirstName(Pchar _fileName, FileType _fileType);
      FB_USE_(bool) FindNextName(void);
      FB_USE_(void) GetSearchData(PCRecord _data);
      FB_USE_(void) CloseFind(void);

   protected:
      FileType        isType;

#if   defined(FV_MSWINDOWS)
      DWORD           fm, ft;
      HANDLE          searchHandle;
      WIN32_FIND_DATA findData;

#elif defined(FV_MACINTOSH)
      FileType        fileType;
      CInfoPBRec      cipbr;
      Str255          fileName;
      char            pattern[sizeString];

#elif defined(FV_UNIX)
      DIR*            dir;
      FileType        fileType;
      dirent          entry;
      struct stat     buff;
      char            pattern[sizeString];

#endif // FV_UNIX
   }; // CSearchFile

#endif // MISC_H

