
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


//*********************************************************************
// odb.h
//*********************************************************************
//

#ifndef ODB_H
#define ODB_H

#include "findlist.h"
#ifndef FV_NOXML
#include "domtree.h"
#endif
#include "lclbook.h"
#include "refs.h"
#include "strlist.h"

enum ODB_LINKTYPE { ODB_NAME, ODB_PARENT, ODB_CHILD, ODB_REF, ODB_QUERY };

const ushort iODB = 1000;
_FBDEF(CODB)

class CODB: public CAtom {
   public:
      FB_USE CODB(Pchar DBName);
      virtual FB_USE ~CODB();

      virtual FB_USE_(short) Ima(void);
      virtual FB_USE_(Pchar) IAm(void);
      virtual FB_USE_(bool)  Isa(const short _type);

      // Set the domain for the session
      FB_USE_(void) SetDomain(Pchar dom);

      // Set sizes for a new ODB
      FB_USE_(void) SetSizes(int nrRefSlots, int nrHshSlots, int szKey);

      // Create a new ODB
      FB_USE_(bool) Create(long _nrParts,
                           long _nrRefSlots,
                           long _nrHshSlots,
                           long _sizeKey,
                           long _nrKeys);

      // Open the ODB
      FB_USE_(bool) Open(void);

      // Close the ODB
      FB_USE_(bool) Close(void);

      // Lock and unlock for atomic transactions 
      FB_USE_(bool) Lock(void);
      FB_USE_(bool) IsLocked(void) { return locked; }
      FB_USE_(bool) Unlock(void);

      // Check if named topic exists
      FB_USE_(bool) TopicExists(Pchar topicName);

      // Add a topic
      FB_USE_(bool) AddTopic(Pchar     topic, Pchar topicName);
      FB_USE_(bool) AddTopic(PCStrList topic, Pchar topicName);
      FB_USE_(bool) AddTopic(PCBlock   topic, Pchar topicName);
#ifndef FV_NOXML
      FB_USE_(bool) AddTopic(PCNode  topic, Pchar topicName);
#endif

      // Add a reference for the specified topic
      FB_USE_(bool) AddRef(Pchar topicName, Pchar reference);

      // Add a set of references for the specified topic
      FB_USE_(bool) AddRefs(Pchar topicName, PCStrList referenceList);

      // Establish a relationship between a parent and a child
      FB_USE_(bool) AddLink(Pchar parentName, Pchar childName);

      // Establish a relationship between a parent and a set of children
      FB_USE_(bool) AddLinks(Pchar parentName, PCStrList childNameList);

      // Remove a reference for the specified topic
      FB_USE_(bool) RemoveRef(Pchar topicName, Pchar reference);

      // Remove a set of references for the specified topic
      FB_USE_(bool) RemoveRefs(Pchar topicName, PCStrList referenceList);

      // Update a topic
      FB_USE_(bool) UpdateTopic(Pchar    topic, Pchar topicName);
      FB_USE_(bool) UpdateTopic(PCBlock topic, Pchar topicName);
#ifndef FV_NOXML
      FB_USE_(bool) UpdateTopic(PCNode  topic, Pchar topicName);
#endif

      // Remove a topic 
      FB_USE_(bool) RemoveTopic(Pchar topicName);

      // Get the named topic
      FB_USE_(PCBlock) GetTopic(Pchar topicName);

      // Obtain a set of topics given a symbol or expression
      FB_USE_(bool) Query(Pchar symbol, ODB_LINKTYPE type);

      // Count the number of topics in the result set
      FB_USE_(long) Length(void);

      // Obtain the specified topic from the result set (0..len-1)
      FB_USE_(PCBlock) Topic(long index);

   private:
      CODB(void) {}

      bool  keyPresent(Pchar topicName);
      PCRef getRef    (Pchar topicName);
      bool  refExists (PCRef ref, Pchar key);
      bool  find      (Pchar query, char mark, Pchar type);

      Pchar      domain;
      PCBook     book;
      bool       locked;
      PCFindList result;
      long       error;
   }; // CODB

#endif // ODB_H

