

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//*********************************************************************
// prefixb.h
//*********************************************************************
//
// There are three symbols that you use to control the compilation to
// selected code for the supported platforms:
//
//    FV_MSWINDOWS
//    FV_MACINTOSH
//    FV_UNIX
//
// When using the FV_MSWINDOWS symbol, you must also choose one of the
// following two symbols:
//
//    FV_MSWIN16
//    FV_MSWIN32
//
// The following modules contain code that is dependent on the
// platform.
//
//    ARGS.CPP
//    BLOCK.CPP
//    COMMON.CPP
//    CONSOLE.CPP
//    FILE.CPP
//    LAUNCH.CPP
//    MISC.CPP
//
// Whenever there is an API that requires different code for different
// platforms, the practice in this library is to provide a structure
// like the following:
//
// #if   defined(FV_MSWINDOWS)
//    (Windows code goes here)
// #elif defined(FV_MACINTOSH)
//    (Macintosh code goes here)
// #elif defined(FV_UNIX)
//    (Unix/Linux code goes here)
// #endif // FV_UNIX
//
// then, to provide the code appropriate to each platform within that
// structure.
//
//*********************************************************************

#ifndef FB_PREFIX_H
#define FB_PREFIX_H

#include <assert.h>
#include <ctype.h>
#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#define __DEBUGGING

#define FV_MSWINDOWS
#define FV_MSWIN32
#define FV_NOREMOTE
#define FV_NOTHREAD
#define FV_NOFACTORY
#define FV_NOXML
#define FV_NOSTEM
#define FV_NOPOINTS

#ifdef _COMPILEDLLS
#define FB_EXPORT
#endif

//---------------------------------------------------------------------
// MSWindows
//
#if   defined(FV_MSWINDOWS)

   #if   defined(__BORLANDC__)
      #include <dir.h>
   #elif defined(_MSC_VER)
      #include <direct.h>
   #endif
   #include <io.h>
   #include <windows.h>
   #include <mmsystem.h>

   #ifndef _SLANGER
      typedef POINT Point;
      typedef RECT  Rect;
   #endif // _SLANGER

   #if   defined(FB_IMPORT)
      #if   defined(__BORLANDC__)
         #define FB_USE         __declspec(dllimport)
         #define FBC_USE        __declspec(dllimport)
         #define FB_USE_(type)  __declspec(dllimport) type
         #define FBF_USE_(type) __declspec(dllimport) type
      #elif defined(_MSC_VER)
         #define FB_USE
         #define FBC_USE        __declspec(dllimport)
         #define FB_USE_(type)  type
         #define FBF_USE_(type) __declspec(dllimport) type
      #else
         #define FB_USE
         #define FBC_USE
         #define FB_USE_(type)  type
         #define FBF_USE_(type) type
      #endif

   #elif defined(FB_EXPORT)
      #if   defined(__BORLANDC__)
         #define FB_USE         __declspec(dllexport)
         #define FBC_USE        __declspec(dllexport)
         #define FB_USE_(type)  __declspec(dllexport) type
         #define FBF_USE_(type) __declspec(dllexport) type
      #elif defined(_MSC_VER)
         #define FB_USE
         #define FBC_USE        __declspec(dllexport)
         #define FB_USE_(type)  type
         #define FBF_USE_(type) __declspec(dllexport) type
      #else
         #define FB_USE
         #define FBC_USE
         #define FB_USE_(type)  type
         #define FBF_USE_(type) type
     #endif

   #else
      #define FB_USE
      #define FBC_USE
      #define FB_USE_(type)  type
      #define FBF_USE_(type) type
   #endif

//---------------------------------------------------------------------
// Macintosh
//
#elif defined(FV_MACINTOSH)
   #include <macheaders.pch++>
   #include <aiff.h>
   #include <console.h>
   #include <files.h>

   #if   defined(FB_EXPORT)
      #define FB_USE        __declspec(export)
      #define FB_USE_(type) __declspec(export) type

   #elif defined(FB_IMPORT)
      #define FB_USE        __declspec(import)
      #define FB_USE_(type) __declspec(import) type

   #else
      #warning FB_EXPORT or FB_IMPORT must be  defined
   #endif

   #define HGLOBAL unsigned long
   #define ltoa(n, s, r) { NumToString(n, (unsigned char*)s); }

//---------------------------------------------------------------------
// Unix/Linux
//
#elif defined(FV_UNIX)
   #include <sys/types.h>
   #include <sys/stat.h>
   #include <dirent.h>
   #include <fcntl.h>
   #include <unistd.h>

   #if   defined(FB_EXPORT)
      #define FB_USE
      #define FBC_USE 
      #define FB_USE_(type) type
      #define FBF_USE_(type) type

   #elif defined(FB_IMPORT)
      #define FB_USE
      #define FBC_USE 
      #define FB_USE_(type) type
      #define FBF_USE_(type) type

   #else
      #warning FB_EXPORT or FB_IMPORT must be  defined
   #endif

   #define DWORD   unsigned long
   #define HANDLE  unsigned long
   #define HGLOBAL unsigned long

#define stricmp strcasecmp
#define strnicmp strncasecmp

#else
   #error FV_MSWINDOWS, FV_MACINTOSH, or FV_UNIX must be defined

#endif // FV_UNIX

//---------------------------------------------------------------------
//
#ifndef true
#define true 1
#endif
#ifndef false
#define false 0
#endif

//---------------------------------------------------------------------
//
const int  sizeString        =  8192;
const int  sizeName          =    64;
const int  sizeFileName      =   256;
const int  sizeLine          =   512;
const int  sizeWord          =    32;
const int  sizeDateTime      =    64;
const long maxBlockSize      = 64000; // 32766;

//---------------------------------------------------------------------
//
typedef unsigned char  uchar;
typedef unsigned short ushort;
typedef unsigned long  ulong;
typedef unsigned int   uint;
typedef unsigned char  byte;
typedef ushort         word;

typedef void*          Pvoid;
typedef bool*          Pbool;
typedef bool&          Rbool;
typedef char*          Pchar;
typedef char&          Rchar;
typedef char*&         RPchar;
typedef uchar*         Puchar;
typedef uchar&         Ruchar;        
typedef uchar*&        RPuchar;
typedef short*         Pshort;
typedef short&         Rshort;
typedef ushort&        Rushort;
typedef int*           Pint;
typedef int&           Rint;
typedef long*          Plong;
typedef long&          Rlong;
typedef ulong&         Rulong;
typedef ulong*         Pulong;
typedef double*        Pdouble;         

typedef byte           Int8;
typedef short          Int16;
typedef long           Int32;

typedef ulong          FB_HANDLE;

enum depth {shallow, deep};

//---------------------------------------------------------------------
//
#if defined(FV_MSWINDOWS)

#define drivDelim   ":"
#define pathDelim   "\\"
#define recordDelim ";"
#define extDelim    "."

#elif defined(FV_UNIX)

#define drivDelim   ":"
#define pathDelim   "/"
#define recordDelim ";"
#define extDelim    "."

#endif

//---------------------------------------------------------------------
// Search constants
//
//const char evalBeg = '{';
//const char evalEnd = '}';
const char strQuote1 = '"';
const char strQuote2 = '\'';
const char strQuote3 = '`';
const char spacer    = ' ';

//---------------------------------------------------------------------
// Macros to declare types commonly associated with a class
//
#define _FBDEF(name) \
   class FBC_USE name; \
   typedef TPtr<name> P##name; \
   typedef P##name & RP##name; \
   typedef name * p##name; \
   typedef name & R##name; \
   typedef name const & RC##name;

#define __FBDEF(name) \
   class FBC_USE name; \
   typedef name * p##name; \
   typedef name & R##name; \
   typedef name const & RC##name;

//---------------------------------------------------------------------
// Macros to support portable run-time type information
//

// Use this macro in the declaration of each class that is derived
// from CAtom
//
#define RTTI_HB \
   virtual FB_USE_(short) Ima(void); \
   virtual FB_USE_(Pchar) IAm(void); \
   virtual FB_USE_(bool)  Isa(const short _type);

// Use this macro in the implementation of each class that is derived
// from CAtom
//
#define RTTI_CPP(type, base) \
   short C##type::Ima(void) { return i##type; } \
   Pchar C##type::IAm(void) { return Pchar("C"#type""); } \
   bool  C##type::Isa(const short _type) { \
      if (_type == i##type) \
         return true; \
      else \
         return C##base::Isa(_type); \
      }

// Use this macro to typecast a smart reference of a CAtom object
//
#define TYPECAST(p, type) \
   (p ? \
      p->Isa(i##type) ?  \
         pC##type(p.getPtr()) : \
         pC##type(fvAssert(Pchar(#type), Pchar(__FILE__), __LINE__)) : \
      pC##type(fvAssert(Pchar(#type), Pchar(__FILE__), __LINE__)))

//---------------------------------------------------------------------
// Macro to (re)initialize a string (Pchar) variable using the contents
// of another string variable. Be sure that p1 is zero if it doesn't
// point to anything.
//

#define STRINIT(p1, p2) \
   { delete[] p1; \
     if (!StrEmpty(p2)) { \
        p1 = new char[strlen(p2)+1]; \
        strcpy(p1, p2); \
        } \
     else  { \
        p1 = new char[1]; \
        p1[0] = 0; \
        } \
     }
//---------------------------------------------------------------------
// Macro to combine two 16 bit values into a single 32 bit value with
// p1 to occupy the upper half and p2 to occupy the lower half.
//

#define MYMAKELONG(p1, p2) \
   (((p1 & 0xffff) << 16) | (p2 & 0xffff))

//---------------------------------------------------------------------
// Macros to assist debugging
//
#define Assert(exp) ((exp) ? Pvoid(0) : fvAssert(Pchar(#exp), Pchar(__FILE__), __LINE__))

FBF_USE_(Pvoid) fvAssert(Pchar _exp, Pchar _file, int _line);

#define Strcpy(to, from, size) \
     { \
     Assert(strlen((const Pchar)from)<size_t(size)); \
     strcpy(to,(const Pchar)from); \
     }
#define StrNcpy(to, from, size) \
     { \
     memset(to,0,size_t(size)); \
     strncpy(to,(const Pchar)from,size_t(size-1)); \
     }

#define Strcat(to, from, size) \
     { \
     Assert(strlen((const Pchar)to)+strlen((const Pchar)from)<size_t(size)); \
     strcat(to,(const Pchar)from); \
     }

//---------------------------------------------------------------------
// Template to support reference counting (smart pointers) with
// automatic object deletion.
//
template <class T> class TPtr {
   public:
      T *getPtr() const {
         return p;
         }

      TPtr():
         p(0) {
         }

      TPtr(T *_p):
         p(0) {
         setPtr(_p);
         }

      TPtr(const TPtr<T> &_p):
         p(0) {
         setPtr(_p.getPtr());
         }

      ~TPtr() {
         zeroPtr();
         }

      void Release(void) {
         zeroPtr();
         }

      TPtr<T>& operator= (const TPtr<T> &_p) {
         setPtr(_p.getPtr());
         return *this;
         }

      TPtr<T>& operator= (T *_p) {
         setPtr(_p);
         return *this;
         }

      TPtr<T>& operator= (int _i) {
         Assert(!_i);
         setPtr((T*)_i);
         return *this;
         }

      int operator== (const TPtr<T> &_a) const {
         return p ==_a.p;
         }
      int operator!= (const TPtr<T> &_a) const {
         return p != _a.p;
         }

      T& operator* () const {
         return (*p);
         }

      T* operator-> () const {
         return p;
         }

      operator T* () const {
         return p;
         }

      bool operator! () const {
         return p == 0;
         }

   protected:
      T *p;

      void zeroPtr() {
         if (p) {
            if (p->DrgRef() == 0)
               delete p;
            p = 0;
            }
         }

      void setPtr(T *_p) {
         if (p != _p) {
            if (p)
               zeroPtr();
            p = _p;
            if (p)
               p->RegRef();
            }
         }
   }; // TPtr

#endif // FB_PREFIX_H

