
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//*********************************************************************
// process.cpp
//*********************************************************************
//
//   ************ IMPORTANT NOTE FOR MACINTOSH USAGE **************
// You must enable the following SIZE flags for this class to work
// properly:
//       "Background Null Events"
//       "Suspend and Resume Events"
//       "Accept ChildDiedEvents"
//
//       "HighLevelEvent - Aware"
//       "Accept Remote HigLevelEvents"
//
// Without these, the system may hang, or the parent won't know when
// the child process has terminated. (thereby never coming to the
// foreground).
//*********************************************************************

#include "prefixb.h"
#include "common.h"
#include "file.h"
#include "process.h"
#include "misc.h"

#ifdef FV_MSWINDOWS
/*
const int MAX_WEXECERROR_LEN = 63;
const char WinExecError[][MAX_WEXECERROR_LEN] = {
   {"System out of memory, exe file corrupt or relocations invalid"},//0
   {"Unknown Error 1"                                            }, //1
   {"File was not found"                                         }, //2
   {"Path not found"                                             }, //3
   {"Unknown Error 4"                                            }, //4
   {"Attempt to dynamically link to task or network error"       }, //5
   {"Library required separate data segments for each task"      }, //6
   {"Unknown error 7"                                            }, //7
   {"Not enough memory to start the application"                 }, //8
   {"Unknown Error 9"                                            }, //9
   {"Incorrect Windows version"                                 }, //10
   {"Exe file invalid"                                          }, //11
   {"Application designed for different OS"                     }, //12
   {"Application designed for MS-DOS v4.0"                      }, //13
   {"Type of exe file unknown"                                  }, //14
   {"Attempt to load real mode application: obtain updated exe" }, //15
   {"Attempt to load application already running"               }, //16
   {"Unknown Error 17"                                          }, //17
   {"Unknown Error 18"                                          }, //18
   {"Attempt to load compressed exe file"                       }, //19
   {"DLL file required to run application is invalid"           }, //20
   {"Application requires Microsoft Windows 32-bit extensions"  }, //21
   {"Return code out of range (0..21)"                  }, //outofrange
   {""}                                                       // ending
   };
*/
#endif

//=====================================================================
// Code common to all platforms
//=====================================================================

Pchar CProcess::name = 0;

//---------------------------------------------------------------------
// Constructor
//
CProcess::CProcess(void):
   CAtom    (),
   amWaiting(false) {
   error = 0;
   } // CProcess

//--------------------------------------------------------------------
// Destructor
//
CProcess::~CProcess() {
   delete[] name;
   name = 0;
   } // ~CProcess

//---------------------------------------------------------------------
//
RTTI_CPP(Process, Atom)

//====================================================================
// Windows specific implementation
//====================================================================

#if defined(FV_MSWINDOWS)

//---------------------------------------------------------------------
// Given a file name, determine the program that should handle it
// Note: user must be sure to delete the returned string if not zero.
//
Pchar CProcess::FindExecutable(Pchar _name) {
   char name[sizeString];
   Strcpy(name, _name, sizeString);

   CutFilePath(name);
   char ext[sizeString];
   GetExtension(name, ext, sizeString);

   // Create a temp file with the specified extension
   if (!StrEmpty(ext)) {
      name[0] = '$';
      name[1] = '.';
      name[2] = 0;
      Strcat(name, ext, sizeString);
      if (name[strlen(name)-1] == '"')
         name[strlen(name)-1] = 0;
      ulong err;
      if (!FileExists(name, err))
         CFile(name).Create();

      // Get the name of the executable from the shell
      if (int(::FindExecutable(name, "", ext)) > 32) {
         Pchar exec = 0;
         STRINIT(exec, ext)
         return exec;
         }
      }

   return 0;
   } // FindExecutable

//---------------------------------------------------------------------
// Test that the child process has completed.
//
bool CProcess::Done(void) {
   Assert(amWaiting);

#if   defined(FV_MSWIN32)
   DWORD exitCode;
   ::WaitForSingleObject(procInfo.hProcess, 50);
   ::GetExitCodeProcess(procInfo.hProcess, &exitCode);
   if (exitCode != STILL_ACTIVE) {
      amWaiting = false;
      return true;
      }
   else
      return false;

#elif defined(FV_MSWIN16)
   Assert(hChild);
   if (::GetModuleUsage(HINSTANCE(hChild)) == 0) {
      amWaiting = false;
      hChild    = 0;
      return true;
      }
   else
      return false;
#endif // FV_MSWIN16
   } // Done

//---------------------------------------------------------------------
// Process program specified in _commandLine parameter
//
bool CProcess::Run(Pchar _cmmd, Pchar _parms, bool _wait) {
   sysErr = 0;

   // They cannot both be empty
   Assert(!StrEmpty(_cmmd) || !StrEmpty(_parms));

   // Piece together a full command line 
   char cmmd[sizeString];
   if      (StrEmpty(_cmmd)) {
      Strcpy(cmmd, _parms, sizeString);
      }
   else if (StrEmpty(_parms)) {
      Strcpy(cmmd, _cmmd, sizeString);
      }
   else {
      Strcpy(cmmd, _cmmd, sizeString);
      ChrCat(cmmd, ' ', sizeString);
      Strcat(cmmd, _parms, sizeString);
      }


#if   defined(FV_MSWIN32)
   char mydir[256];
   ::GetCurrentDirectory(256, mydir);

   STARTUPINFO startUpInfo;
   memset(&startUpInfo, 0, sizeof(STARTUPINFO));
   memset(&procInfo, 0, sizeof(PROCESS_INFORMATION));
   if (::CreateProcess(0,
                       cmmd,
                       0,
                       0,
                       false,
                       CREATE_DEFAULT_ERROR_MODE,
                       0,
                       mydir,
                       &startUpInfo,
                       &procInfo)) {

      amWaiting = true;

      // Wait until child process terminates
      if (_wait) {          // See WIN32 Programming API Bible pg 1208
         ::WaitForInputIdle(::GetCurrentProcess(), INFINITE);
         if (procInfo.hProcess) {
            DWORD exitCode = STILL_ACTIVE;
            while (exitCode == STILL_ACTIVE) {
               ::WaitForSingleObject(procInfo.hProcess, 1000);
               ::GetExitCodeProcess (procInfo.hProcess, &exitCode);
               }
            }
         amWaiting = false;
         }
      return true;
      }
   else {
      sysErr = long(::GetLastError());
      return false;
      }

#elif defined(FV_MSWIN16)
   // Call WinExec to launch program. (Windows returns a
   // value less than 32 if error, otherwise it returns
   // the window handle)
   hChild = ::WinExec(cmmd, SW_SHOWNORMAL);

   // Error checks
   if (hChild >= 32) {                       // < 32 indicates an error
      MSG msg;

      // Wait until the child process begins
      while (::GetModuleUsage(HINSTANCE(hChild)) == 0) {
         if (::PeekMessage(&msg, 0, 0, 0, PM_REMOVE))
            if (msg.message == WM_QUIT)
               break;
            else {
               ::TranslateMessage(&msg);
               ::DispatchMessage(&msg);
               }
         }

       // Wait until the child process completes
       amWaiting = true;
       if (_wait) {
          while (::GetModuleUsage(HINSTANCE(hChild)) > 0) {
             if (::PeekMessage(&msg, 0, 0, 0, PM_REMOVE))
                if (msg.message == WM_QUIT)
                   break;
                else {
                   ::TranslateMessage(&msg);
                   ::DispatchMessage(&msg);
                   }
             }
          amWaiting = false;
          hChild    = 0;
          }
      return true;
      }
   else {
      amWaiting = false;
      hChild    = 0;
      return false;
      }

#endif // FV_MSWIN16
   } // Run

//---------------------------------------------------------------------
//
bool CProcess::Run(Pchar _fileName) {
   ShellExecute(0, "open", _fileName, NULL, NULL, SW_SHOWNORMAL);

   return true;
   } // Run

//---------------------------------------------------------------------
// Does nothing in the Windows version
//
void CProcess::DeleteParmsFile(void) {
   } // DeleteParmsFile

//=====================================================================
// Macintosh specific code.
//=====================================================================

#elif defined(FV_MACINTOSH)

//---------------------------------------------------------------------
// Test that the child process has completed.
// (To use this function, user should call ProcessChild with the
// _wait parameter set to false.)
//
bool CProcess::IsChildDone(void) {
   Assert(false);
   // TBD
   } // IsChildDone

//---------------------------------------------------------------------
// Process the program specified in the _commandLine parameter
//
//
bool CProcess::Run(Pchar _cmmd, Pchar _parms, bool _wait) {
   // Declare variables
   AppleEvent          theODocEvent;
   ProcessParamBlockRec theApp;
   AEDesc              parmDesc;
   long                prefSize;
   long                minSize;
   long                availSize;

	availSize;

   // Initialize
   theApp.launchAppParameters = nil;
   theODocEvent.dataHandle    = nil;
   parmDesc.dataHandle        = nil;

   // Protect the innocent
   Pchar cmmd  = new char[strlen(_cmmd)+1];
   strcpy(cmmd, _cmmd);

   Pchar parms = new char[strlen(_parms)+1];
   strcpy(parms, _parms);

   // Change the command to a valid Pascal string
   ::CtoPstr(cmmd);

   // Set up a file spec for the program to be launched
   FSSpec appFile;
   FSSpec parmsFile[1];
   parmsFile;
   sysErr = long(::FSMakeFSSpec(0, 0, (unsigned char*)cmmd, &appFile));
   if (OSErr(sysErr) != noErr)
      goto clean;

   // Pass the parameters to the child program
   // (writes a $$$parms.sfc file to the temporary
   // items folder)
   if (!writeParmsFile(parms))
      goto clean;

   // Fill out the launchParamBlockRec
   theApp.launchBlockID      = extendedBlock;
   theApp.launchEPBLength    = extendedBlockLen;
   theApp.launchFileFlags    = launchNoFileFlags;
   theApp.launchControlFlags = launchContinue + launchNoFileFlags;
   theApp.launchAppSpec      = &appFile;

/*?
   // This is where we indicate which Apple Event gets sent
   // to the app first.
   if (_tempDir) {
      // Coerce the AppleEvent into app parameters for ProcessApp
      sysErr = long(::AECoerceDesc(&theODocEvent,
                                   typeAppParameters,
                                   &parmDesc));
      if (OSErr(sysErr) != noErr)
         goto clean;

      // Fill the launch parameter block
      ::HLock(Handle(parmDesc.dataHandle));
      theApp.launchAppParameters =
         AppParametersPtr(*(parmDesc.dataHandle));
      }
   else
?*/
   // To send an OpenApplication event to the child
   theApp.launchAppParameters = 0L;

   // Process the application and check returned values
   sysErr   = long(::ProcessApplication(&theApp));
   prefSize = theApp.launchPreferredSize;
   minSize  = theApp.launchMinimumSize;

   if (sysErr != noErr)
      goto clean;

   // Wait for the child process to complete
   if (_wait) {
      waitForTerm();
      DeleteParmsFile();
      }

clean:
   delete cmmd;
   delete parms;

   if (theODocEvent.dataHandle != nil)
      (void)::AEDisposeDesc(&theODocEvent);
   if (parmDesc.dataHandle != nil)
      (void)::AEDisposeDesc(&parmDesc);

   short shortval = OSErr(sysErr);
   short retval;
   if (sysErr != noErr)
      retval = false;
   else
      retval = true;

   if (OSErr(sysErr) != noErr)
      return false;
   else
      return true;
   } // ProcessChild

//---------------------------------------------------------------------
// Ccreate a file and put the parameter string in it
//
bool CProcess::writeParmsFile(Pchar _parms) {
   // Write the parameters into a file named "$$$PARMS.SFC" stored
   // into the TemporaryItems folder on the boot disk.
   if (name == 0) {
      name = new char[sizeString];
      if (!getParmsName())
         return false;
      }

   // Open a file object for output
   CFile parms(name);
   parms.OpenOutput();
   if (parms.Error() != SFCE_NOPROBLEM) {
      sysErr = parms.SystemError();
      return false;
      }

   // Write the parameter string as the first
   // line of the (text) file
   if (_parms)
    	parms.WriteString(_parms);
   else
    	parms.WriteString("");

   // Write the signature of this program as the
   // second line of the (text) file

   // Get the signature from the system

   // Get the process number first
   ProcessSerialNumber myPSN;
   sysErr = long(::GetCurrentProcess(&myPSN));
   if (OSErr(sysErr) != noErr)
      return false;

   // Then use it to get the process information block...
   ProcessInfoRec info;
   memset(&info, 0, sizeof(ProcessInfoRec));
   info.processInfoLength = sizeof(ProcessInfoRec);
   sysErr = long(::GetProcessInformation(&myPSN, &info));
   if (OSErr(sysErr) != noErr)
      return false;

   // ...which contains the signature
   OSType signature = info.processSignature;

   char work[sizeString];
   sprintf(work, "0x%08xl", signature);
   Assert(strlen(work) < sizeString);
   parms.WriteString(work);

   // Thats it!
   parms.Close();

   return true;
   } // writeParmsFile

//---------------------------------------------------------------------
// Construct the path name for the parameter file.
//
bool CProcess::getParmsName(void) {
   if (!GetTempDir(name, sizeString))
      return false;
   else {
      Strcat(name, "$$$PARMS.SFC", sizeString);
      return true;
      }
   } // getParmsName

//---------------------------------------------------------------------
// Delete the parameter file in the temporary items folder.
//
void CProcess::DeleteParmsFile(void) {
   Assert(name);
   if (strlen(name))
      RemoveFile(name);
   memset(name, 0, sizeString);
   } // DeleteParmsFile

//---------------------------------------------------------------------
// Wait for spawned app to exit
//
void CProcess::waitForTerm(void) {
   // go into a loop until we get a kAEApplicationDied event
   while (true) {
      // set up AEApplicationDied apple event
      EventRecord eventRec;
      if (::WaitNextEvent(everyEvent, &eventRec, 0L, nil))
         if ((eventRec.what == kHighLevelEvent) &&
             ((*(long*)(&eventRec.where)) == kAEApplicationDied))
            break;
         else
            ::AEProcessAppleEvent(&eventRec);
      }
   return;
   } // waitForTerm

#elif defined(FV_UNIX)

//---------------------------------------------------------------------
// Given a file name, determine the program that should handle it
// Note: user must be sure to delete the returned string if not zero.
//
Pchar CProcess::FindExecutable(Pchar _name) {
   #warning FV_UNIX TBD (FindExecutable)

   char name[sizeString];
   Strcpy(name, _name, sizeString);

   CutFilePath(name);
   char ext[sizeString];
   GetExtension(name, ext, sizeString);

   // TBD

   return 0;
   } // FindExecutable

//---------------------------------------------------------------------
// Test that the child process has completed.
//
bool CProcess::Done(void) {
   #warning FV_UNIX TBD (Done)

   Assert(amWaiting);
   
   // TBD

   } // Done

//---------------------------------------------------------------------
// Process program specified in _commandLine parameter
//
bool CProcess::Run(Pchar _cmmd, Pchar _parms, bool _wait) {
   #warning FV_UNIX TBD (Run)
   sysErr = 0;

   // They cannot both be empty
   Assert(!StrEmpty(_cmmd) || !StrEmpty(_parms));

   // Piece together a full command line 
   char cmmd[sizeString];
   if      (StrEmpty(_cmmd)) {
      Strcpy(cmmd, _parms, sizeString);
      }
   else if (StrEmpty(_parms)) {
      Strcpy(cmmd, _cmmd, sizeString);
      }
   else {
      Strcpy(cmmd, _cmmd, sizeString);
      ChrCat(cmmd, ' ', sizeString);
      Strcat(cmmd, _parms, sizeString);
      }

   // TBD

   return false;
   } // Run

//---------------------------------------------------------------------
//
bool CProcess::Run(Pchar _fileName) {
   #warning FV_UNIX TBD (Run)
   // TBD

   return true;
   } // Run

//---------------------------------------------------------------------
// Does nothing in the Unix version
//
void CProcess::DeleteParmsFile(void) {
   } // DeleteParmsFile

#endif // FV_UNIX

