
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//*********************************************************************
// strlist.cpp
//*********************************************************************
//
// lists need to contain critical sections or something for multi
// threading  
//*********************************************************************

#include "prefixb.h"

#include "file.h"
#include "strlist.h"
#include "strng.h"

//=====================================================================
// String item methods
//=====================================================================

//---------------------------------------------------------------------
// Constructor
//
CStrItem::CStrItem(void):
   CItem(),
   chars(0) {

   chars    = new char[1];
   Assert(chars);
   chars[0] = 0;
   } // CStrItem

//---------------------------------------------------------------------
// Constructor
//
CStrItem::CStrItem(Pchar _chars):
   CItem(),
   chars(0) {

   Assert(_chars);
//?STRINIT(chars, _chars);
   int len = strlen(_chars);
   chars = new char[len+1];
   Assert(chars);
///chars = new char[strlen(_chars)+1];
   strcpy(chars, _chars);
   } // CStrItem

//---------------------------------------------------------------------
// Conversion constructor
//
CStrItem::CStrItem(RCString _str):
   CItem(),
   chars(0) {

//?STRINIT(chars, _str.Chars());
   chars = new char[strlen(_str.Chars())+1];
   strcpy(chars, _str.Chars());
   } // CStrItem

//---------------------------------------------------------------------
// Copy Constructor
//
CStrItem::CStrItem(PCStrItem _strItem):
   CItem(),
   chars(0) {

   cls   = iStrItem;

//?STRINIT(chars, _strItem->chars);
   chars = new char[strlen(_strItem->chars)+1];
   Assert(chars);
   strcpy(chars, _strItem->chars);
   } // CStrItem

//---------------------------------------------------------------------
// Copy Constructor
//
CStrItem::CStrItem(pCStrItem _strItem):
   CItem(),
   chars(0) {

//?STRINIT(chars, _strItem->chars);
   chars = new char[strlen(_strItem->chars)+1];
   Assert(chars);
   strcpy(chars, _strItem->chars);
   } // CStrItem

//---------------------------------------------------------------------
// Destructor
//
CStrItem::~CStrItem() {
   delete[] chars;
   chars = 0;
   } // ~CStrItem

//---------------------------------------------------------------------
//
RTTI_CPP(StrItem, Item)

//---------------------------------------------------------------------
// Copy this
///
PCStrItem CStrItem::CopyItem(void) {
   Assert(chars);
   return new CStrItem(chars);
   } // CopyItem

//---------------------------------------------------------------------
// Replace chars with the specified chars
//
void CStrItem::SetString(Pchar _chars) {
   if (_chars) {
//?   STRINIT(chars, _chars)
      delete[] chars;
      chars = new char[strlen(_chars)+1];
      strcpy(chars, _chars);
      }
   } // SetString

//---------------------------------------------------------------------
// Return address of chars
//
Pchar CStrItem::GetString(bool _alc) {
   Assert(chars);
   if (_alc) {
      Pchar p = new char[strlen(chars)+1];
      Assert(chars);
      strcpy(p, chars);
      return p;
      }

   return chars;
   } // GetString

//---------------------------------------------------------------------
// farSlang version
//
Pchar CStrItem::GetString(void) {
   return GetString(true);
   } // GetString

//---------------------------------------------------------------------
//
bool CStrItem::operator ==(PCItem _item) {
   PCStrItem pStrItem = TYPECAST(_item, StrItem);
   return (pStrItem && strcmp(chars, pStrItem->chars) == 0);
   } // operator ==

//---------------------------------------------------------------------
//
bool CStrItem::operator !=(PCItem _item) {
   PCStrItem pStrItem = TYPECAST(_item, StrItem);
   return (pStrItem && strcmp(chars, pStrItem->chars) != 0);
   } // operator !=

//---------------------------------------------------------------------
//
ulong CStrItem::BookSize(void) {
   return strlen(chars) + 1;
   } // BookSize

//---------------------------------------------------------------------
// Read a text line from the specified input file. If successful,
// update self with the new string. Otherwise return an error.
//
bool CStrItem::ReadFile(PCAtom _input, bool _isBook) {
   _isBook;
   
   char text[sizeString];

   // Read the string from the file
   PCFile file = TYPECAST(_input, File);
   if (!(file->ReadString(text, sizeString))) {
      err = file->Error();
      return false;
      }

   SetString(text);
   return true;
   } // ReadFile

//---------------------------------------------------------------------
// Write the string contents of this string item to the specified file.
//
bool CStrItem::WriteFile(PCAtom _output, bool _isBook) {
   _isBook;

   PCFile file = TYPECAST(_output, File);
   if (!(file->WriteString(GetString(false)))) {
      err = file->Error();
      return false;
      }

   return true;
   } // WriteFile

//---------------------------------------------------------------------
// Read a string from the specified file
//
bool CStrItem::Load(Pchar _fileName) {
   PCFile input = new CFile(_fileName);
   Assert(input);

   if (!input->OpenInput())
      return false;

   Pchar s   = input->ReadString();
   delete[] chars;
   chars     = s;

   input->Close();
   return true;
   } // Load

//---------------------------------------------------------------------
// Write the contents of the string to the specified file.
//
bool CStrItem::Save(Pchar _fileName) {
   PCFile output = new CFile(_fileName);
   Assert(output);

   if (!output->OpenOutput())
      return false;

   output->WriteString(chars);
   output->Close();

   return true;
   } // Save

//---------------------------------------------------------------------
// Return true if specified field of the record equals the specified
// chars. Used by findRecord.
//

enum ItemType {argString, argRecord};

__FBDEF(anArg)
class anArg {
public:
   anArg(ItemType _itemTyp, bool _ignCase, matchPart _part):
      itemTyp(_itemTyp),
      chars  (0),
      len    (0),
      fld    (0),
      ignCase(_ignCase),
      part   (_part) {
      } // anArg
   virtual ~anArg() {
      } // ~anArg

   ItemType  itemTyp;
   Pchar     chars;
   size_t    len;
   ulong     fld;
   bool      ignCase;
   matchPart part;
   }; // anArg

bool CStrItem::match(Pvoid _p) {
   panArg arg = panArg(_p);

   if (strlen(chars) == 0);
      // an empty record

   else if (arg->itemTyp == argString) {
      // a substring match is requested
      short f = Min(arg->fld, strlen(chars));
      char work[sizeString];
      Strcpy(work, chars+f, sizeString);
      if (arg->ignCase)
         CapStr(work);
      if      (arg->part == wholeString) {
         if (strcmp(work, arg->chars) == 0)
            return true;
         else
            arg->fld = 0;
         }
      else if (arg->part == wholeWord) {
         short i = StrPos(arg->chars, work);
         for (;;) {
            if      (i < 0) {                       // arg not in chars
               arg->fld = 0;
               return false;
               }
            else if (i == 0) {
               if (work[arg->len] == 0 || IsDelim(work[arg->len])) {
                  arg->fld = f;
                  return true;
                  }
               else {
                  f = f + arg->len;
                  strcpy(work, work+arg->len);
                  i = StrPos(arg->chars, work);
                  }
               }
            else if (IsDelim(work[i-1])) {
               if (work[arg->len+i] == 0 || IsDelim(work[arg->len+i])) {
                  arg->fld = f + i;
                  return true;
                  }
               else {
                  f = f + i + arg->len;
                  strcpy(work, work+i+arg->len);
                  i = StrPos(arg->chars, work);
                  }
               }
            else {
               f = f + i + arg->len;
               strcpy(work, work+i+arg->len);
               i = StrPos(arg->chars, work);
               }
            }
         }
      else {
         Pchar p = strstr(work, arg->chars);
         if (p) {
            arg->fld = short(p - work)+f;
            return true;
            }
         else
            arg->fld = 0;
         }
      }

   else if (arg->fld > 0) {
      // field match requested
      PCRecord val = new CRecord(chars);
      Assert(val);

      // compare the specified field in the record
      char test[maxSizeField];
      if (val->GetField(arg->fld, test, maxSizeField))
         if (strlen(test) == (unsigned)arg->len) {
            if (arg->ignCase)
               CapStr(test);
            if (strcmp(test, arg->chars) == 0)
               return true;
            }
      }

   return false;
   } // match

//=====================================================================
// String list methods
//=====================================================================

//---------------------------------------------------------------------
// Constructor
//
CStrList::CStrList(void):
   CList     (),
   iChar     (0),
   wrap      (false),
   descending(false),
   byLength  (false),
   ignCase   (true),
   fields    (0) {
   } // CStrList

//---------------------------------------------------------------------
// Copy Constructor
//
CStrList::CStrList(PCStrList _s):
   CList     (),
   iChar     (0),
   wrap      (false),
   descending(false),
   byLength  (false),
   ignCase   (true),
   fields    (0) {

   if (_s) {
      iChar = _s->iChar;
      wrap  = _s->wrap;

      if (_s->GotoHead()) do
         StrInsert(_s->StrItem(false));
      while (_s->GotoNext());
      }
   } // CStrList

//---------------------------------------------------------------------
//
CStrList::~CStrList() {
   delete[] fields;
   fields = 0;
   } // ~CStrList
   
//---------------------------------------------------------------------
//
RTTI_CPP(StrList, List)

//---------------------------------------------------------------------
//
ulong CStrList::ListSize(void) {
   ulong size = 1;

   if (GotoHead()) do
      size += StrSize() + strlen(CFile::GetDefaultDelim());
//?   size += StrSize()+1;
      while (GotoNext());

   return size;
   } // ListSize

//---------------------------------------------------------------------
// Insert the specified list into this list. (Note that, if sorted or
// unique is specified, the specified list will be inserted sorted.
// Later: this code inserts the list into the point following the point
// where the first item is inserted. This is probably not the most
// intuitive when the AtWhere = sorted.)
//
void CStrList::ListInsert(PCStrList _sl, AtWhere _where) {
   AtWhere where = _where;
   if (_sl->GotoHead()) do {
      StrInsert(_sl->StrItem(false), where);
      if (where == atTop ||
          where == before)
         // after first insertion make sure
         // list is inserted in sequence
         where = after;
      } while (_sl->GotoNext());
   } // ListInsert

//---------------------------------------------------------------------
//
void CStrList::SortSpec(bool  _descending,
                        bool  _byLength,
                        bool  _ignCase,
                        Pchar _fields) {
   descending = _descending;
   byLength   = _byLength;
   ignCase    = _ignCase;
   STRINIT(fields, _fields);
   } // SortSpec

//---------------------------------------------------------------------
// Remove blank lines from the top and end of list as specified
//
void CStrList::Trim(whichEnd _where) {
   if ((_where == frontEnd || _where == bothEnds) && GotoHead()) do
      if (StrEmpty(StrItem(false)))
         Remove(atTop);
      else
         break;
      while (true);

   if ((_where == bothEnds || _where == backEnd) && GotoTail()) do
      if (StrEmpty(StrItem(false)))
         Remove(atEnd);
      else
         break;
      while (true);
   } // Trim

//---------------------------------------------------------------------
// Encode the list from ASCII characters to entity string. Return true
// if any encodes took place.
//
//    &lt;    less-than
//    &gt;    greater-than
//    &amp;   ampersand
//    &apos;  apostrophe
//    &quot;  quote
//
bool CStrList::EncodeXML(void) {
   char work[sizeString];

   long iNr1   = itemNr;
   pCItem cr1  = cur;

   bool enc    = false;
   PCItem item = GetHeadItem();
   while (item) {
      PCStrItem str = TYPECAST(item, StrItem);
      Strcpy(work, str->chars, sizeString);
      if (::EncodeXML(work, sizeString)) {
         enc = true;
         str->SetString(work);
         }
      item = item->NextSibItem();
      }

   itemNr = iNr1;
   cur    = cr1;

   return enc;
   } // EncodeXML

//---------------------------------------------------------------------
// Decode from entity string to ASCII characters. Return true if any
// decodes took place.
//
//    &lt;    less-than
//    &gt;    greater-than
//    &amp;   ampersand
//    &apos;  apostrophe
//    &quot;  quote
//
//
bool CStrList::DecodeXML(void) {
   char work[sizeString];

   long iNr1   = itemNr;
   pCItem cr1  = cur;

   bool dec    = false;
   PCItem item = GetHeadItem();
   while (item) {
      PCStrItem str = TYPECAST(item, StrItem);
      Strcpy(work, str->chars, sizeString);
      if (::DecodeXML(work, sizeString)) {
         dec= true;
         str->SetString(work);
         }
      item = item->NextSibItem();
      }

   itemNr = iNr1;
   cur    = cr1;

   return dec;
   } // DecodeXML

//---------------------------------------------------------------------
// 2004/12/01 pjm Enhanced sort to support
//
//    * sort ascending or descending
//    * sort by length or by value
//    * ignore or not ignore case
//    * sort by multiple fields (ex., 351 = sort field 1 w/in field 5
//      w/in field 3; i.e., field 3 major, field 1 minor. Up to nine
//      fields, since each character identifies a field. Warning: don't
//      use zero - I don't check.
//
// Can combine options in any meaningful way.
// 
bool CStrList::smaller(Pchar _chars) {
   Pchar p = StrItem(false);

   if (descending) {
      if      (!StrEmpty(fields)) {
         PCRecord arg = new CRecord(_chars);
         Assert(arg);
         PCRecord itm = new CRecord(p);
         Assert(itm);
         int len      = strlen(fields);
         char warg[sizeString];
         char witm[sizeString];

         if      (byLength) {
            for (int i = 0; i < len; i++) {
               int ii = unsigned(fields[i]-'0');
               arg->GetField(ii, warg, sizeString);
               itm->GetField(ii, witm, sizeString);
               if (strlen(warg) != strlen(witm))
                  break;
               }
            return (strlen(warg) > strlen(witm));
            }

         else if (ignCase) {
            for (int i = 0; i < len; i++) {
               int ii = unsigned(fields[i]-'0');
               arg->GetField(ii, warg, sizeString);
               itm->GetField(ii, witm, sizeString);
               if (stricmp(warg, witm) != 0)
                  break;
               }
            return (stricmp(warg, witm) > 0);
            }

         else {
            for (int i = 0; i < len; i++) {
               int ii = unsigned(fields[i]-'0');
               arg->GetField(ii, warg, sizeString);
               itm->GetField(ii, witm, sizeString);
               if (strcmp(warg, witm) != 0)
                  break;
               }
            return (strcmp(warg, witm) > 0);
            }
         }

      else if (byLength)
         return (strlen(_chars) > strlen(p));

      else if (ignCase)
         return (stricmp(_chars, p) > 0);

      else
         return (strcmp (_chars, p) > 0);
      }
   else {                                                  // ascending
      if      (!StrEmpty(fields)) {
         PCRecord arg = new CRecord(_chars);
         Assert(arg);
         PCRecord itm = new CRecord(p);
         Assert(itm);
         int len      = strlen(fields);
         char warg[sizeString];
         char witm[sizeString];

         if      (byLength) {
            for (int i = 0; i < len; i++) {
               int ii = unsigned(fields[i]-'0');
               arg->GetField(ii, warg, sizeString);
               itm->GetField(ii, witm, sizeString);
               if (strlen(warg) != strlen(witm))
                  break;
               }
            return (strlen(warg) < strlen(witm));
            }

         else if (ignCase) {
            for (int i = 0; i < len; i++) {
               int ii = unsigned(fields[i]-'0');
               arg->GetField(ii, warg, sizeString);
               itm->GetField(ii, witm, sizeString);
               if (stricmp(warg, witm) != 0)
                  break;
               }
            return (stricmp(warg, witm) < 0);
            }

         else {
            for (int i = 0; i < len; i++) {
               int ii = unsigned(fields[i]-'0');
               arg->GetField(ii, warg, sizeString);
               itm->GetField(ii, witm, sizeString);
               if (strcmp(warg, witm) != 0)
                  break;
               }
            return (strcmp(warg, witm) < 0);
            }
         }

      else if (byLength)
         return (strlen(_chars) < strlen(p));

      else if (ignCase)
         return (stricmp(_chars, p) < 0);

      else
         return (strcmp (_chars, p) < 0);
      }
   } // smaller

//---------------------------------------------------------------------
//
bool CStrList::NextWord(Pchar _word) {
   _word[0] = '\0';

   // Point the beginning of the current word
   Pchar first = StrItem(false);
   if (*first == 0) {
      if (!GotoNext())
         return false;
      first = StrItem(false);

      if (*first == 0)
         return true;
      }

   Pchar last = first;

   // Skip leading spaces
   while (*last == ' ')          ++last;

   // Skip non-blank characters
   while (*last && *last != ' ') ++last;

   // Extract the word and return it
   strncpy(_word, first, last-first);
   _word[last-first] = '\0';
   return true;
   } // NextWord

//---------------------------------------------------------------------
//
short CStrList::WhereChar(void) {
   return charNr;
   } // WhereChar

//---------------------------------------------------------------------
// Construct a StrItem using the specified string and insert it into
// the list. If unique is specified and it is already in the list, then
// don't insert it.
//
bool CStrList::StrInsert(Pchar _p, AtWhere _where) {
   PCItem s = new CStrItem(_p);
   Assert(s);
   if (top && (_where == sorted || _where == unique)) {
      for (int i = 1; i <= Number(); i++) {
         GotoItem(i);
         if      (_where == unique && stricmp(_p, StrItem(false)) == 0)
            return true;
         else if (smaller(_p))
            return CList::Insert(s, before);
         }
      return CList::Insert(s, atEnd);
      }
   return CList::Insert(s, _where);
   } // StrInsert

//---------------------------------------------------------------------
// Construct a StrItem using the specified record and insert it into
// the list
//
bool CStrList::RecInsert(PCRecord _rec, AtWhere _where) {
   char rec[sizeString];
   _rec->GetDelimStrD(rec, sizeString);
   return StrInsert(rec, _where);
   } // RecInsert

//---------------------------------------------------------------------
// Return the address of the item contents
//
Pchar CStrList::StrItem(bool _alc) {
   if (!cur)
      return 0;

   PCItem item     = Item();
   PCStrItem pItem = TYPECAST(item, StrItem);
   if (pItem)
      return pItem->GetString(_alc);

   if (_alc) {
      Pchar p = new char[1];
      Assert(p);
      p[0]    = 0;
      return p;
      }

   return 0;
   } // StrItem

//---------------------------------------------------------------------
// farSlang version
//
Pchar CStrList::StrItem(void) {
   return StrItem(true);
   } // StrItem

//---------------------------------------------------------------------
// Return the size of the item contents
//
ulong CStrList::StrSize(void) {
   PCItem item     = Item();
   PCStrItem pItem = TYPECAST(item, StrItem);
   if (pItem)
      return strlen(pItem->GetString(false));
   else
      return 0;
   } // StrSize

//---------------------------------------------------------------------
// Copy the item contents to the specified char array
//
bool CStrList::StrItem(Pchar _chars, short _size) {
   Assert(_chars);
   memset(_chars, 0, _size);
   if (!cur)
      return false;

   PCItem item     = Item();
   PCStrItem pItem = TYPECAST(item, StrItem);
   if (pItem)
      strncpy(_chars, pItem->GetString(false), _size-1);
   return true;
   } // StrItem

//---------------------------------------------------------------------
// Copy the item contents to the specified string
//
bool CStrList::StrItem(RCString _str) {
   Pchar p = StrItem(false);
   _str.NewString(p);
   return true;
   } // StrItem

//---------------------------------------------------------------------
// Copy the contents of the specified item to the specified char array

//
bool CStrList::StrItem(long _where, Pchar _chars, short _size) {
   if (GotoItem(_where))
      return StrItem(_chars, _size);
   else
      return false;
   } // StrItem

//---------------------------------------------------------------------
// Return the address of the contents of the specified item
//
Pchar CStrList::StrItem(long _where, bool _alc) {
   if (GotoItem(_where))
      return StrItem(_alc);

   if (_alc) {
      Pchar p = new char[1];
      Assert(p);
      p[0]    = 0;
      return p;
      }

   return 0;
   } // StrItem

/* conflicts with StrItem(bool)
//---------------------------------------------------------------------
// farSlang version - return the address of the contents of the
// specified item
//
Pchar CStrList::StrItem(long _where) {
   return StrItem(_where, true);
   } // StrItem
*/

//---------------------------------------------------------------------
// Copy the contents of the specified item to the specified string
//
bool CStrList::StrItem(long _where, RCString _str) {
   if (GotoItem(_where)) {
      StrItem(_str);
      return true;
      }
   else
      return false;
   } // StrItem

//---------------------------------------------------------------------
//
PCRecord CStrList::RecItem(void) {
   PCRecord rec = new CRecord();
   Assert(rec);
   Pchar p = StrItem(false);
   if (p)
      rec->NewRecord(p);

   return rec;
   } // RecItem

//---------------------------------------------------------------------
//
PCRecord CStrList::RecItem(Pchar _delm) {
   Assert(!StrEmpty(_delm));
   PCRecord rec = new CRecord();
   Assert(rec);
   Pchar p = StrItem(false);
   if (p)
      rec->NewRecord(p, _delm);

   return rec;
   } // RecItem

//---------------------------------------------------------------------
//
bool CStrList::RecItem(PCRecord _rec) {
   Pchar p = StrItem(false);
   if (p) {
      _rec->NewRecord(p);
      return true;
      }
   else
      return false;
   } // RecItem

//---------------------------------------------------------------------
//
bool CStrList::RecItem(PCRecord _rec, Pchar _delm) {
   Assert(!StrEmpty(_delm));
   Pchar p = StrItem(false);
   if (p) {
      _rec->NewRecord(p, _delm);
      return true;
      }
   else
      return false;
   } // RecItem

//---------------------------------------------------------------------
//
bool CStrList::RecItem(long _where, PCRecord _rec) {
   if (GotoItem(_where)) {
      _rec->NewRecord(StrItem(false));
      return true;
      }
   else
      return false;
   } // RecItem

//---------------------------------------------------------------------
//
bool CStrList::RecItem(long _where, PCRecord _rec, Pchar _delm) {
   Assert(!StrEmpty(_delm));
   if (GotoItem(_where)) {
      _rec->NewRecord(StrItem(false), _delm);
      return true;
      }
   else
      return false;
   } // RecItem

//---------------------------------------------------------------------
// Replace the current item
//
bool CStrList::StrReplace(Pchar _chars) {
   if (cur) {
      SetDirty(true, shallow);
      pCStrItem(cur)->SetString(_chars);
      return true;
      }
   else
      return false;
   } // StrReplace

//---------------------------------------------------------------------
// Replace the current string item
//
bool CStrList::StrReplace(RCString _str) {
   return StrReplace(_str.Chars());
   } // StrReplace

//---------------------------------------------------------------------
// Replace the current record
//
bool CStrList::RecReplace(PCRecord _rec) {
   char rec[sizeString];
   _rec->GetDelimStrD(rec, sizeString);
   return StrReplace(rec);
   } // RecReplace

//---------------------------------------------------------------------
// Return true if the specified string appears within some item in the
// sublist from the current item to the end. Start at the beginning
// of the item.
//
bool CStrList::FindString(Pchar     _chars,
                          bool      _ignCase,
                          matchPart _part) {
   panArg p  = new anArg(argString, _ignCase, _part);
   p->chars  = new char[strlen(_chars)+1];
   strcpy(p->chars, _chars);
   if (_ignCase)
      CapStr(p->chars);
   p->len    = strlen(_chars);
   p->fld    = 0;
   iChar     = 0;
   bool rslt = FindItem(p);
   if (rslt)
      iChar = p->fld;
   delete[] p->chars;
   delete p;

   return rslt;
   } // FindString

//---------------------------------------------------------------------
// Return true if the specified string appears within some item in the
// sublist from the current item to the end.
//
bool CStrList::FindString(RCString  _str,
                          bool      _ignCase,
                          matchPart _part) {
   return FindString(_str.Chars(), _ignCase, _part);
   } // FindString

//---------------------------------------------------------------------
// Return true if the specified string appears within some item in the
// list. Start at the beginning of the first line in the list.
//
bool CStrList::FindFirstString(Pchar     _chars,
                               bool      _ignCase,
                               matchPart _part) {
   panArg p  = new anArg(argString, _ignCase, _part);
   p->chars  = new char[strlen(_chars)+1];
   strcpy(p->chars, _chars);
   if (_ignCase)
      CapStr(p->chars);
   p->len    = strlen(_chars);
   p->fld    = 0;
   iChar     = 0;
   bool rslt = FindFirst(p);
   if (rslt)
      iChar = p->fld;
   delete[] p->chars;
   delete p;

   return rslt;
   } // FindFirstString

//---------------------------------------------------------------------
// Return true if the specified string appears within some item in the
// list. Start at the beginning of the first line in the list.
//
bool CStrList::FindFirstString(RCString  _str,
                               bool      _ignCase,
                               matchPart _part) {
   return FindFirstString(_str.Chars(), _ignCase, _part);
   } // FindFirstString

//---------------------------------------------------------------------
// Return true if the specified string appears within some item in the
// list. Starts at the beginning of the next line.
//
bool CStrList::FindNextString(Pchar     _chars,
                              bool      _ignCase,
                              matchPart _part) {
   panArg p  = new anArg(argString, _ignCase, _part);
   p->chars  = new char[strlen(_chars)+1];
   strcpy(p->chars, _chars);
   if (_ignCase)
      CapStr(p->chars);
   p->len    = strlen(_chars);
   p->fld    = 0;
   bool rslt = FindNext(p);
   if (rslt)
      iChar = p->fld;
   delete[] p->chars;
   delete p;

   return rslt;
   } // FindNextString

//---------------------------------------------------------------------
// Return true if the specified string appears within some item in the
// list. Starts at the the beginning of the next line.
//
bool CStrList::FindNextString(RCString  _str,
                              bool      _ignCase,
                              matchPart _part) {
   return FindNextString(_str.Chars(), _ignCase, _part);
   } // FindNextString

//---------------------------------------------------------------------
// Return true if specified string appears within some item in the
// list. Starts at current item at the offset specified by iChar+1.
//
bool CStrList::FindNextSubString(Pchar     _chars,
                                 bool      _ignCase,
                                 matchPart _part) {
   panArg p  = new anArg(argString, _ignCase, _part);
   p->chars  = new char[strlen(_chars)+1];
   Assert(p->chars);
   strcpy(p->chars, _chars);
   if (_ignCase)
      CapStr(p->chars);
   p->len    = strlen(_chars);
   p->fld    = iChar;
   bool rslt = FindItem(p);
   if (rslt)
      iChar = p->fld;
   delete[] p->chars;
   delete p;

   return rslt;
   } // FindNextString

//---------------------------------------------------------------------
// Return true if the specified string appears
// within some item in the list. Starts at the
// top of the list.
//
bool CStrList::FindNextSubString(RCString  _str,
                                 bool      _ignCase,
                                 matchPart _part) {
   return FindNextSubString(_str.Chars(), _ignCase, _part);
   } // FindNextString

//---------------------------------------------------------------------
// Return true if specified field of a record in the list
// equals the specified chars.
//
bool CStrList::FindRecord(short _iField,
                          Pchar _chars,
                          bool  _ignCase) {
   bool rslt = false;
   panArg p  = new anArg(argRecord, _ignCase, wholeString);

   if (p) {
      p->chars = new char[strlen(_chars)+1];
      if (p->chars) {
         strcpy(p->chars, _chars);
         if (_ignCase)
            CapStr(p->chars);
         p->len = strlen(_chars);
         p->fld = _iField;
         rslt   = FindItem(p);
         delete[] p->chars;
         }
      delete p;
      }
   return rslt;
   } // FindRecord

//---------------------------------------------------------------------
// Return true if specified field of a record in the list
// equals the specified chars.
//
bool CStrList::FindFirstRecord(short _iField,
                               Pchar _chars,
                               bool  _ignCase) {
   bool rslt = false;
   panArg p  = new anArg(argRecord, _ignCase, wholeString);

   if (p) {
      p->chars = new char[strlen(_chars)+1];
      if (p->chars) {
         strcpy(p->chars, _chars);
         if (_ignCase)
            CapStr(p->chars);
         p->len = strlen(_chars);
         p->fld = _iField;
         rslt   = FindFirst(p);
         delete[] p->chars;
         }
      delete p;
      }
   return rslt;
   } // FindFirstRecord

//---------------------------------------------------------------------
// Return true if specified field of a record in the list
// equals the specified chars.
//
bool CStrList::FindNextRecord(short _iField,
                              Pchar _chars,
                              bool  _ignCase) {
   bool rslt = false;
   panArg p  = new anArg(argRecord, _ignCase, wholeString);

   if (p) {
      p->chars = new char[strlen(_chars)+1];
      if (p->chars) {
         strcpy(p->chars, _chars);
         if (_ignCase)
            CapStr(p->chars);
         p->len = strlen(_chars);
         p->fld = _iField;
         rslt   = FindNext(p);
         delete[] p->chars;
         }
      delete p;
      }
   return rslt;
   } // FindNextRecord

/*?
//---------------------------------------------------------------------
// Count the number of characters in the range starting at line _from
// and ending at the line before _to
//
ulong CStrList::Span(ulong _from, ulong _to) {
   if (!top)
      return 0;
   if (_from > nrItems)
      return 0;
   if (_to   > nrItems)
      return 0;
   if (from > _to)
      return 0;

   ulong inr = itemNr;
   pItem cr  = cur;

   ulong nrChars = 0;
   if (GotoItem(_from)) do {
      if (itemNr >= _to)
         break;
      nrChars += strlen(cur->chars);
      }


   itemNr = inr;
   cur    = cr;

   } // Span
?*/

//---------------------------------------------------------------------
// Parse the expression in _name
//
// The form of _name is:
//
//    .listName.modifier1.modifier2...modifierN
//
// Modifiers are processed left-to-right to recursively sublist the
// specified string list.
//
// Modifiers are:
//
//    head       the first line of the sublist
//    tail       all the lines of the sublist excluding the first line
//    ,h,t       all the lines in the sublist beginning at line h and
//               ending at line t.
//    searchArg  the record sublist beginning with a record matching
//               searchArg and extending (exclusively) to the next record
//               not matching searchArg in the corresponding field.
//    n          the nth field of the current (single) record. Is
//               recursive: Use .n again if you want a field within a
//               field; e.g., ...3.5.2 means the second field of the
//               fifth field of the third field of the line found for
//               the "..."
//
bool CStrList::SubList(Pchar _name, long& _head, long& _tail,
                       Pchar _chars, short _size) {
   char chars[sizeString];

   // _name has the form: listName.mods
   // (ignore the listName)
   PCRecord r1 = new CRecord(_name);
   Assert(r1);
   char work[sizeString];
   GotoHead();
   Strcpy(chars, StrItem(false), sizeString);

   // process modifiers (.head, .tail, .#, .$h$t, .searchArg)
   _head      = 1;
   _tail      = Number();
   short test = 1;
   for (short i = 2; i <= r1->Number(); ++i) {
      r1->GetField(i, work, sizeString);
      CapStr(work);
      if      (strcmp(work, Pchar("HEAD")) == 0) {
         // modifier is head of list
         if (_tail > _head) {
            _tail = _head;
            GotoItem(_head);
            Strcpy(chars, StrItem(false), sizeString);
            }
         else
            memset(chars, 0, sizeString);
         test = 1;
         }
      else if (strcmp(work, Pchar("TAIL")) == 0) {
         // modifier is tail of list
         if (_tail > _head) {
            ++_head;
            GotoItem(_head);
            Strcpy(chars, StrItem(false), sizeString);
            }
         else
            memset(chars, 0, sizeString);
         test = 1;
         }
      else if (IsDelim(work[0])) {
         // modifier is ,h,t sublist
         PCRecord subs = new CRecord(work);
         Assert(subs);
         long h        = subs->GetIntField(1);
         if (h) {
            _head += h - 1;
            GotoItem(_head);
            Strcpy(chars, StrItem(false), sizeString);
            }
         long t = subs->GetIntField(2);
         if (t)
            _tail += t - 1;
         test = 1;
         }
      else if (IsNumber(work)) {
         // modifier is field selector
         if (_tail >= _head) {
            short field = StrToShort(work);
            _tail       = _head;
            PCRecord r3 = new CRecord(chars);
            Assert(r3);
            r3->GetField(field, chars, sizeString);
            }
         test = -1;
         }
      else if (FindRecord(test, work) && Where() <= _tail) {
         // modifier is field chars
         _head = Where();
         Strcpy(chars, StrItem(false), sizeString);

         // find the tail for the sublist
         PCRecord r4 = new CRecord(StrItem(false));
         Assert(r4);
         int j;
         for (j = _head+1; j <= _tail; ++j) {
            GotoItem(j);
            PCRecord r5 = new CRecord(StrItem(false));
            Assert(r5);
            short k;
            for (k = 1; k <= test; ++k) {
               char arg1[sizeString];
               r4->GetField(k, arg1, sizeString);
               CapStr(arg1);
               char arg2[sizeString];
               r5->GetField(k, arg2, sizeString);
               CapStr(arg2);
               if (strcmp(arg1, arg2))
                  if (StrEmpty(arg2))
                     continue;
                  else {
                     _tail = j - 1;
                     j = -1;
                     break;
                     }
               }
            if (j < 0)
               break;
            }
         ++test;
         }
      else {
         // sublist search failed
         GotoItem(_head);
         memset(_chars, 0, sizeString);
         return false;
         }
      GotoItem(_head);
      }

   // square _head and _tail with the size of the list
   if (_head > Number())
       _head = Number();
   if (_tail > Number())
       _tail = Number();

   Strcpy(_chars, chars, _size);
   return true;
   } // SubList

//---------------------------------------------------------------------
// Given a line number and a character number, compute the character
// offset from the top of the list. If the specified line number is
// greater than the number of lines in the list, return the total
// number of characters in the list. The result is both returned and
// stored in iChar.
//
ulong CStrList::CharOffset(ulong _line, ulong _col)  {
   iChar = 0;

   if (GotoHead()) do {
      if      (itemNr == _line) {
         iChar += Min(_col, StrSize());
         break;
         }
      else if (itemNr > _line)
         break;
      iChar += StrSize();
      } while (GotoNext());

   return iChar;
   } // CharOffset

//---------------------------------------------------------------------
// Given a character offset, compute the line offset from the top of
// the list. Leave the remainder in iChar.
//
ulong CStrList::LineOffset(ulong _iChar) {
   ulong iLine = 0;

   if (GotoHead()) do {
      iLine++;
      if (_iChar <= StrSize()) {
         iChar = _iChar;
         break;
         }
      _iChar -= StrSize();
      } while (GotoNext());

   return iLine;
   } // LineOffset

//---------------------------------------------------------------------
// Append the contents of the block to the end of the list. Layout of
// block is a collection of char arrays separated either by a zero-byte
// or by \r\n line terminators
//
bool CStrList::BlockTo(PCBlock _block) {
   if (!(_block->Isa(iBlock)))
      return false;

//?   PCBlock block = TYPECAST(_block, Block);

   // Transfer the array of char arrays in the block to the string list
   ulong size = _block->GetSize();
   if (size == 0)
      return true;

   Pchar p    = _block->Lock();
   char work[sizeString];
   memset(work, 0, sizeString);
   ulong w    = 0;
   for (ulong i = 0; i >= 0; i++)
      if      (i >= size) {
         StrInsert(work);
         break;
         }
      else if (p[i] == 0) {
         StrInsert(work);
         work[0] = 0;
         w       = 0;
         }
      else if (p[i] == '\n') {
         if (p[i+1] == '\r')
            i++;
         StrInsert(work);
         work[0] = 0;
         w       = 0;
         }
      else if (p[i] == '\r') {
         if (p[i+1] == '\n')
            i++;
         StrInsert(work);
         work[0] = 0;
         w       = 0;
         }
      else if (w >= sizeString-2) { // ..Line && p[i] == ' ') {
         StrInsert(work);
         work[0] = 0;
         w       = 0;
         }
      else {
         ChrCat(work, p[i], sizeString);
         w++;
         }

   // Remove trailing empty lines
   while (GotoTail() && StrEmpty(StrItem(false)))
      Remove();
   GotoHead();

   _block->Unlock();
   return true;
   } // BlockTo

//---------------------------------------------------------------------
// Copy the contents of the string list into the block. Layout of block
// is array of char arrays, each terminated by \n.
//
bool CStrList::ToBlock(PCBlock _block) {
   if (!(_block->Isa(iBlock)))
      return false;

//?   PCBlock block = TYPECAST(_block, Block);

   // Get the amount of space needed
   ulong size = Max(ListSize(), 1) + 1;

   // Allocate space to hold the text
   // (Rounding up to nearest 8 bytes for encryption)
   _block->Alloc(size, 8);
   Pchar p = _block->Lock();
   _block->Clear(0);

   // Transfer the text lines into the block
   if (GotoHead()) do {
      Pchar q = StrItem(false);
      strcpy(p, q);
      p += strlen(q);

      strcpy(p, CFile::GetDefaultDelim());
      p += strlen(CFile::GetDefaultDelim());
      Assert(p - _block->Pointer() <= size);
      } while (GotoNext());

   _block->Unlock();
   return true;
   } // ToBlock

//---------------------------------------------------------------------
//
bool CStrList::readItem(PCAtom _input, bool _isBook) {
   PCStrItem p = new CStrItem(Pchar(""));
   Assert(p);

   if (!(p->ReadFile(_input, _isBook))) {
      err = p->Error();
      return false;
      }

   PCItem it = TYPECAST(p, Item);
   Insert(it, after);
   return true;
   } // readItem

//---------------------------------------------------------------------
//
bool CStrList::writeItem(PCAtom _output, bool _isBook) {
   PCItem it = Item();
   PCStrItem item = TYPECAST(it, StrItem);
   if (!item->WriteFile(_output, _isBook)) {
      err = _output->Error();
      return false;
      }

   return true;
   } // writeItem


