
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// note: _isBook parameter never used

//*********************************************************************
// strlist.h
//*********************************************************************

#ifndef STRLIST_H
#define STRLIST_H

#ifndef FARBOOK_H
#include "block.h"
#include "list.h"
#include "record.h"
#include "strng.h"
#endif

//=====================================================================
// String item class definition
//
_FBDEF(CStrItem)
class FBC_USE CStrItem: public CItem {
   friend class CStrList;
   friend class CIniList;

   public:
      FB_USE CStrItem(void);
      FB_USE CStrItem(Pchar _s);
#ifndef _SLANGER
      FB_USE CStrItem(RCString  _s);
#endif // _SLANGER
      FB_USE CStrItem(PCStrItem _p);
      FB_USE CStrItem(pCStrItem _p);
      virtual FB_USE ~CStrItem ();

      virtual FB_USE_(short) Ima(void);
      virtual FB_USE_(Pchar) IAm(void);
      virtual FB_USE_(bool)  Isa(const short _type);

      virtual FB_USE_(PCStrItem) CopyItem(void);

      FB_USE_(ushort) Size     (void) { return ushort(strlen(chars)); }

      FB_USE_(void)   SetString(Pchar _chars);
#ifndef _SLANGER
      FB_USE_(Pchar)  GetString(bool _alc);
#endif // _SLANGER
      FB_USE_(Pchar)  GetString(void);              // farSlang version

      // Conditional operators
      virtual FB_USE_(bool) operator ==(PCItem _item);
      virtual FB_USE_(bool) operator !=(PCItem _item);

      // IO methods
      virtual FB_USE_(ulong) BookSize (void);
      virtual FB_USE_(bool)  ReadFile (PCAtom _input, bool _isBook);
      virtual FB_USE_(bool)  WriteFile(PCAtom _output, bool _isBook);

      virtual FB_USE_(bool)  Load     (Pchar _fileName);
      virtual FB_USE_(bool)  Save     (Pchar _fileName);

   protected:
      virtual bool match(Pvoid _p);

      Pchar chars;
   }; // CStrItem

//=====================================================================
// String list class definition
//
enum matchPart {anyPart, wholeWord, wholeString};
enum order     {ascending, descending};

_FBDEF(CStrList)
class FBC_USE CStrList: public CList {
   public:
      FB_USE CStrList(void);
      FB_USE CStrList(PCStrList _sl);
      virtual FB_USE ~CStrList();

      virtual FB_USE_(short) Ima(void);
      virtual FB_USE_(Pchar) IAm(void);
      virtual FB_USE_(bool)  Isa(const short _type);

      FB_USE_(ulong) ListSize      (void);

      FB_USE_(void)  ListInsert    (PCStrList _sl, AtWhere _where = atEnd);

      FB_USE_(void)  SortSpec      (bool  _descending,
                                    bool  _byLength,
                                    bool  _ignCase,
                                    Pchar _fields);

      FB_USE_(void)  Trim          (whichEnd _where);
      
      // XML support methods
      FB_USE_(bool)  EncodeXML     (void);
      FB_USE_(bool)  DecodeXML     (void);

      // Single word parser methods
      FB_USE_(bool)  NextWord      (Pchar _word);
      FB_USE_(short) WhereChar     (void);

      // Single item manipulator methods
      FB_USE_(bool)  StrInsert     (Pchar    _chars, AtWhere _where = atEnd);
#ifndef _SLANGER
      FB_USE_(bool)  StrInsert     (RCString _str,   AtWhere _where = atEnd);
#endif // _SLANGER
      FB_USE_(bool)  RecInsert     (PCRecord _rec,  AtWhere _where = atEnd);

      FB_USE_(ulong) StrSize       (void);

#ifndef _SLANGER
      FB_USE_(bool)  StrItem       (Pchar _chars, short _size);
      FB_USE_(Pchar) StrItem       (bool _alc);
      FB_USE_(bool)  StrItem       (RCString _str);
#endif // _SLANGER
      FB_USE_(Pchar) StrItem       (void);          // farSlang version

#ifndef _SLANGER
      FB_USE_(bool)  StrItem       (long _where, Pchar _chars, short _size);
      FB_USE_(Pchar) StrItem       (long _where, bool _alc);
      FB_USE_(bool)  StrItem       (long _where, RCString _str);
#endif // _SLANGER
//?   FB_USE_(Pchar) StrItem       (long _where); conflicts w/ StrItem(bool)

      FB_USE_(PCRecord) RecItem    (void);
      FB_USE_(PCRecord) RecItem    (Pchar _delm);
      FB_USE_(bool)  RecItem       (PCRecord _rec);
      FB_USE_(bool)  RecItem       (PCRecord _rec, Pchar _delm);
      FB_USE_(bool)  RecItem       (long _where, PCRecord _rec);
      FB_USE_(bool)  RecItem       (long _where, PCRecord _rec, Pchar _delim);

      FB_USE_(bool)  StrReplace    (Pchar     _chars);
#ifndef _SLANGER
      FB_USE_(bool)  StrReplace    (RCString  _str);
#endif // _SLANGER
      FB_USE_(bool)  RecReplace    (PCRecord _rec);

      // Search methods
      FB_USE_(bool) FindString     (Pchar     _chars,
                                    bool      _ignCase = true,
                                    matchPart _part    = anyPart);
#ifndef _SLANGER
      FB_USE_(bool) FindString     (RCString  _str,
                                    bool      _ignCase = true,
                                    matchPart _part    = anyPart);
#endif // _SLANGER

      FB_USE_(bool) FindFirstString(Pchar     _chars,
                                    bool      _ignCase = true,
                                    matchPart _part    = anyPart);
#ifndef _SLANGER
      FB_USE_(bool) FindFirstString(RCString  _str,
                                    bool      _ignCase = true,
                                    matchPart _part    = anyPart);
#endif // _SLANGER

      FB_USE_(bool) FindNextString (Pchar     _chars,
                                    bool      _ignCase = true,
                                    matchPart _part    = anyPart);
#ifndef _SLANGER
      FB_USE_(bool) FindNextString (RCString  _str,
                                    bool      _ignCase = true,
                                    matchPart _part    = anyPart);
#endif // _SLANGER

      FB_USE_(bool) FindNextSubString
                                   (Pchar     _chars,
                                    bool      _ignCase = true,
                                    matchPart _part    = anyPart);
#ifndef _SLANGER
      FB_USE_(bool) FindNextSubString
                                   (RCString  _str,
                                    bool      _ignCase = true,
                                    matchPart _part    = anyPart);
#endif // _SLANGER

      FB_USE_(bool) FindRecord     (short _iField,
                                    Pchar _chars,
                                    bool  _ignCase = true);
      FB_USE_(bool) FindFirstRecord(short _iField,
                                    Pchar _chars,
                                    bool  _ignCase = true);
      FB_USE_(bool) FindNextRecord (short _iField,
                                    Pchar _chars,
                                    bool  _ignCase = true);

      FB_USE_(bool) SubList        (Pchar _name,
                                    Rlong _head,
                                    Rlong _tail,
                                    Pchar _chars,
                                    short _size);
      virtual FB_USE_(bool) smaller(Pchar _chars);

      FB_USE_(ulong) IChar(void) { return iChar; }
      FB_USE_(void)  IChar(ulong _iChar) { iChar = _iChar; }

      FB_USE_(ulong) CharOffset(ulong _line, ulong _col);
      FB_USE_(ulong) LineOffset(ulong _ichar);

      FB_USE_(bool) Wrap(void) { return wrap; }
      FB_USE_(void) Wrap(bool _wrap) { wrap = _wrap; }

      // Conversion methods
      FB_USE_(bool) BlockTo(PCBlock _block);
      FB_USE_(bool) ToBlock(PCBlock _block);

   protected:
      // Per item read method
      virtual bool readItem (PCAtom _input, bool _isBook);
      virtual bool writeItem(PCAtom _input, bool _isBook);

      ulong iChar;
      bool  wrap;
      bool  descending;
      bool  byLength;
      bool  ignCase;
      Pchar fields;
   }; // CStrList

#endif // STRLIST_H

