
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


//*********************************************************************
// strng.h
//*********************************************************************

#ifndef STRNG_H
#define STRNG_H

#ifndef FARBOOK_H
#include "common.h"
#endif

#ifndef _SLANGER

_FBDEF(CString)
class FBC_USE CString: public CAtom {
   public:
      FB_USE CString(void);                                  // default
      FB_USE CString(char _fill, int _size);                  // filled
      FB_USE CString(const Pchar _chars);                 // conversion
      FB_USE CString(const long _nr, int _radix = 10);     //conversion
      FB_USE CString(const PCString _string);                  // copy
      virtual FB_USE ~CString();                          // destructor

      virtual FB_USE_(short) Ima(void);
      virtual FB_USE_(Pchar) IAm(void);
      virtual FB_USE_(bool)  Isa(const short _type);

      FB_USE_(void)  NewString(const Pchar _chars);

      FB_USE_(Pchar) Chars(void) { return chars; } // rtrn ptr to chars

#ifndef _SLANGER
      const FB_USE_(RCString) operator =(const RCString _string);

      // concatenate
      FB_USE_(CString) operator +=(const RCString _string);
      friend
      FB_USE_(CString) operator + (const RCString _strA,
                                   const RCString _strB);

      FB_USE_(bool) operator  !(void) const;          // test for empty
      FB_USE_(bool) operator ==(const RCString _string);
      FB_USE_(bool) operator !=(const RCString _string);
      FB_USE_(bool) operator  <(const RCString _string);
      FB_USE_(bool) operator  >(const RCString _string);
      FB_USE_(bool) operator <=(const RCString _string);
      FB_USE_(bool) operator >=(const RCString _string);

      FB_USE_(Rchar)   operator[](short _index);     // return char ref
      FB_USE_(CString) operator()(short _begin, short _end);
#endif // _SLANGER

      FB_USE_(bool)    IsEqual(PCString _strB);

      FB_USE_(bool)    IsEmpty(void) { return size == 0; }

      FB_USE_(PCString) Insert(const PCString _string, short _where);
      FB_USE_(PCString) Delete(short _where, short _size);

      FB_USE_(PCString) Before(const PCString _pattern);
                                     // Return substring before pattern
      FB_USE_(PCString) After (const PCString _pattern);
                                      // Return substring after pattern

      FB_USE_(short)   Size  (void) const;      // return string length

      // Return position of pattern
      FB_USE_(short) Position(const PCString _pattern);

      FB_USE_(void)  Format  (Pchar _format, ...);

      // Miscellaneous
      FB_USE_(void)  Cap     (void);
      FB_USE_(short) Squeeze (char _ch);
      FB_USE_(short) Pack    (void);
      FB_USE_(short) Trim    (whichEnd _end = bothEnds);
      FB_USE_(short) Pad     (short _sz);

      // Numeric conversions
      FB_USE_(bool)  IsNumber(void);         // return true if a number
      FB_USE_(long)  Long    (void);               // return long value
      FB_USE_(short) Short   (void);              // return short value

      virtual FB_USE_(bool)  BlockTo  (PCAtom _block);
      virtual FB_USE_(bool)  ToBlock  (PCAtom _block);

      // Input/output
      virtual FB_USE_(ulong) BookSize (void);
      virtual FB_USE_(bool)  ReadFile (PCAtom _file, bool _isBook);
      virtual FB_USE_(bool)  WriteFile(PCAtom _file, bool _isBook);

      virtual FB_USE_(bool)  Load     (Pchar _fileName);
      virtual FB_USE_(bool)  Save     (Pchar _fileName);

   private:
      Pchar chars;                             // pointer to char array
      int   size;                                 // size of char array
   }; // CString

#endif // _SLANGER

#endif // STRNG_H

