
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Paul Medlock.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Paul Medlock, <paul@medlock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


//*********************************************************************
// syslog.h
//*********************************************************************

#ifndef SYSLOG_H
#define SYSLOG_H

#ifndef FARBOOK_H
#include "common.h"
#include "strlist.h"
#endif

//=====================================================================
// Low-level log file class
//
__FBDEF(SysLog)
class FBC_USE SysLog {
   public:
      static FB_USE_(pSysLog) Logger(void) { return logger; }
      static FB_USE_(bool) Enter(void);
      static FB_USE_(void) Leave(void);
      static FB_USE_(void) Log  (char _flag, Pchar _format, ...);

      FB_USE SysLog            (void);
      FB_USE SysLog            (const Pchar _name,
                                const Pchar _path = 0);
      virtual FB_USE ~SysLog   ();

      FB_USE_(ulong) Error     (void);             // Return last error
      FB_USE_(void)  Error     (ulong _error);

      FB_USE_(bool) Create     (void);          // create a closed file
      FB_USE_(bool) OpenOutput (void);
      FB_USE_(bool) OpenAppend (void);  // append to file or create one
      FB_USE_(bool) Close      (void);                    // Close file

      FB_USE_(bool) Seek       (void);

      FB_USE_(bool) Printf     (Pchar _format, ...);// format and write
      FB_USE_(bool) WriteString(Pchar _p);    // Write a string and eol

   protected:
      bool write      (Pvoid _buffer, ulong _size);

      static char     eolDelim[];      // default end-of-line delimiter
      static pSysLog logger;

      // Critical section to restrict access to the log file
      class loCS {
         public:
            loCS(void);
            ~loCS();

            void Enter(void);
            void Leave(void);

         protected:

      #if   defined(FV_MSWIN16)
         #warning Critical Section not supported

      #elif defined(FV_MSWIN32)
            CRITICAL_SECTION m_cs;

      #elif defined(FV_MACINTOSH)
         #warning FV_MACINTOSH TBD

      #elif defined(FV_UNIX)

      #endif // FV_UNIX
         }; // loCS

      loCS      cs;
      Pchar     name;                              // full name of file
      bool      fileExists;
      bool      fileIsOpen;
      bool      fileCanWrite;
      FB_HANDLE file;
      PCStrList list;
      char      delim[sizeName];           // current end-of-line delim
      long      whereAt;                    // position within the file
      ulong     sysErr;                       // operating system error
      ulong     error;
      char      ch;
   }; // SysLog

#endif // SYSLOG_H

